@@ -17,6 +17,7 @@ Test signing service.
 
 import base64
 import json
+import socket
 import subprocess
 from pathlib import Path
 from typing import Any
@@ -27,7 +28,6 @@ import yaml
 
 from debusine.artifacts.models import ArtifactCategory
 from debusine.assets.models import SigningKeyData
-from debusine.client.models import FileResponse
 from utils.client import Client
 from utils.common import Configuration
 from utils.integration_test_helpers_mixin import IntegrationTestHelpersMixin
@@ -84,7 +84,7 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
         cls,
         signing_key_data: SigningKeyData,
         image_data: bytes,
-        signed_file: FileResponse,
+        signed_file: dict[str, Any],
     ) -> None:
         """Verify a signed file against its UEFI signing key."""
         with cls._temporary_directory() as temp_path:
@@ -119,6 +119,7 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
         )
         self.assertEqual(result.returncode, 0)
         work_request_id = yaml.safe_load(result.stdout)["work_request_id"]
+        assert isinstance(work_request_id, int)
         return work_request_id
 
     def grant_sign_permission(self, asset_id: int) -> None:
@@ -139,7 +140,9 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
         result = Client.execute_command(
             "list-assets", "--work-request", str(work_request_id)
         )
-        return yaml.safe_load(result.stdout)[0]
+        assets = yaml.safe_load(result.stdout)[0]
+        assert isinstance(assets, dict)
+        return assets
 
     def grub_signed_package_name(self) -> str | None:
         """Return the name of a signed grub-efi binary package."""
@@ -208,6 +211,7 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
         )
         self.assertEqual(result.returncode, 0)
         work_request_id = yaml.safe_load(result.stdout)["work_request_id"]
+        assert isinstance(work_request_id, int)
         return work_request_id
 
     def wait_and_assert_result(
@@ -217,8 +221,7 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
         status = Client.wait_for_work_request_completed(work_request_id, result)
         if not status:
             self.print_work_request_debug_logs(work_request_id)
-        if status == "success":
-            self.assertTrue(status)
+        self.assertTrue(status)
 
     def assert_successful_sign(
         self, work_request_id: int, signing_key_data: SigningKeyData
@@ -260,11 +263,13 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
                 self.signing_input_files[name],
                 signing_output_artifact["files"][f"{name}.sig"],
             )
-        return signing_output_artifact["id"]
+        signing_output_artifact_id = signing_output_artifact["id"]
+        assert isinstance(signing_output_artifact_id, int)
+        return signing_output_artifact_id
 
     def assemble_signed_source(
         self, template_package_name: str, signing_output_artifact_id: int
-    ) -> None:
+    ) -> int:
         """Assemble a signed source package, return the task."""
         # We need an environment for this.
         self.create_system_images_mmdebstrap(["bookworm"])
@@ -284,6 +289,7 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
                 }
             ),
         )["work_request_id"]
+        assert isinstance(work_request_id, int)
         return work_request_id
 
     def assert_successful_assembly(
@@ -366,3 +372,36 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
         self.assert_successful_assembly(
             assemble_signed_source_task_id, signed_package_name
         )
+
+    def test_sign_repository_indexes(self) -> None:
+        """Generate and sign indexes for a package repository."""
+        suite_name = "signing-experiment"
+        self.create_suite(
+            suite_name, {"components": ["main"], "architectures": ["amd64"]}
+        )
+        self.add_suite_to_archive(suite_name)
+
+        workflow_template_result = Client.execute_command(
+            "create-workflow-template",
+            "update-suites",
+            "update_suites",
+            stdin=yaml.safe_dump({}),
+        )
+        assert workflow_template_result.returncode == 0
+
+        workflow_id = Client.execute_command(
+            "create-workflow",
+            "update-suites",
+            stdin=yaml.safe_dump({"force_basic_indexes": True}),
+        )["workflow_id"]
+        assert Client.wait_for_work_request_completed(workflow_id, "success")
+
+        # Check that APT is happy with the resulting archive.
+        archive_url = f"http://deb.{socket.getfqdn()}/debusine/System"
+        with self.apt_indexes(
+            suite_name,
+            url=archive_url,
+            signed_by=self.get_archive_signing_key(),
+            architectures=["amd64"],
+        ):
+            pass
