@@ -18,6 +18,9 @@ function copy_logs ()
 	journalctl -u debusine-server-provisioner.service > "$DESTINATION/debusine-server-provisioner.log"
 	journalctl -u debusine-server-scheduler.service > "$DESTINATION/debusine-server-scheduler.log"
 
+	echo "Copying list of work requests to $DESTINATION/list-work-requests.yaml"
+	debusine list-work-requests > "$DESTINATION/list-work-requests.yaml"
+
 	WR_DESTINATION="$DESTINATION/work-request"
 	mkdir "$WR_DESTINATION"
 	echo "Copying most recent work request's debusine:work-request-debug-logs to $WR_DESTINATION"
