@@ -9,7 +9,7 @@ set -e
 debusine_worker_config_directory=/etc/debusine/worker
 
 sed "s,api-url = .*,api-url = https://$(hostname -f)/api," \
-	/usr/share/doc/debusine-worker/examples/config.ini \
+	examples/worker/config.ini \
 	>"$debusine_worker_config_directory/config.ini"
 chown debusine-worker:debusine-worker "$debusine_worker_config_directory"
 
