@@ -23,9 +23,10 @@ import yaml
 from debian.deb822 import Deb822
 
 from debusine.artifacts.local_artifact import BinaryPackage, SourcePackage
-from debusine.artifacts.models import ArtifactCategory
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.test import TestCase
 from utils.client import Client
+from utils.server import DebusineServer
 
 
 class IntegrationTestHelpersMixin(TestCase):
@@ -50,8 +51,11 @@ class IntegrationTestHelpersMixin(TestCase):
         cls,
         suite_name: str,
         url: str = "https://deb.debian.org/debian",
-        signed_by: str = "/usr/share/keyrings/debian-archive-keyring.gpg",
+        signed_by: (
+            str | None
+        ) = "/usr/share/keyrings/debian-archive-keyring.gpg",
         architectures: list[str] | None = None,
+        trusted: bool = False,
     ) -> Generator[Path, None, None]:
         """Fetch APT indexes for a suite."""
         if architectures is None:
@@ -75,19 +79,22 @@ class IntegrationTestHelpersMixin(TestCase):
                     """
                 )
             )
-            if "\n" in signed_by:
-                # A multi-line Signed-By is an embedded public key.
-                signed_by = "\n" + "\n".join(
-                    f" {line}" if line else " ."
-                    for line in signed_by.splitlines()
-                )
             source = {
                 "Types": "deb deb-src",
                 "URIs": url,
                 "Suites": suite_name,
                 "Components": "main",
-                "Signed-By": signed_by,
             }
+            if signed_by is not None:
+                if "\n" in signed_by:
+                    # A multi-line Signed-By is an embedded public key.
+                    signed_by = "\n" + "\n".join(
+                        f" {line}" if line else " ."
+                        for line in signed_by.splitlines()
+                    )
+                source["Signed-By"] = signed_by
+            if trusted:
+                source["Trusted"] = "yes"
             (apt_path / "etc/apt/sources.list.d/test.sources").write_text(
                 str(Deb822(source))
             )
@@ -163,7 +170,7 @@ class IntegrationTestHelpersMixin(TestCase):
                 [dsc_file] = [
                     file for file in files if file.name.endswith(".dsc")
                 ]
-                version = dsc_file.name.split("_")[1]
+                version = dsc_file.name.removesuffix(".dsc").split("_")[1]
                 src_artifact = SourcePackage.create(
                     name=package_name, version=version, files=files
                 )
@@ -334,3 +341,72 @@ class IntegrationTestHelpersMixin(TestCase):
         assert Client.wait_for_work_request_completed(
             workflow_id, "success", timeout=1800 * len(suites)
         )
+
+    def create_suite(
+        self, suite_name: str, data: dict[str, Any] | None = None
+    ) -> None:
+        """Create a suite collection."""
+        result = DebusineServer.execute_command(
+            "create_collection",
+            suite_name,
+            CollectionCategory.SUITE,
+            stdin=yaml.safe_dump(data or {}),
+        )
+        self.assertEqual(result.returncode, 0)
+
+    def add_suite_to_archive(self, suite_name: str) -> None:
+        """Add a suite to our workspace's archive."""
+        # TODO: We don't have a sensible command to do this yet.
+        result = DebusineServer.execute_command(
+            "shell",
+            "-c",
+            dedent(
+                f"""\
+                from debusine.artifacts.models import *
+                from debusine.db.models import *
+
+                user = User.objects.get(username="test-user")
+                workspace = default_workspace()
+                archive = workspace.get_singleton_collection(
+                    user=user, category=CollectionCategory.ARCHIVE
+                )
+                suite = workspace.get_collection(
+                    user=user,
+                    category=CollectionCategory.SUITE,
+                    name={suite_name!r},
+                )
+                archive.manager.add_collection(suite, user=user)
+                """
+            ),
+        )
+        self.assertEqual(result.returncode, 0)
+
+    def get_archive_signing_key(self) -> str:
+        """Get the signing key for our workspace's archive."""
+        # TODO: We don't have a sensible command to do this yet.
+        result = DebusineServer.execute_command(
+            "shell",
+            "-c",
+            dedent(
+                """\
+                import base64
+
+                from debusine.assets.models import AssetCategory, KeyPurpose
+                from debusine.db.models import default_workspace
+
+                workspace = default_workspace()
+                signing_key = workspace.asset_set.filter(
+                    category=AssetCategory.SIGNING_KEY,
+                    data__purpose=KeyPurpose.OPENPGP_REPOSITORY,
+                ).latest("created_at")
+                print(
+                    base64.b64decode(
+                        signing_key.data["public_key"].encode()
+                    ).decode(),
+                    end="",
+                )
+                """
+            ),
+        )
+        self.assertEqual(result.returncode, 0)
+        return result.stdout
