@@ -17,7 +17,7 @@ import json
 import re
 from collections.abc import Iterable, Sequence
 from json import JSONDecodeError
-from pathlib import Path
+from pathlib import Path, PurePath
 from typing import Any, ClassVar, Generic, Self, TypeVar, cast, overload
 
 from debian import deb822, debfile
@@ -704,6 +704,23 @@ class AutopkgtestArtifact(LocalArtifact[data_models.DebianAutopkgtest]):
         return artifact
 
 
+class PiupartsArtifact(LocalArtifact[data_models.DebianPiuparts]):
+    """Piuparts: encapsulate result of a ``piuparts`` run."""
+
+    _category = data_models.ArtifactCategory.PIUPARTS
+
+    @classmethod
+    def create(
+        cls, *, piuparts_output: Path, data: data_models.DebianPiuparts
+    ) -> Self:
+        """Return a ``PiupartsArtifact`` with the files and data set."""
+        artifact = cls(category=cls._category, data=data)
+
+        artifact.add_local_file(piuparts_output, override_name="piuparts.txt")
+
+        return artifact
+
+
 class DebianSystemTarballArtifact(
     LocalArtifact[data_models.DebianSystemTarball]
 ):
@@ -745,18 +762,24 @@ class DebianSystemTarballArtifact(
         return files
 
 
-class BlhcArtifact(LocalArtifact[data_models.EmptyArtifactData]):
+class BlhcArtifact(LocalArtifact[data_models.DebianBlhc]):
     """BlhcArtifact: encapsulate result of the blhc run."""
 
     _category = data_models.ArtifactCategory.BLHC
-    data: data_models.EmptyArtifactData = pydantic.Field(
-        default_factory=data_models.EmptyArtifactData
-    )
 
     @classmethod
-    def create(cls, blhc_output: Path) -> Self:
+    def create(
+        cls,
+        blhc_output: Path,
+        build_log: data_models.DebianPackageBuildLog | None,
+    ) -> Self:
         """Return a BlhcArtifact with the files set."""
-        artifact = cls(category=cls._category)
+        data = data_models.DebianBlhc(
+            source=None if build_log is None else build_log.source,
+            version=None if build_log is None else build_log.version,
+            architecture=None if build_log is None else build_log.architecture,
+        )
+        artifact = cls(category=cls._category, data=data)
 
         artifact.add_local_file(blhc_output, override_name="blhc.txt")
 
@@ -898,17 +921,15 @@ class SigningOutputArtifact(LocalArtifact[data_models.DebusineSigningOutput]):
         return artifact
 
 
-class RepositoryIndex(LocalArtifact[data_models.EmptyArtifactData]):
+class RepositoryIndex(LocalArtifact[data_models.DebianRepositoryIndex]):
     """An index file in a repository."""
 
     _category = data_models.ArtifactCategory.REPOSITORY_INDEX
-    data: data_models.EmptyArtifactData = pydantic.Field(
-        default_factory=data_models.EmptyArtifactData
-    )
 
     @classmethod
-    def create(cls, *, file: Path, override_name: str | None = None) -> Self:
+    def create(cls, *, file: Path, path: str) -> Self:
         """Return a RepositoryIndex."""
-        artifact = cls(category=cls._category)
-        artifact.add_local_file(file, override_name=override_name)
+        data = data_models.DebianRepositoryIndex(path=path)
+        artifact = cls(category=cls._category, data=data)
+        artifact.add_local_file(file, override_name=PurePath(path).name)
         return artifact
