@@ -274,6 +274,7 @@ class ArtifactCategory(enum.StrEnum):
     BINARY_PACKAGES = "debian:binary-packages"
     LINTIAN = "debian:lintian"
     AUTOPKGTEST = "debian:autopkgtest"
+    PIUPARTS = "debian:piuparts"
     SYSTEM_TARBALL = "debian:system-tarball"
     BLHC = "debian:blhc"
     DEBDIFF = "debian:debdiff"
@@ -708,6 +709,19 @@ class DebianAutopkgtest(ArtifactData):
         return self.source_package.name
 
 
+class DebianPiuparts(ArtifactData):
+    """Data for debian:piuparts artifacts."""
+
+    piuparts_version: str
+    binary_packages: dict[str, str]
+    architecture: str
+    distribution: str
+
+    def get_label(self) -> str:
+        """Return a short human-readable label for the artifact."""
+        return f"piuparts: {', '.join(sorted(self.binary_packages))}"
+
+
 class DebusineSigningInput(ArtifactData):
     """Input to a Sign task."""
 
@@ -752,6 +766,26 @@ class DebusineSigningOutput(ArtifactData):
         return None
 
 
+class DebianBlhc(ArtifactData):
+    """Data for debian:blhc artifacts."""
+
+    # TODO: Once old artifacts created without them have expired, these
+    # should become required.
+    source: str | None = None
+    version: str | None = None
+    architecture: str | None = None
+
+    def get_label(self) -> str:
+        """Return a short human-readable label for the artifact."""
+        if self.architecture is None:
+            return "blhc analysis"
+        else:
+            return (
+                f"blhc analysis for"
+                f" {self.source}_{self.version}_{self.architecture}"
+            )
+
+
 class DebDiff(ArtifactData):
     """Data for debian:debdiff artifacts."""
 
