@@ -96,9 +96,12 @@ class ArtifactPlayground:
         source: bool = True,
         src_name: str = "hello",
         version: str = "1.0-1",
+        src_architectures: set[str] | None = None,
         binary: bool = False,
         binaries: list[str] | None = None,
         bin_architecture: str = "all",
+        section: str = "unknown",
+        priority: str = "optional",
     ) -> Upload:
         """Create an Upload artifact."""
         architectures: set[str] = set()
@@ -128,6 +131,7 @@ class ArtifactPlayground:
                 source=src_name,
                 binaries=binaries,
                 version=version,
+                architectures=src_architectures,
             )
             files += [dsc] + src_files
         if binary:
@@ -135,7 +139,13 @@ class ArtifactPlayground:
             for bin_name in binaries:
                 deb = destdir / f"{bin_name}_{version}_{bin_architecture}.deb"
                 cls.write_deb_file(
-                    deb, source_name=src_name, source_version=version
+                    deb,
+                    source_name=src_name,
+                    source_version=version,
+                    extra_control_fields={
+                        "Section": section,
+                        "Priority": priority,
+                    },
                 )
                 files.append(deb)
         else:
@@ -149,6 +159,7 @@ class ArtifactPlayground:
             source=src_name,
             binaries=binaries,
             version=version,
+            section=section,
         )
         return Upload.create(changes_file=changes)
 
@@ -195,6 +206,7 @@ class ArtifactPlayground:
         source_version: str | None = None,
         control_file_names: list[str] | None = None,
         maintainer: str = "Example Maintainer <example@example.org>",
+        extra_control_fields: dict[str, str] | None = None,
         data_files: dict[PurePath, bytes] | None = None,
         data_symlinks: dict[PurePath, PurePath] | None = None,
     ) -> dict[str, Any]:
@@ -226,6 +238,9 @@ class ArtifactPlayground:
             )
             if source != package:
                 control_contents += f"Source: {source}\n"
+            if extra_control_fields is not None:
+                for key, value in extra_control_fields.items():
+                    control_contents += f"{key}: {value}\n"
             (build_directory / "DEBIAN" / "control").write_text(
                 control_contents
             )
@@ -272,6 +287,7 @@ class ArtifactPlayground:
         architectures: set[str] | None = None,
         binnmu: bool = False,
         maintainer: str = "Example Maintainer <example@example.org>",
+        section: str = "unknown",
     ) -> dict[str, Any]:
         """Write dsc or changes file with files information."""
         architectures = set(architectures or set())
@@ -357,7 +373,9 @@ class ArtifactPlayground:
                     " only Dsc and Changes are supported"
                 )
         changes_contents += cls.hash_deb822_files(
-            file_type, {file.name: file.read_bytes() for file in files}
+            file_type,
+            {file.name: file.read_bytes() for file in files},
+            section=section,
         )
         path.write_text(changes_contents)
 
@@ -369,6 +387,7 @@ class ArtifactPlayground:
         file_contents: dict[str, bytes],
         break_hashes: Set[str] = frozenset(),
         break_sizes: Set[str] = frozenset(),
+        section: str = "unknown",
     ) -> str:
         """
         Build the file-manifest parts of a Deb822 file that we're generating.
@@ -397,7 +416,7 @@ class ArtifactPlayground:
                     size += 1
                 if hash_ == "md5" and file_type == deb822.Changes:
                     output.append(
-                        f" {hasher.hexdigest()} {size} unknown optional "
+                        f" {hasher.hexdigest()} {size} {section} optional "
                         f"{filename}"
                     )
                 else:
