@@ -1218,14 +1218,25 @@ class BlhcArtifactTests(TestCase):
 
     def test_create(self) -> None:
         """Test _create()."""
+        build_log = data_models.DebianPackageBuildLog(
+            source="hello",
+            version="1.0",
+            architecture="amd64",
+            filename="hello_1.0_amd64.buildlog",
+        )
         output = self.create_temporary_file(
             suffix=".txt", contents=self.output_content
         )
 
-        package = BlhcArtifact.create(blhc_output=output)
+        package = BlhcArtifact.create(blhc_output=output, build_log=build_log)
 
         self.assertIsInstance(package, BlhcArtifact)
-        self.assertEqual(package.data, data_models.EmptyArtifactData())
+        self.assertEqual(
+            package.data,
+            data_models.DebianBlhc(
+                source="hello", version="1.0", architecture="amd64"
+            ),
+        )
 
         self.assertEqual(
             package.files,
@@ -1237,12 +1248,21 @@ class BlhcArtifactTests(TestCase):
     def test_json_serializable(self) -> None:
         """Artifact returned by create is JSON serializable."""
         """upload_artifact() and relation_create() is called."""
+        build_log = data_models.DebianPackageBuildLog(
+            source="hello",
+            version="1.0",
+            architecture="amd64",
+            filename="hello_1.0_amd64.buildlog",
+        )
+
         # Create file for the artifact
         blhc_output = self.create_temporary_file(
             suffix=".txt", contents=b"NONVERBOSE BUILD: Compiling gkrust\n"
         )
 
-        artifact = BlhcArtifact.create(blhc_output=blhc_output)
+        artifact = BlhcArtifact.create(
+            blhc_output=blhc_output, build_log=build_log
+        )
 
         # No exception is raised
         json.dumps(
