@@ -29,9 +29,9 @@ class EnumTests(TestCase):
             data_models.DebianAutopkgtestResultStatus,
             data_models.DebianLintianSeverity,
         ):
-            with self.subTest(enum_cls=enum_cls):
+            with self.subTest(enum_cls=repr(enum_cls)):
                 for el in enum_cls:
-                    with self.subTest(el=el):
+                    with self.subTest(el=el.value):
                         self.assertEqual(str(el), el.value)
 
 
@@ -569,6 +569,19 @@ class DebianAutopkgtestTest(TestCase):
         self.assertEqual(data.get_label(), "hello")
 
 
+class DebianPiupartsTest(TestCase):
+    """Test the DebianPiuparts model."""
+
+    def test_get_label(self) -> None:
+        data = data_models.DebianPiuparts(
+            piuparts_version="1.6.0",
+            binary_packages={"hello": "1.0", "hello-doc": "1.0"},
+            architecture="amd64",
+            distribution="debian:bookworm",
+        )
+        self.assertEqual(data.get_label(), "piuparts: hello, hello-doc")
+
+
 class DebusineSigningInputTest(TestCase):
     """Test the DebusineSigningInput model."""
 
@@ -663,6 +676,20 @@ class DebusineSigningOutputTest(TestCase):
         self.assertIsNone(label)
 
 
+class DebianBlhcTests(TestCase):
+    """Test the DebianBlhc model."""
+
+    def test_get_label_no_architecture(self) -> None:
+        data = data_models.DebianBlhc()
+        self.assertEqual(data.get_label(), "blhc analysis")
+
+    def test_get_label(self) -> None:
+        data = data_models.DebianBlhc(
+            source="hello", version="1.0", architecture="amd64"
+        )
+        self.assertEqual(data.get_label(), "blhc analysis for hello_1.0_amd64")
+
+
 class DebDiffTest(TestCase):
     """Test the DebDiff model."""
 
