@@ -104,8 +104,15 @@ def asset_categories() -> list[AssetCategory]:
 class KeyPurpose(DjangoChoicesEnum):
     """Choices for SigningKey.purpose."""
 
+    # Sign a UEFI Secure Boot image.
     UEFI = "uefi"
-    OPENPGP = "openpgp"
+
+    # Sign a Debian upload (.changes) using OpenPGP.  (The value should
+    # perhaps be "openpgp-upload", but is kept this way for compatibility.)
+    OPENPGP_UPLOAD = "openpgp"
+
+    # Sign Debian repository indexes using OpenPGP.
+    OPENPGP_REPOSITORY = "openpgp-repository"
 
 
 class SigningKeyData(
