@@ -27,6 +27,7 @@ from itertools import islice
 from pathlib import Path
 from typing import Any, NoReturn
 
+import argcomplete
 import yaml
 from dateutil.parser import isoparse
 from requests.exceptions import RequestException
@@ -53,6 +54,7 @@ from debusine.client.models import (
     WorkRequestRequest,
     WorkRequestResponse,
     WorkflowTemplateRequest,
+    WorkspaceInheritanceChain,
     model_to_json_serializable_dict,
     model_to_json_serializable_list,
 )
@@ -60,6 +62,7 @@ from debusine.client.task_configuration import (
     InvalidRepository,
     LocalTaskConfigurationRepository,
 )
+from debusine.utils.input import YamlEditor
 
 
 class Cli:
@@ -487,8 +490,50 @@ class Cli:
             help="push the collection, skipping all checks",
         )
 
+        workspace_inheritance = subparsers.add_parser(
+            "workspace-inheritance",
+            help="Query and change the inheritance chain of a workspace",
+        )
+        workspace_inheritance.add_argument(
+            "workspace", type=str, help="workspace name"
+        )
+        workspace_inheritance.add_argument(
+            "--append",
+            type=str,
+            nargs="+",
+            help="workspaces (id, 'name', or 'scope/name')"
+            " to add to the end of the parent list",
+        )
+        workspace_inheritance.add_argument(
+            "--prepend",
+            type=str,
+            nargs="+",
+            help="workspaces (id, 'name', or 'scope/name')"
+            " to add to the beginning of the parent list",
+        )
+        workspace_inheritance.add_argument(
+            "--remove",
+            type=str,
+            nargs="+",
+            help="workspaces (id, 'name', or 'scope/name')"
+            " to remove from the parent list",
+        )
+        workspace_inheritance.add_argument(
+            "--set",
+            type=str,
+            nargs="+",
+            help="workspaces (id, 'name', or 'scope/name')"
+            " that compose the new parent list",
+        )
+        workspace_inheritance.add_argument(
+            "--edit",
+            action="store_true",
+            help="edit the parent list using $EDITOR",
+        )
+
         subparsers.add_parser("setup", help="Guided setup of debusine-client")
 
+        argcomplete.autocomplete(parser)
         self.args = parser.parse_args(self._argv)
 
     def _build_debusine_object(self) -> Debusine:
@@ -685,6 +730,11 @@ class Cli:
                     dry_run=self.args.dry_run,
                     force=self.args.force,
                 )
+            case "workspace-inheritance":
+                self._workspace_inheritance(
+                    debusine,
+                    self.args.workspace,
+                )
             case _ as unreachable:
                 raise AssertionError(
                     f"Unexpected sub-command name: {unreachable}"
@@ -1456,3 +1506,62 @@ class Cli:
             results.removed,
             results.unchanged,
         )
+
+    def _workspace_inheritance(
+        self, debusine: Debusine, workspace: str
+    ) -> None:
+        with self._api_call_or_fail():
+            try:
+                current = debusine.get_workspace_inheritance(workspace)
+            except DebusineError as err:
+                output = {"result": "failure", "error": err.asdict()}
+                self._print_yaml(output)
+                raise SystemExit(3)
+
+        new: WorkspaceInheritanceChain = current
+        if self.args.append:
+            new = new + WorkspaceInheritanceChain.from_strings(self.args.append)
+        if self.args.prepend:
+            new = (
+                WorkspaceInheritanceChain.from_strings(self.args.prepend) + new
+            )
+        if self.args.remove:
+            new = new - WorkspaceInheritanceChain.from_strings(self.args.remove)
+        if self.args.set:
+            new = WorkspaceInheritanceChain.from_strings(self.args.set)
+        if self.args.edit:
+            editor: YamlEditor[list[Any]] = YamlEditor(
+                new.dict()["chain"],
+                help_text=[
+                    "Here you can configure the inheritance"
+                    f" of workspace '{workspace}'.",
+                    "",
+                    "This is a YAML list of dictionaries with"
+                    " 'id/scope/workspace' keys.",
+                    "Usually you specify the workspace,"
+                    " and optionally the scope. The scope",
+                    "is only required to refer to a workspace"
+                    " outside of the current scope.",
+                    "",
+                    "For example:",
+                    "",
+                    "- workspace: base",
+                    "- scope: some-other-scope",
+                    "  workspace: foobar",
+                ],
+            )
+            if editor.edit():
+                new = WorkspaceInheritanceChain(chain=editor.value)
+
+        if new != current:
+            with self._api_call_or_fail():
+                try:
+                    current = debusine.set_workspace_inheritance(
+                        workspace, chain=new
+                    )
+                except DebusineError as err:
+                    output = {"result": "failure", "error": err.asdict()}
+                    self._print_yaml(output)
+                    raise SystemExit(3)
+
+        self._print_yaml(current.dict())
