@@ -36,6 +36,10 @@ from debusine.client.models import (
 )
 
 
+class TokenFetchFailed(Exception):
+    """Exception raised when fetching a token failed."""
+
+
 class ServerInfo(NamedTuple):
     """Information about a known server."""
 
@@ -47,6 +51,36 @@ class ServerInfo(NamedTuple):
     api_url: str
     #: Default Scope
     scope: str
+    #: API token
+    api_token: str | None = None
+
+    def lint(self) -> list[str]:
+        """
+        Run consistency checks on the current draft configuration.
+
+        :returns: a list of messages, or empty if everything looks ordinary
+        """
+        res: list[str] = []
+
+        api_url_parsed = urllib.parse.urlparse(self.api_url)
+
+        if api_url_parsed.hostname == "localhost":
+            expected_scheme = "http"
+        else:
+            expected_scheme = "https"
+
+        if api_url_parsed.scheme != expected_scheme:
+            res.append(
+                f"Server url is an [bold]{api_url_parsed.scheme}[/] url,"
+                f" should it be [bold]{expected_scheme}[/]?"
+            )
+
+        if not self.api_url.endswith("/api") and not self.api_url.endswith(
+            "/api/"
+        ):
+            res.append("API URL does not end in /api or /api/")
+
+        return res
 
     @classmethod
     def from_config(cls, name: str, section: configobj.Section) -> Self:
@@ -54,9 +88,10 @@ class ServerInfo(NamedTuple):
         inferred = cls.from_string(name, desc="temporary serverinfo")
         return cls(
             name=name,
-            desc=f"Configured in {section.main.filename}",
+            desc=f"Configured in {os.path.basename(section.main.filename)}",
             api_url=section.get("api-url", inferred.api_url),
             scope=section.get("scope", inferred.scope),
+            api_token=section.get("token", None),
         )
 
     @classmethod
@@ -119,122 +154,19 @@ class ServerInfo(NamedTuple):
             name=server_name, desc=desc, api_url=api_url, scope="debusine"
         )
 
-
-KNOWN_SERVERS: dict[str, ServerInfo] = {
-    info.name: info
-    for info in (
-        ServerInfo(
-            name="debian",
-            desc="debusine.debian.net",
-            api_url="https://debusine.debian.net/api/",
-            scope="debian",
-        ),
-        ServerInfo(
-            name="freexian",
-            desc="Freexian (E)LTS",
-            api_url="https://debusine.freexian.com/api/",
-            scope="freexian",
-        ),
-        ServerInfo(
-            name="localhost",
-            desc="debusine-admin runserver for development",
-            api_url="http://localhost:8000/api/",
-            scope="debusine",
-        ),
-    )
-}
-
-
-class ServerConfig:
-    """Server configuration editor backend."""
-
-    #: Server to configure
-    server: ServerInfo
-    #: Default scope
-    scope: str
-    #: API URL
-    api_url: str
-    #: API token
-    api_token: str | None
-
-    def __init__(self, config: ConfigObj, server: ServerInfo) -> None:
-        """
-        Create a ServerConfig.
-
-        :param config: configuration file
-        :param server: ServerInfo with the server to configure
-        """
-        self.config = config
-        self.server = server
+    def fetch_token(self, console: Console) -> str | None:
+        """Get an API token from the server."""
+        # Load data for generating challenges
         xkcd_wordfile = xkcd_password.locate_wordfile()
-        self.xkcd_words = xkcd_password.generate_wordlist(
+        xkcd_words = xkcd_password.generate_wordlist(
             wordfile=xkcd_wordfile, min_length=5, max_length=8
         )
 
-        # Load initial values from config, using server as defaults
-        server_section = self.server_section
-        self.api_url = server_section.get("api-url", server.api_url)
-        self.scope = server_section.get("scope", server.scope)
-        self.api_token = server_section.get("token")
-
-    def save(self) -> None:
-        """Save the configuration."""
-        path = Path(self.config.filename)
-        path.parent.mkdir(parents=True, exist_ok=True)
-        section = self.server_section
-        section["api-url"] = self.api_url
-        section["scope"] = self.scope
-        if self.api_token is None:
-            if "token" in section:
-                del section["token"]
-        else:
-            section["token"] = self.api_token
-        self.config.write()
-
-    @property
-    def server_section(self) -> configobj.Section:
-        """Return the ConfigObj section we are editing."""
-        key = f"server:{self.server.name}"
-        if (section := self.config.get(key)) is None:
-            self.config[key] = {}
-            section = self.config[key]
-        return section
-
-    def lint(self) -> list[str]:
-        """
-        Run consistency checks on the current draft configuration.
-
-        :returns: a list of messages, or empty if everything looks ordinary
-        """
-        res: list[str] = []
-
-        api_url_parsed = urllib.parse.urlparse(self.api_url)
-
-        if api_url_parsed.hostname == "localhost":
-            expected_scheme = "http"
-        else:
-            expected_scheme = "https"
-
-        if api_url_parsed.scheme != expected_scheme:
-            res.append(
-                f"Server url is an [bold]{api_url_parsed.scheme}[/] url,"
-                f" should it be [bold]{expected_scheme}[/]?"
-            )
-
-        if not self.api_url.endswith("/api") and not self.api_url.endswith(
-            "/api/"
-        ):
-            res.append("API URL does not end in /api or /api/")
-
-        return res
-
-    def fetch_token(self, console: Console) -> bool:
-        """Get an API token from the server."""
         payload = EnrollPayload(
             # Nonce used to identify the enrolling session
             nonce=secrets.token_urlsafe(16),
             # Challenge text
-            challenge=xkcd_password.generate_xkcdpassword(self.xkcd_words),
+            challenge=xkcd_password.generate_xkcdpassword(xkcd_words),
             scope=self.scope,
             hostname=platform.node(),
         )
@@ -273,68 +205,202 @@ class ServerConfig:
         try:
             response = requests.post(enroll_url, json=payload_dict)
         except Exception as e:
-            console.print(":exclamation_mark:", "request to server failed:", e)
-            return False
+            raise TokenFetchFailed(f"request to server failed: {e}") from e
 
         if response.status_code != 200:
-            console.print(
-                ":exclamation_mark:",
-                "The server returned status code",
-                response.status_code,
-            )
+            lead = f"The server returned status code {response.status_code}: "
             try:
                 result = response.json()
             except requests.exceptions.JSONDecodeError:
-                console.print(
-                    ":exclamation_mark:", rich.markup.escape(response.text)
-                )
-                return False
+                raise TokenFetchFailed(lead + response.text)
             else:
                 if message := result.get("title"):
-                    console.print(
-                        ":exclamation_mark:", rich.markup.escape(message)
-                    )
+                    raise TokenFetchFailed(lead + message)
                 else:
-                    console.print(":exclamation_mark:", json.dumps(result))
-            return False
+                    raise TokenFetchFailed(lead + json.dumps(result))
 
         try:
             result = response.json()
         except requests.exceptions.JSONDecodeError:
-            console.print(":exclamation_mark: Invalid JSON in response")
-            return False
+            raise TokenFetchFailed("Invalid JSON in response")
 
         try:
             confirmation = EnrollConfirmPayload.parse_obj(result)
         except ValueError:
-            console.print(":exclamation_mark: Invalid response payload")
-            return False
+            raise TokenFetchFailed("Invalid response payload")
 
         match confirmation.outcome:
             case EnrollOutcome.CONFIRM:
-                self.api_token = confirmation.token
                 console.print(
                     ":white_check_mark: Confirmation confirmed, token acquired"
                 )
+                return confirmation.token
             case EnrollOutcome.CANCEL:
-                self.api_token = None
                 console.print(":exclamation_mark: Confirmation cancelled")
+                return None
             case _ as unreachable:
                 assert_never(unreachable)
 
+
+KNOWN_SERVERS: dict[str, ServerInfo] = {
+    info.name: info
+    for info in (
+        ServerInfo(
+            name="debian",
+            desc="debusine.debian.net",
+            api_url="https://debusine.debian.net/api/",
+            scope="debian",
+        ),
+        ServerInfo(
+            name="freexian",
+            desc="Freexian (E)LTS",
+            api_url="https://debusine.freexian.com/api/",
+            scope="freexian",
+        ),
+        ServerInfo(
+            name="localhost",
+            desc="debusine-admin runserver for development",
+            api_url="http://localhost:8000/api/",
+            scope="debusine",
+        ),
+    )
+}
+
+
+class DebusineClientConfig:
+    """Editable configuration for debusine client."""
+
+    def __init__(self, path: Path) -> None:
+        """
+        Initialize from a path.
+
+        :param path: path to the configuration file
+        """
+        self.path = path
+        self.config = ConfigObj(
+            self.path.as_posix(),
+            interpolation=False,
+            list_values=False,
+        )
+        self.servers: dict[str, ServerInfo] = {
+            name[7:]: ServerInfo.from_config(name[7:], section)
+            for name, section in self.config.items()
+            if name.startswith("server:")
+        }
+        self.edited: dict[str, ServerInfo] = {}
+
+        # Currently selected server
+        self.current_server: ServerInfo | None = None
+
+        # Name of the server set as default
+        self.default_server: str | None = None
+        if section := self.config.get("General"):
+            self.default_server = section.get("default-server")
+
+    def select(self, name: str, desc: str) -> None:
+        """
+        Select the server to edit.
+
+        :param name: server name
+        :param desc: optional description used if creating a new server
+        """
+        if server := self.edited.get(name):
+            # Server is already being edited
+            pass
+        elif server := self.servers.get(name):
+            # Server is in the existing configuration
+            self.edited[server.name] = server
+        elif server := KNOWN_SERVERS.get(name):
+            # Server is known
+            self.edited[server.name] = server
+        else:
+            server = ServerInfo.from_string(name, desc)
+            self.edited[server.name] = server
+        self.current_server = server
+
+    @property
+    def server(self) -> ServerInfo:
+        """Require the current server to be set and return it."""
+        if self.current_server is None:
+            raise RuntimeError("current server has not been selected")
+        return self.current_server
+
+    def set_api_url(self, api_url: str) -> None:
+        """Set the API url of the current server."""
+        edited = self.server._replace(api_url=api_url)
+        self.current_server = edited
+        self.edited[edited.name] = edited
+
+    def set_scope(self, scope: str) -> None:
+        """Set the scope of the current server."""
+        edited = self.server._replace(scope=scope)
+        self.current_server = edited
+        self.edited[edited.name] = edited
+
+    def set_api_token(self, api_token: str | None) -> None:
+        """Set the API token of the current server."""
+        edited = self.server._replace(api_token=api_token)
+        self.current_server = edited
+        self.edited[edited.name] = edited
+
+    def set_default(self) -> None:
+        """Set the current server as default server."""
+        self.default_server = self.server.name
+
+    def fetch_token(self, console: Console) -> bool:
+        """Fetch a token and set it in the current configuration."""
+        try:
+            api_token = self.server.fetch_token(console)
+        except TokenFetchFailed as e:
+            console.print(":exclamation_mark:", rich.markup.escape(str(e)))
+            return False
+        self.set_api_token(api_token)
         return True
 
+    def save(self) -> None:
+        """Save the configuration."""
+        for name, current in self.edited.items():
+            orig = self.servers.get(name)
+
+            # Get the server section to save
+            key = f"server:{name}"
+            if (section := self.config.get(key)) is None:
+                self.config[key] = {}
+                section = self.config[key]
+
+            # Update with the edited configuration
+            if orig is None or orig.api_url != current.api_url:
+                section["api-url"] = current.api_url
+            if orig is None or orig.scope != current.scope:
+                section["scope"] = current.scope
+            if orig is None or orig.api_token != current.api_token:
+                if current.api_token is None:
+                    if "token" in section:
+                        del section["token"]
+                else:
+                    section["token"] = current.api_token
+
+        if self.default_server is not None:
+            if section := self.config.get("General"):
+                if section.get("default-server") != self.default_server:
+                    section["default-server"] = self.default_server
+            else:
+                self.config["General"] = {"default-server": self.default_server}
+
+        self.path.parent.mkdir(parents=True, exist_ok=True)
+        self.config.write()
+
 
 class ServerConfigEditor(DataEntry):
     """Interactive editor for ServerConfig."""
 
     prompt = "> "
 
-    config: ServerConfig
+    config: DebusineClientConfig
 
     def __init__(
         self,
-        config: ServerConfig,
+        config: DebusineClientConfig,
         console: Console | None = None,
     ) -> None:
         """Initialize the ServerConfig editor."""
@@ -347,59 +413,69 @@ class ServerConfigEditor(DataEntry):
 
     def menu(self) -> None:
         """Print a menu for the user."""
+        server = self.config.server
+        is_default = self.config.server.name == self.config.default_server
+        title = f"Configuration for {server.name}"
+        if is_default:
+            title += " [green not bold](default server)[/]"
+        else:
+            title += (
+                " [yellow not bold]"
+                f"(default is [i]{self.config.default_server}[/])"
+                "[/]"
+            )
         table = Table(box=rich.box.SIMPLE)
-        table.add_column(
-            f"Configuration for {self.config.server.name}",
-            justify="left",
-            no_wrap=True,
-        )
+        table.add_column(title, justify="left", no_wrap=True)
         table.add_column("", justify="left", no_wrap=True)
 
-        table.add_row(fr"\[server:[bold]{self.config.server.name}[bold]]", "")
+        table.add_row(fr"\[server:[bold]{server.name}[bold]]", "")
+        table.add_row(f"scope = [bold]{server.scope}[bold]", "([bold]sc[/]ope)")
         table.add_row(
-            f"scope = [bold]{self.config.scope}[bold]", "([bold]sc[/]ope)"
-        )
-        table.add_row(
-            f"api-url = [bold]{self.config.api_url}[bold]", "([bold]u[/]rl)"
+            f"api-url = [bold]{server.api_url}[bold]", "([bold]u[/]rl)"
         )
         table.add_row(
             "token = [italic]"
-            + (
-                "acquired on save"
-                if self.config.api_token is None
-                else "present"
-            )
+            + ("acquired on save" if server.api_token is None else "present")
             + "[/]",
             "([bold]t[/]oken)",
         )
 
         self.console.print(table)
 
-        for msg in self.config.lint():
+        for msg in server.lint():
             self.lint_message(msg)
         self.console.print()
         self.console.print(
-            "Commands: [bold]u[/]rl, [bold]t[/]oken,"
-            " [bold]sc[/]ope, [bold]s[/]ave, [bold]q[/]uit:"
+            "Commands: [bold]u[/]rl, [bold]t[/]oken, [bold]sc[/]ope, "
+            "(make) [bold]d[/]efault, [bold]s[/]ave, [bold]q[/]uit:"
         )
 
+    def do_default(self, arg: str) -> None:  # noqa: U100
+        """Set as default server."""
+        self.config.set_default()
+
     def do_scope(self, arg: str) -> None:  # noqa: U100
         """Change the scope."""
-        self.config.scope = self.input_line(
-            "Enter scope", initial=self.config.scope, required=True
+        server = self.config.server
+        self.config.set_scope(
+            self.input_line("Enter scope", initial=server.scope, required=True)
         )
 
     def do_url(self, arg: str) -> None:  # noqa: U100
         """Change the API URL."""
-        self.config.api_url = self.input_line(
-            "Enter API URL", initial=self.config.api_url, required=True
+        server = self.config.server
+        self.config.set_api_url(
+            self.input_line(
+                "Enter API URL", initial=server.api_url, required=True
+            )
         )
 
     def do_token(self, arg: str) -> None:  # noqa: U100
         """Change the token."""
-        self.config.api_token = (
+        server = self.config.server
+        self.config.set_api_token(
             self.input_line(
-                "Enter (or clear) API token", initial=self.config.api_token
+                "Enter (or clear) API token", initial=server.api_token
             )
             or None
         )
@@ -413,7 +489,7 @@ class ServerConfigEditor(DataEntry):
 
     def do_save(self, arg: str) -> bool:  # noqa: U100
         """Save the new configuration."""
-        if not self.config.api_token:
+        if not self.config.server.api_token:
             while True:
                 if self.config.fetch_token(self.console):
                     break
@@ -437,11 +513,15 @@ class ServerSelector(DataEntry):
     selected: ServerInfo | None
 
     def __init__(
-        self, entries: list[ServerInfo], console: Console | None = None
+        self,
+        entries: list[ServerInfo],
+        console: Console | None = None,
+        default: str | None = None,
     ) -> None:
         """Initialize the Server selector."""
         super().__init__(console=console)
         self.entries = entries
+        self.default_name = default
         self.selected = None
 
     def menu(self) -> None:
@@ -449,10 +529,16 @@ class ServerSelector(DataEntry):
         table = Table(title="Configured servers", box=rich.box.SIMPLE)
         table.add_column("#", justify="right", no_wrap=True)
         table.add_column("Name", justify="left", no_wrap=True)
+        table.add_column("Current", justify="left", no_wrap=True)
         table.add_column("Description", justify="left", no_wrap=True)
 
         for idx, info in enumerate(self.entries, start=1):
-            table.add_row(str(idx), info.name, info.desc)
+            table.add_row(
+                str(idx),
+                info.name,
+                "yes" if info.name == self.default_name else "",
+                info.desc,
+            )
 
         self.console.print(table)
 
@@ -517,37 +603,38 @@ def setup_server(
     console: Console | None = None,
 ) -> None:
     """Create or edit a server entry."""
-    config_path = Path(config_file_path)
-    config = ConfigObj(
-        config_path.as_posix(),
-        interpolation=False,
-        list_values=False,
-    )
+    console = console or Console()
+    config = DebusineClientConfig(Path(config_file_path))
 
     if server is None:
         # Ask the user to pick a server
-        servers = [
-            ServerInfo.from_config(name[7:], section)
-            for name, section in config.items()
-            if name.startswith("server:")
-        ]
-        seen = {x.name for x in servers}
+        servers: list[ServerInfo] = []
+        servers.extend(config.servers.values())
         for info in KNOWN_SERVERS.values():
-            if info.name in seen:
+            if info.name in config.servers.keys():
                 continue
             servers.append(info)
-        server_selector = ServerSelector(servers, console=console)
+        server_selector = ServerSelector(
+            servers, console=console, default=config.default_server
+        )
         server_selector.cmdloop()
         if server_selector.selected is None:
             # Abort if no server was selected
             return
-        server_info = server_selector.selected
+        config.select(server_selector.selected.name, "selected from menu")
     else:
-        server_info = ServerInfo.from_string(server, "provided on command line")
+        config.select(server, "provided on command line")
 
-    server_config = ServerConfig(config, server_info)
     if scope is not None:
-        server_config.scope = scope
+        config.set_scope(scope)
 
-    server_config_editor = ServerConfigEditor(server_config, console=console)
+    # Set server as default if no default was set
+    if config.default_server is None:
+        config.set_default()
+        console.print(
+            ":star: Default server was not set,"
+            f" setting it to [bold]{config.default_server}[/]"
+        )
+
+    server_config_editor = ServerConfigEditor(config, console=console)
     server_config_editor.cmdloop()
