@@ -1349,7 +1349,7 @@ class DebusineTests(TestCase):
         work_request = 5
 
         asset_data = SigningKeyData(
-            purpose=KeyPurpose.OPENPGP,
+            purpose=KeyPurpose.OPENPGP_UPLOAD,
             fingerprint="ABC123",
             public_key="PUBLIC KEY",
             description="Description",
@@ -1407,7 +1407,7 @@ class DebusineTests(TestCase):
     ) -> None:
         """Call client.asset_list with the specified options, assert results."""
         asset_data = SigningKeyData(
-            purpose=KeyPurpose.OPENPGP,
+            purpose=KeyPurpose.OPENPGP_UPLOAD,
             fingerprint="ABC123",
             public_key="PUBLIC KEY",
             description="Description",
