@@ -53,7 +53,11 @@ class DebusineHttpClientTests(TestCase):
         self.token = "token-for-server"
 
     def responses_add_test_endpoint(
-        self, status: int, method: str = responses.GET
+        self,
+        status: int,
+        method: str = responses.GET,
+        *,
+        rsps: responses.RequestsMock = responses.mock,
     ) -> None:
         """
         Add a test response.
@@ -63,7 +67,7 @@ class DebusineHttpClientTests(TestCase):
         json = None
         if status == requests.codes.ok:  # pragma: no cover
             json = {'id': 12}
-        responses.add(
+        rsps.add(
             method=method,
             url=self.api_url,
             json=json,
@@ -318,7 +322,6 @@ class DebusineHttpClientTests(TestCase):
         [int(part) for part in version("responses").split(".")] < [0, 21],
         "Requires responses >= 0.21 that supports testing Retry",
     )
-    @responses.activate(registry=OrderedRegistry)
     def test_api_request_retry_errors(self) -> None:  # pragma: no cover
         """Some HTTP error responses are retried."""
         debusine_client = DebusineHttpClient(self.api_url, self.token)
@@ -335,21 +338,25 @@ class DebusineHttpClientTests(TestCase):
                 responses.POST,
                 responses.PUT,
             ):
-                with self.subTest(method=method, code=code):
-                    responses.reset()
-                    self.responses_add_test_endpoint(method=method, status=code)
+                with (
+                    self.subTest(method=method, code=code),
+                    responses.RequestsMock(registry=OrderedRegistry) as rsps,
+                ):
                     self.responses_add_test_endpoint(
-                        method=method, status=requests.codes.ok
+                        method=method, status=code, rsps=rsps
+                    )
+                    self.responses_add_test_endpoint(
+                        method=method, status=requests.codes.ok, rsps=rsps
                     )
                     debusine_client._api_request(
                         method, self.api_url, SampleResponse
                     )
+                    rsps.reset()
 
     @skipIf(
         [int(part) for part in version("responses").split(".")] < [0, 21],
         "Requires responses >= 0.21 that supports testing Retry",
     )
-    @responses.activate(registry=OrderedRegistry)
     def test_api_request_dont_retry_error(self) -> None:  # pragma: no cover
         """Many HTTP error responses are not retried, test a few."""
         debusine_client = DebusineHttpClient(self.api_url, self.token)
@@ -361,10 +368,14 @@ class DebusineHttpClientTests(TestCase):
             # Enabled by default in Retry:
             requests.codes.request_entity_too_large,  # 413
         ):
-            with self.subTest(code=code):
-                responses.reset()
-                self.responses_add_test_endpoint(status=code)
-                self.responses_add_test_endpoint(status=requests.codes.ok)
+            with (
+                self.subTest(code=code),
+                responses.RequestsMock(registry=OrderedRegistry) as rsps,
+            ):
+                self.responses_add_test_endpoint(status=code, rsps=rsps)
+                self.responses_add_test_endpoint(
+                    status=requests.codes.ok, rsps=rsps
+                )
                 with self.assertRaises(
                     (
                         exceptions.ClientForbiddenError,
@@ -374,6 +385,7 @@ class DebusineHttpClientTests(TestCase):
                     debusine_client._api_request(
                         responses.GET, self.api_url, SampleResponse
                     )
+                rsps.reset()
 
     @responses.activate
     def test_api_request_returns_not_json_raise_exception(self) -> None:
@@ -566,7 +578,7 @@ class DebusineHttpClientTests(TestCase):
             requests.exceptions.RequestException,
             ConnectionError,
         ):
-            with self.subTest(exception=exception):
+            with self.subTest(exception=repr(exception)):
                 responses.add(
                     responses.GET,
                     self.api_url,
