@@ -377,7 +377,7 @@ class AssetsResponsesTest(TestCase):
                     category=AssetCategory.SIGNING_KEY,
                     workspace="test",
                     data=SigningKeyData(
-                        purpose=KeyPurpose.OPENPGP,
+                        purpose=KeyPurpose.OPENPGP_UPLOAD,
                         fingerprint="ABC123",
                         public_key="PUBLIC KEY",
                         description="Description",
@@ -591,7 +591,7 @@ class WorkspaceInheritanceChainElementTests(TestCase):
             (sample, El(id=42, scope="invalid", workspace="workspace"), False),
             (sample, El(id=42, scope="scope", workspace="invalid"), False),
         ):
-            with self.subTest(a=a, b=b):
+            with self.subTest(a=repr(a), b=repr(b)):
                 self.assertEqual(a.matches(b), expected)
 
     def test_invalid(self) -> None:
@@ -682,5 +682,5 @@ class WorkspaceInheritanceChainTests(TestCase):
             (chain(el1, el2, el3), chain(), chain(el1, el2, el3)),
             (chain(el1, el2, el3), chain(el1, el3), chain(el2)),
         ):
-            with self.subTest(a=a, b=b):
+            with self.subTest(a=repr(a), b=repr(b)):
                 self.assertEqual(a - b, expected)
