@@ -41,6 +41,7 @@ from debusine.db.models.scopes import FileStoreInScope, Scope
 from debusine.db.models.task_database import TaskDatabase
 from debusine.db.models.work_requests import (
     NotificationChannel,
+    WellKnownWorkflowTemplate,
     WorkRequest,
     WorkflowTemplate,
 )
@@ -77,6 +78,7 @@ __all__ = [
     "TaskDatabase",
     "Token",
     "User",
+    "WellKnownWorkflowTemplate",
     "Worker",
     "WorkerPool",
     "WorkerPoolStatistics",
