@@ -239,6 +239,21 @@ class Artifact(models.Model):
 
     objects = ArtifactManager.from_queryset(ArtifactQuerySet)()
 
+    class Meta(TypedModelMeta):
+        indexes = [
+            # Used by debusine.server.open_metrics.DebusineCollector.  This
+            # is imperfect (on debusine.debian.net at the time of writing it
+            # only takes the artifact-counting query from ~550ms to ~300ms).
+            # Doing any better seems likely to involve a separate
+            # trigger-maintained statistics table.
+            models.Index(
+                "workspace",
+                "category",
+                name="%(app_label)s_%(class)s_workspace_category",
+                include=["id"],
+            )
+        ]
+
     def clean(self) -> None:
         """
         Ensure that data is valid for this artifact category.
