@@ -10,11 +10,12 @@
 """Data models for assets."""
 
 import enum
-from typing import Any, Generic, TYPE_CHECKING, TypeAlias, TypeVar
+from typing import Any, Generic, Self, TYPE_CHECKING, TypeAlias, TypeVar, cast
 
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import models
-from django.db.models.constraints import CheckConstraint
+from django.db.models import Q
+from django.db.models.constraints import CheckConstraint, UniqueConstraint
 
 from debusine.assets import (
     AssetCategory,
@@ -28,10 +29,12 @@ from debusine.db.models import permissions
 from debusine.db.models.permissions import (
     Allow,
     PermissionUser,
+    Role,
     enforce,
     permission_check,
     permission_filter,
 )
+from debusine.db.models.scopes import Scope
 from debusine.db.models.workspaces import Workspace
 from debusine.tasks.models import WorkerType
 from debusine.utils.typing_utils import copy_signature_from
@@ -42,13 +45,106 @@ else:
     TypedModelMeta = object
 
 
-class UnknownPermissionError(Exception):
-    """A permission was requested, but does not exist."""
-
-
 A = TypeVar("A")
 
 
+class AssetRoles(permissions.Roles, permissions.RoleBase, enum.ReprEnum):
+    """Available roles for an Asset."""
+
+    OWNER = "owner"
+
+
+AssetRoles.setup()
+
+
+class AssetUsageRoleBase(permissions.RoleBase):
+    """AssetUsage role implementation."""
+
+    implied_by_scope_roles: frozenset[Scope.Roles]
+    implied_by_workspace_roles: frozenset[Workspace.Roles]
+    implied_by_asset_usage_roles: frozenset["AssetUsageRoles"]
+
+    def _setup(self) -> None:
+        """Set up implications for a newly constructed role."""
+        implied_by_scope_roles: set[Scope.Roles] = set()
+        implied_by_workspace_roles: set[Workspace.Roles] = set()
+        implied_by_asset_usage_roles: set[AssetUsageRoles] = {
+            cast(AssetUsageRoles, self)
+        }
+        for i in self.implied_by:
+            match i:
+                case Workspace.Roles():
+                    implied_by_scope_roles |= i.implied_by_scope_roles
+                    implied_by_workspace_roles |= i.implied_by_workspace_roles
+                case Role():
+                    # Resolve a role passed during class definition into its
+                    # enum instance
+                    role = self.__class__(i.value)
+                    implied_by_scope_roles |= role.implied_by_scope_roles
+                    implied_by_workspace_roles |= (
+                        role.implied_by_workspace_roles
+                    )
+                    implied_by_asset_usage_roles |= (
+                        role.implied_by_asset_usage_roles
+                    )
+                case _:
+                    raise ImproperlyConfigured(
+                        f"AssetUsage roles do not support implications by {i!r}"
+                    )
+        self.implied_by_scope_roles = frozenset(implied_by_scope_roles)
+        self.implied_by_workspace_roles = frozenset(implied_by_workspace_roles)
+        self.implied_by_asset_usage_roles = frozenset(
+            implied_by_asset_usage_roles
+        )
+
+    def q(self, user: PermissionUser) -> Q:
+        """Return a Q expression to select asset usages with this role."""
+        q = Q(
+            roles__group__users=user,
+            roles__role__in=self.implied_by_asset_usage_roles,
+        )
+        if self.implied_by_workspace_roles:
+            q |= Q(
+                workspace__in=Workspace.objects.filter(
+                    Q(
+                        roles__group__users=user,
+                        roles__role__in=self.implied_by_workspace_roles,
+                    )
+                    | Q(
+                        scope__in=Scope.objects.filter(
+                            roles__group__users=user,
+                            roles__role__in=self.implied_by_scope_roles,
+                        )
+                    )
+                )
+            )
+        return q
+
+    def implies(self, role: "AssetUsageRoles") -> bool:
+        """Check if this role implies the given one."""
+        return (
+            self.implied_by_scope_roles <= role.implied_by_scope_roles
+            and self.implied_by_workspace_roles
+            <= role.implied_by_workspace_roles
+            and self.implied_by_asset_usage_roles
+            <= role.implied_by_asset_usage_roles
+        )
+
+
+class AssetUsageRoles(permissions.Roles, AssetUsageRoleBase, enum.ReprEnum):
+    """Available roles for an AssetUsage."""
+
+    SIGNER = Role("signer")
+    REPOSITORY_SIGNER = Role(
+        "repository_signer",
+        label="Repository signer",
+        implied_by=[SIGNER, Workspace.Roles.OWNER],
+    )
+
+
+AssetUsageRoles.setup()
+
+
 class AssetQuerySet(models.QuerySet["Asset", A], Generic[A]):
     """Custom QuerySet for Asset."""
 
@@ -99,6 +195,28 @@ class AssetManager(models.Manager["Asset"]):
                 raise ValueError(f"No slug defined for category '{category}'")
 
 
+class AssetUsageQuerySet(models.QuerySet["AssetUsage", A], Generic[A]):
+    """Custom QuerySet for AssetUsage."""
+
+    def with_role(self, user: PermissionUser, role: AssetUsageRoles) -> Self:
+        """Keep only resources where the user has the given role."""
+        if not user or not user.is_authenticated:
+            return self.none()
+        return self.filter(role.q(user)).distinct()
+
+    @permission_filter(workers=Allow.PASS)
+    def can_sign_with(self, user: PermissionUser) -> "AssetUsageQuerySet[A]":
+        """Keep only AssetUsages that the user can sign with."""
+        return self.with_role(user, AssetUsageRoles.SIGNER)
+
+    @permission_filter(workers=Allow.PASS)
+    def can_sign_repository_with(
+        self, user: PermissionUser
+    ) -> "AssetUsageQuerySet[A]":
+        """Keep only AssetUsages that the user can sign a repository with."""
+        return self.with_role(user, AssetUsageRoles.REPOSITORY_SIGNER)
+
+
 class AssetUsageManager(models.Manager["AssetUsage"]):
     """Manager for the AssetUsage model."""
 
@@ -107,24 +225,6 @@ class AssetUsageManager(models.Manager["AssetUsage"]):
         return AssetUsageRole
 
 
-class AssetRoles(permissions.Roles, permissions.RoleBase, enum.ReprEnum):
-    """Available roles for an Asset."""
-
-    OWNER = "owner"
-
-
-AssetRoles.setup()
-
-
-class AssetUsageRoles(permissions.Roles, permissions.RoleBase, enum.ReprEnum):
-    """Available roles for an AssetUsage."""
-
-    SIGNER = "signer"
-
-
-AssetUsageRoles.setup()
-
-
 class Asset(models.Model):
     """Asset model."""
 
@@ -226,30 +326,6 @@ class Asset(models.Model):
             )
         )
 
-    def has_permission(
-        self,
-        permission: str,
-        user: PermissionUser,
-        workspace: Workspace | None = None,
-    ) -> bool:
-        """Check user permissions on this asset (outside request context)."""
-        if user is None or not user.is_authenticated:
-            return False
-        try:
-            role = {
-                'edit': AssetRoles.OWNER,
-                'sign_with': AssetUsageRoles.SIGNER,
-                'manage_permissions': AssetRoles.OWNER,
-            }[permission]
-        except KeyError:
-            raise UnknownPermissionError()
-        if workspace is not None:
-            return self.usage.filter(
-                workspace=workspace, roles__role=role, roles__group__users=user
-            ).exists()
-        else:
-            return self.roles.filter(role=role, group__users=user).exists()
-
     @property
     def data_model(self) -> BaseAssetDataModel:
         """Instantiate AssetData from data."""
@@ -325,7 +401,35 @@ class AssetUsage(models.Model):
         related_name="asset_usage",
     )
 
-    objects = AssetUsageManager()
+    objects = AssetUsageManager.from_queryset(AssetUsageQuerySet)()
+
+    class Meta(TypedModelMeta):
+        constraints = [
+            UniqueConstraint(
+                fields=["asset", "workspace"],
+                name="%(app_label)s_%(class)s_unique_asset_workspace",
+            )
+        ]
+
+    def has_role(self, user: PermissionUser, role: AssetUsageRoles) -> bool:
+        """Check if the user has the given role on this AssetUsage."""
+        return (
+            AssetUsage.objects.with_role(user, role).filter(pk=self.pk).exists()
+        )
+
+    @permission_check("{user} cannot sign with {resource}", workers=Allow.PASS)
+    def can_sign_with(self, user: PermissionUser) -> bool:
+        """Check if the user can sign with this resource."""
+        assert user is not None and user.is_authenticated
+        return self.has_role(user, AssetUsageRoles.SIGNER)
+
+    @permission_check(
+        "{user} cannot sign a repository with {resource}", workers=Allow.PASS
+    )
+    def can_sign_repository_with(self, user: PermissionUser) -> bool:
+        """Check if the user can sign a repository with this resource."""
+        assert user is not None and user.is_authenticated
+        return self.has_role(user, AssetUsageRoles.REPOSITORY_SIGNER)
 
 
 class AssetUsageRole(models.Model):
@@ -343,4 +447,4 @@ class AssetUsageRole(models.Model):
         related_name="asset_usage_roles",
     )
 
-    role = models.CharField(max_length=16, choices=AssetUsageRoles.choices)
+    role = models.CharField(max_length=32, choices=AssetUsageRoles.choices)
