@@ -10,6 +10,7 @@
 """Data models for db users and authentication."""
 
 import abc
+import enum
 import hashlib
 import re
 import secrets
@@ -26,14 +27,13 @@ from typing import (
     TypeAlias,
     TypeVar,
     Union,
-    assert_never,
     cast,
 )
 
 from django.conf import settings
 from django.contrib.auth.models import AbstractUser
 from django.contrib.auth.models import UserManager as DjangoUserManager
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.core.validators import MaxLengthValidator, MinLengthValidator
 from django.db import models
 from django.db.models import CheckConstraint, Q, QuerySet, UniqueConstraint
@@ -48,8 +48,9 @@ except ImportError:
 from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import permissions
 from debusine.db.models.permissions import (
-    PartialCheckResult,
+    Allow,
     PermissionUser,
+    Role,
     get_resource_scope,
     permission_check,
     permission_filter,
@@ -84,7 +85,10 @@ class TokenManager(models.Manager["Token"]):
         )
 
     def get_tokens(
-        self, username: str | None = None, key: str | None = None
+        self,
+        username: str | None = None,
+        key: str | None = None,
+        include_expired: bool = False,
     ) -> QuerySet["Token"]:
         """
         Return all the tokens filtered by a specific owner and/or token.
@@ -100,6 +104,9 @@ class TokenManager(models.Manager["Token"]):
             token_hash = hashlib.sha256(key.encode()).hexdigest()
             tokens = tokens.filter(hash=token_hash)
 
+        if not include_expired:
+            tokens = tokens.exclude(expire_at__lt=timezone.now())
+
         return tokens
 
     def get_token_or_none(self, token_key: str) -> Optional["Token"]:
@@ -221,22 +228,18 @@ def system_user() -> "User":
 class UserQuerySet(QuerySet["User", A], Generic[A]):
     """Custom QuerySet for User."""
 
-    @permission_filter
+    @permission_filter(workers=Allow.ALWAYS)
     def can_display(
         self, user: PermissionUser  # noqa: U100
     ) -> "UserQuerySet[A]":
         """Keep only Users that can be displayed."""
         assert user is not None  # Enforced by decorator
-        if not user.is_authenticated:
-            return self.none()
         return self
 
-    @permission_filter
+    @permission_filter()
     def can_manage(self, user: PermissionUser) -> "UserQuerySet[A]":
         """Keep only Users that the given user can manage."""
         assert user is not None  # Enforced by decorator
-        if not user.is_authenticated:
-            return self.none()
         return self.filter(pk=user.pk)
 
 
@@ -285,7 +288,9 @@ class User(AbstractUser):
             ),
         ]
 
-    @permission_check("{user} cannot display user {resource}")
+    @permission_check(
+        "{user} cannot display user {resource}", workers=Allow.ALWAYS
+    )
     def can_display(self, user: PermissionUser) -> bool:  # noqa: U100
         """Check if the user can be displayed."""
         assert user is not None  # enforced by decorator
@@ -296,15 +301,12 @@ class User(AbstractUser):
         #
         # This could help make life a bit harder, for example, for drive-by
         # spammers trying to enumerate users and their personal information.
-        return user.is_authenticated
+        return True
 
     @permission_check("{user} cannot manage user {resource}")
     def can_manage(self, user: PermissionUser) -> bool:
         """Check if the user can manage this user."""
         assert user is not None  # enforced by decorator
-        # Token is not taken into account here
-        if not user.is_authenticated:
-            return False
         return self.pk == user.pk
 
     def get_absolute_url(self) -> str:
@@ -372,39 +374,28 @@ def validate_group_name(value: str) -> None:
 class GroupQuerySet(QuerySet["Group", A], Generic[A]):
     """Custom QuerySet for Group."""
 
-    @permission_filter
+    @permission_filter()
     def can_display(self, user: PermissionUser) -> "GroupQuerySet[A]":
         """Keep only groups that can be displayed."""
         assert user is not None  # Enforced by decorator
-        # Workers cannot display group details
-        if context.worker_token:
-            return self.none()
-        # Anonymous users cannot display group details
-        if not user.is_authenticated:
-            return self.none()
-        # Otherwise all groups can be displayed
         return self.all()
 
-    @permission_filter
+    @permission_filter()
     def can_display_audit_log(self, user: PermissionUser) -> "GroupQuerySet[A]":
         """Keep only groups whose audit log can be displayed."""
         return self.can_display(user)
 
-    @permission_filter
+    @permission_filter()
     def can_manage(self, user: PermissionUser) -> "GroupQuerySet[A]":
         """Keep only groups that can be managed."""
         assert user is not None  # Enforced by decorator
-        # Workers cannot manage groups
-        if context.worker_token:
-            return self.none()
-        # Anonymous users cannot manage groups
-        if not user.is_authenticated:
-            return self.none()
 
         # Allow to scope owners
         constraints = Q(
-            scope__roles__group__users=user,
-            scope__roles__role=Scope.Roles.OWNER,
+            scope__in=Scope.objects.filter(
+                roles__group__users=user,
+                roles__role=Scope.Roles.OWNER,
+            )
         )
         # Allow to group admins
         constraints |= Q(
@@ -412,6 +403,22 @@ class GroupQuerySet(QuerySet["Group", A], Generic[A]):
         )
         return self.filter(constraints).distinct()
 
+    def unused_ephemeral(self) -> "GroupQuerySet[Any]":
+        """List ephemeral groups with no roles assigned."""
+        constraints = Q()
+        # Introspect reverse foreign keys to detect which resources have a
+        # *Roles table
+        for field in Group._meta.get_fields():
+            if (
+                field.auto_created
+                and not field.concrete
+                and field.name.endswith("_roles")
+            ):
+                # If this resource is currently assigning roles, then the group
+                # is still in use
+                constraints &= Q(**{f"{field.name}__isnull": True})
+        return self.filter(ephemeral=True).filter(constraints)
+
 
 class GroupManager(models.Manager["Group"]):
     """Manager for Group model."""
@@ -432,28 +439,48 @@ class GroupManager(models.Manager["Group"]):
         group.add_user(owner, Group.Roles.ADMIN)
         return group
 
-    def unused_ephemeral(self) -> QuerySet["Group"]:
-        """List ephemeral groups with no roles assigned."""
-        constraints = Q()
-        # Introspect reverse foreign keys to detect which resources have a
-        # *Roles table
-        for field in Group._meta.get_fields():
-            if (
-                field.auto_created
-                and not field.concrete
-                and field.name.endswith("_roles")
-            ):
-                # If this resource is currently assigning roles, then the group
-                # is still in use
-                constraints &= Q(**{f"{field.name}__isnull": True})
-        return Group.objects.filter(ephemeral=True).filter(constraints)
+
+class GroupRoleBase(permissions.RoleBase):
+    """Group role implementation."""
+
+    implied_by_scope_roles: frozenset["Scope.Roles"]
+    implied_by_group_roles: frozenset["GroupRoles"]
+
+    def _setup(self) -> None:
+        """Set up implications for a newly constructed role."""
+        implied_by_scope_roles: set[Scope.Roles] = set()
+        implied_by_group_roles: set[GroupRoles] = {cast(GroupRoles, self)}
+        for i in self.implied_by:
+            match i:
+                case Scope.Roles():
+                    implied_by_scope_roles |= i.implied_by_scope_roles
+                case Role():
+                    # Resolve a role passed during class definition into its
+                    # enum instance
+                    gr = self.__class__(i.value)
+                    implied_by_scope_roles |= gr.implied_by_scope_roles
+                    implied_by_group_roles |= gr.implied_by_group_roles
+                case _:
+                    raise ImproperlyConfigured(
+                        "Group roles do not support"
+                        f" implications by {type(i)}"
+                    )
+        self.implied_by_scope_roles = frozenset(implied_by_scope_roles)
+        self.implied_by_group_roles = frozenset(implied_by_group_roles)
+
+    def implies(self, role: "GroupRoles") -> bool:
+        """Check if this role implies the given one."""
+        return self in role.implied_by_group_roles
 
 
-class GroupRoles(permissions.Roles):
+class GroupRoles(permissions.Roles, GroupRoleBase, enum.ReprEnum):
     """Available roles for a Scope."""
 
-    ADMIN = "admin", "Admin"
-    MEMBER = "member", "Member"
+    ADMIN = Role("admin", implied_by=[Scope.Roles.OWNER])
+    MEMBER = Role("member", implied_by=[ADMIN])
+
+
+GroupRoles.setup()
 
 
 class Group(models.Model):
@@ -519,13 +546,6 @@ class Group(models.Model):
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the group can be displayed."""
         assert user is not None  # enforced by decorator
-        # Workers cannot display group details
-        if context.worker_token:
-            return False
-        # Anonymous users cannot display group details
-        if not user.is_authenticated:
-            return False
-        # Otherwise any logged in user can display group details
         return True
 
     @permission_check("{user} cannot display the audit log of {resource}")
@@ -537,25 +557,18 @@ class Group(models.Model):
     def can_manage(self, user: PermissionUser) -> bool:
         """Check if the group can be managed."""
         assert user is not None  # enforced by decorator
-        # Workers cannot manage groups
-        if context.worker_token:
-            return False
-        # Anonymous users cannot manage groups
-        if not user.is_authenticated:
-            return False
         # Shortcut to avoid hitting the database for common cases
-        match self.scope.context_has_role(user, Scope.Roles.OWNER):
-            case PartialCheckResult.ALLOW:
-                return True
-            case PartialCheckResult.DENY | PartialCheckResult.PASS:
-                pass
-            case _ as unreachable:
-                assert_never(unreachable)
+        if (
+            context.user == user
+            and context.scope == self.scope
+            and not context.scope_roles.isdisjoint(
+                GroupRoles.ADMIN.implied_by_scope_roles
+            )
+        ):
+            return True
         return Group.objects.can_manage(user).filter(pk=self.pk).exists()
 
-    def assign_role(
-        self, resource: models.Model, role: permissions.Roles | str
-    ) -> models.Model:
+    def assign_role(self, resource: models.Model, role: str) -> models.Model:
         """Assign a role on a resource."""
         if get_resource_scope(resource) != self.scope:
             raise ValueError(
@@ -563,7 +576,7 @@ class Group(models.Model):
                 f" is not in scope {self.scope}"
             )
 
-        role = resolve_role(getattr(resource, "Roles"), role)
+        role = resolve_role(resource, role)
 
         roles_model_cls: models.Model = getattr(
             resource.__class__, "objects"
