@@ -106,6 +106,12 @@ class CollectionQuerySet(QuerySet["Collection", A], Generic[A]):
         # Delegate to workspace can_display check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
+    @permission_filter()
+    def can_configure(self, user: PermissionUser) -> "CollectionQuerySet[A]":
+        """Keep only Collections that can be configured."""
+        # Delegate to workspace can_configure check
+        return self.filter(workspace__in=Workspace.objects.can_configure(user))
+
 
 class CollectionManager(models.Manager["Collection"]):
     """Manager for Collection model."""
@@ -239,6 +245,13 @@ class Collection(models.Model):
         """Check if the collection can be displayed."""
         return self.workspace.can_display(user)
 
+    @permission_check(
+        "{user} cannot configure collection {resource}",
+    )
+    def can_configure(self, user: PermissionUser) -> bool:
+        """Check if the collection can be configured."""
+        return self.workspace.can_configure(user)
+
     def __str__(self) -> str:
         """Return id, name, category."""
         # Stringify using a valid lookup syntax
