@@ -53,7 +53,7 @@ class ArtifactManagerTests(TestCase):
         paths[0].write_text("1\n")
         paths[1].write_text("2\n")
         local_artifact = WorkRequestDebugLogs.create(files=paths)
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
 
         with context.disable_permission_checks():
             artifact = Artifact.objects.create_from_local_artifact(
