@@ -672,7 +672,7 @@ class GroupTests(TestCase):
             (self.scenario.workspace, Workspace.Roles.OWNER),
             (self.scenario.workspace, "owner"),
         ):
-            with self.subTest(resource=resource, role=role):
+            with self.subTest(resource=repr(resource), role=repr(role)):
                 assigned = self.group.assign_role(resource, role)
                 # TODO: we can remove the type ignores once we find a way to
                 # generically type the role assignment models
@@ -1073,7 +1073,7 @@ class GroupAuditLogEntryTests(TestCase):
                 "actor created group test in scope debusine",
             ),
         ):
-            with self.subTest(group=group):
+            with self.subTest(group=repr(group)):
                 entry = auth.GroupAuditLogCreated.create(group)
                 entry.actor = "actor"
                 self.assertEqual(
@@ -1120,7 +1120,7 @@ class GroupAuditLogEntryTests(TestCase):
                 "actor updated group, no changes detected",
             ),
         ):
-            with self.subTest(other_group=other_group):
+            with self.subTest(other_group=repr(other_group)):
                 entry = auth.GroupAuditLogUpdated.create_diff(
                     group, other_group
                 )
