@@ -9,8 +9,9 @@
 
 """Unit tests for the models."""
 
-import django.db.utils
 from django.core.exceptions import ValidationError
+from django.db import transaction
+from django.db.utils import IntegrityError
 
 from debusine.db.models import File, FileInStore, FileStore
 from debusine.db.tests.utils import _calculate_hash_from_data
@@ -25,6 +26,7 @@ class FileManagerTests(TestCase):
 
     def setUp(self) -> None:
         """Set up File to be used in the tests."""
+        super().setUp()
         self.file_contents = b"test"
         self.file_hash = _calculate_hash_from_data(self.file_contents)
 
@@ -70,6 +72,7 @@ class FileTests(TestCase):
 
     def setUp(self) -> None:
         """Set up File to be used in the tests."""
+        super().setUp()
         self.file_contents = b"test"
         self.file_hash = _calculate_hash_from_data(self.file_contents)
 
@@ -121,7 +124,7 @@ class FileTests(TestCase):
     def test_unique_constraint_hash_size(self) -> None:
         """File with same hash_digest and size cannot be created."""
         with self.assertRaisesRegex(
-            django.db.utils.IntegrityError, "db_file_unique_sha256_size"
+            IntegrityError, "db_file_unique_sha256_size"
         ):
             File.objects.create(
                 **{
@@ -141,9 +144,7 @@ class FileTests(TestCase):
 
     def test_constraint_hash_digest_not_empty(self) -> None:
         """File.hash_digest cannot be empty."""
-        with self.assertRaisesRegex(
-            django.db.utils.IntegrityError, "db_file_sha256_not_empty"
-        ):
+        with self.assertRaisesRegex(IntegrityError, "db_file_sha256_not_empty"):
             File.objects.get_or_create(hash_digest=b"", size=5)
 
 
@@ -152,6 +153,7 @@ class FileStoreTests(TestCase):
 
     def setUp(self) -> None:
         """Set up FileStore to be used in the tests."""
+        super().setUp()
         self.backend = FileStore.BackendChoices.LOCAL
         self.file_store_name = "nas-01"
 
@@ -209,6 +211,50 @@ class FileStoreTests(TestCase):
         # Assert that self.file_store through model return the expected file
         self.assertEqual(self.file_store.files.first(), file)
 
+    def test_total_size(self) -> None:
+        """Adding/removing files to/from the store maintains `total_size`."""
+        files = [
+            self.playground.create_file(contents=b"\0" * length)
+            for length in (0, 4, 10, 1024, 512)
+        ]
+
+        expected_total_size = 0
+        self.file_store.refresh_from_db()
+        self.assertEqual(self.file_store.total_size, expected_total_size)
+        for file in files:
+            FileInStore.objects.create(file=file, store=self.file_store)
+            expected_total_size += file.size
+            self.file_store.refresh_from_db()
+            self.assertEqual(self.file_store.total_size, expected_total_size)
+        for file in files:
+            FileInStore.objects.filter(
+                file=file, store=self.file_store
+            ).delete()
+            expected_total_size -= file.size
+            self.file_store.refresh_from_db()
+            self.assertEqual(self.file_store.total_size, expected_total_size)
+
+    def test_max_size_constraint(self) -> None:
+        """If both are set, `soft_max_size` must be <= `max_size`."""
+        for soft_max_size, max_size, ok in (
+            (None, None, True),
+            (1024, None, True),
+            (None, 1024, True),
+            (1024, 1024, True),
+            (1024, 2048, True),
+            (2048, 1024, False),
+        ):
+            with transaction.atomic():
+                self.file_store.soft_max_size = soft_max_size
+                self.file_store.max_size = max_size
+                if ok:
+                    self.file_store.save()
+                else:
+                    with self.assertRaisesRegex(
+                        IntegrityError, "db_filestore_consistent_max_sizes"
+                    ):
+                        self.file_store.save()
+
     def test_get_backend_object(self) -> None:
         """get_backend_object instantiates the right backend."""
         fs_local = FileStore(
@@ -256,6 +302,7 @@ class FileInStoreTests(TestCase):
 
     def setUp(self) -> None:
         """Set up FileInStore to be used in the tests."""
+        super().setUp()
         self.file_store_name = "nas-01"
         self.file_store = FileStore.objects.create(
             name="nas-01",
