@@ -9,28 +9,91 @@
 
 """Unit tests for the scopes models."""
 from typing import ClassVar
-from unittest import mock
 
 from django.contrib.auth.models import AnonymousUser
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import IntegrityError, transaction
 from django.test import override_settings
 
 from debusine.db.context import context
-from debusine.db.models import FileInStore, FileStore, Group, Scope
-from debusine.db.models.permissions import PartialCheckResult
-from debusine.db.models.scopes import ScopeRole, is_valid_scope_name
+from debusine.db.models import FileInStore, FileStore, Group, Scope, User
+from debusine.db.models.permissions import Allow
+from debusine.db.models.scopes import (
+    ScopeRole,
+    ScopeRoleBase,
+    ScopeRoles,
+    is_valid_scope_name,
+)
+from debusine.db.models.tests.utils import RolesTestCase
 from debusine.db.playground import scenarios
 from debusine.test.django import TestCase
 
 
+class ScopeRolesTests(RolesTestCase[ScopeRoles]):
+    """Tests for the ScopeRoles class."""
+
+    roles_class = ScopeRoles
+
+    def test_invariants(self) -> None:
+        self.assertRolesInvariants()
+
+    def test_from_iterable(self) -> None:
+        for value, expected in (
+            ([], []),
+            (["owner"], [Scope.Roles.OWNER]),
+            ([Scope.Roles.OWNER], [Scope.Roles.OWNER]),
+            (["owner", "owner", Scope.Roles.OWNER], [Scope.Roles.OWNER]),
+        ):
+            with self.subTest(value=repr(value)):
+                self.assertEqual(
+                    Scope.Roles.from_iterable(value),
+                    frozenset(expected),
+                )
+
+    def test_choices(self) -> None:
+        self.assertChoices()
+
+    def test_values(self) -> None:
+        self.assertEqual(ScopeRoles.OWNER.label, "Owner")
+        self.assertEqual(
+            ScopeRoles.OWNER.implied_by_scope_roles,
+            frozenset([ScopeRoles.OWNER]),
+        )
+
+    def test_init_invalid_implication(self) -> None:
+        r = ScopeRoleBase("test", implied_by=[Scope.Roles.OWNER])
+        with self.assertRaisesRegex(
+            ImproperlyConfigured,
+            r"Scope roles do not support implications"
+            r" by <enum 'ScopeRoles'>",
+        ):
+            r._setup()
+
+
 class ScopeManagerTests(TestCase):
     """Tests for the ScopeManager class."""
 
+    scenario = scenarios.DefaultScopeUser()
+
     def test_get_roles_model(self) -> None:
         """Test the get_roles_model method."""
         self.assertIs(Scope.objects.get_roles_model(), ScopeRole)
 
+    def test_with_role(self) -> None:
+        self.scenario.scope_owners.users.add(self.scenario.user)
+        other = self.playground.create_user("other")
+
+        for user, expected in (
+            (None, []),
+            (AnonymousUser(), []),
+            (other, []),
+            (self.scenario.user, [self.scenario.scope]),
+        ):
+            with self.subTest(user=repr(user)):
+                self.assertQuerySetEqual(
+                    Scope.objects.with_role(user, Scope.Roles.OWNER), expected
+                )
+
 
 @override_settings(LANGUAGE_CODE="en-us")
 class ScopeTests(TestCase):
@@ -38,17 +101,33 @@ class ScopeTests(TestCase):
 
     scenario = scenarios.DefaultScopeUser()
 
-    def assertAllow(self, value: PartialCheckResult) -> None:
-        """Check that the value is ALLOW."""
-        self.assertEqual(value, PartialCheckResult.ALLOW)
+    def assertHasRole(
+        self,
+        *,
+        user: User | None = None,
+        scope: Scope | None = None,
+        hits_database: bool = False,
+    ) -> None:
+        """Ensure the user has OWNER role on the scope."""
+        scope = scope or self.scenario.scope
+        user = user or self.scenario.user
+        expected_queries = 1 if hits_database else 0
+        with self.assertNumQueries(expected_queries):
+            self.assertTrue(scope.has_role(user, Scope.Roles.OWNER))
 
-    def assertDeny(self, value: PartialCheckResult) -> None:
-        """Check that the value is DENY."""
-        self.assertEqual(value, PartialCheckResult.DENY)
-
-    def assertPass(self, value: PartialCheckResult) -> None:
-        """Check that the value is PASS."""
-        self.assertEqual(value, PartialCheckResult.PASS)
+    def assertHasNoRole(
+        self,
+        *,
+        user: User | None = None,
+        scope: Scope | None = None,
+        hits_database: bool = False,
+    ) -> None:
+        """Ensure the user does not have OWNER role on the scope."""
+        scope = scope or self.scenario.scope
+        user = user or self.scenario.user
+        expected_queries = 1 if hits_database else 0
+        with self.assertNumQueries(expected_queries):
+            self.assertFalse(scope.has_role(user, Scope.Roles.OWNER))
 
     def test_create(self) -> None:
         """Test basic behavior."""
@@ -128,25 +207,27 @@ class ScopeTests(TestCase):
             {'name': ["'_foo' is not a valid scope name"]},
         )
 
-    def test_get_roles_not_authenticated(self) -> None:
-        """Test Scope.get_roles when not authenticated."""
+    def test_get_group_roles_not_authenticated(self) -> None:
+        """Test Scope.get_group_roles when not authenticated."""
         scope = Scope.objects.create(name="Scope")
         user = AnonymousUser()
-        self.assertQuerySetEqual(scope.get_roles(user), [])
+        self.assertQuerySetEqual(scope.get_group_roles(user), [])
 
-    def test_get_roles(self) -> None:
-        """Test Scope.get_roles."""
+    def test_get_group_roles(self) -> None:
+        """Test Scope.get_group_roles."""
         scope = Scope.objects.create(name="Scope")
         group = self.playground.create_group_role(scope, ScopeRole.Roles.OWNER)
 
         user = self.scenario.user
-        self.assertQuerySetEqual(scope.get_roles(user), [])
+        self.assertQuerySetEqual(scope.get_group_roles(user), [])
 
         self.playground.add_user(group, user)
-        self.assertEqual(list(scope.get_roles(user)), [ScopeRole.Roles.OWNER])
+        self.assertEqual(
+            list(scope.get_group_roles(user)), [ScopeRole.Roles.OWNER]
+        )
 
-    def test_get_roles_multiple_groups(self) -> None:
-        """Test Scope.get_roles."""
+    def test_get_group_roles_multiple_groups(self) -> None:
+        """Test Scope.get_group_roles with the user in multiple groups."""
         scope = Scope.objects.create(name="Scope")
         group1 = self.playground.create_group_role(
             scope, ScopeRole.Roles.OWNER, name="Group1"
@@ -156,63 +237,61 @@ class ScopeTests(TestCase):
         )
 
         user = self.scenario.user
-        self.assertQuerySetEqual(scope.get_roles(user), [])
+        self.assertQuerySetEqual(scope.get_group_roles(user), [])
 
         self.playground.add_user(group1, user)
-        self.assertEqual(list(scope.get_roles(user)), [ScopeRole.Roles.OWNER])
+        self.assertEqual(
+            list(scope.get_group_roles(user)), [ScopeRole.Roles.OWNER]
+        )
 
         self.playground.add_user(group2, user)
-        self.assertEqual(list(scope.get_roles(user)), [ScopeRole.Roles.OWNER])
-
-    def test_context_has_role_no_context(self) -> None:
-        """Test context_has_role with no context set."""
-        self.assertPass(
-            self.scenario.scope.context_has_role(
-                self.scenario.user, Scope.Roles.OWNER
-            )
+        self.assertEqual(
+            list(scope.get_group_roles(user)), [ScopeRole.Roles.OWNER]
         )
 
-    def test_context_has_role_wrong_user(self) -> None:
-        """Test context_has_role with a different user than in context."""
+    def test_has_role_anonymous_user(self) -> None:
+        """Text has_role with anonymous users."""
+        for user in (None, AnonymousUser()):
+            with self.subTest(user=repr(user)), self.assertNumQueries(0):
+                self.assertFalse(
+                    self.scenario.scope.has_role(user, Scope.Roles.OWNER)
+                )
+
+    def test_has_role_no_context(self) -> None:
+        """Test has_role with no context set."""
+        self.assertHasNoRole(hits_database=True)
+        self.scenario.scope_owners.users.add(self.scenario.user)
+        self.assertHasRole(hits_database=True)
+
+    def test_has_role_user_not_in_context(self) -> None:
+        """Test has_role with a different user than in context."""
         self.scenario.set_current()
         user = self.playground.create_user("test")
-        self.assertPass(
-            self.scenario.scope.context_has_role(user, Scope.Roles.OWNER)
-        )
+        self.assertHasNoRole(user=user, hits_database=True)
+        self.scenario.scope_owners.users.add(user)
+        self.assertHasRole(user=user, hits_database=True)
 
-    def test_context_has_role_wrong_scope(self) -> None:
-        """Test context_has_role with a different scope than in context."""
+    def test_has_role_scope_not_in_context(self) -> None:
+        """Test has_role with a different scope than in context."""
         self.scenario.set_current()
         scope = self.playground.get_or_create_scope("test")
-        self.assertPass(
-            scope.context_has_role(self.scenario.user, Scope.Roles.OWNER)
-        )
-
-    def test_context_has_role_no_roles(self) -> None:
-        """Test context_has_role with no roles."""
-        with self.assertRaisesRegex(
-            ValueError, r"context_has_role needs at least one role"
-        ):
-            self.scenario.scope.context_has_role(self.scenario.user, ())
-
-    def test_context_has_role(self) -> None:
-        """Test context_has_role."""
-        scope = self.scenario.scope
-        user = self.scenario.user
-
-        with context.local():
-            self.scenario.set_current()
-            self.assertDeny(scope.context_has_role(user, Scope.Roles.OWNER))
-            self.assertDeny(scope.context_has_role(user, [Scope.Roles.OWNER]))
-
+        self.assertHasNoRole(scope=scope, hits_database=True)
         self.playground.create_group_role(
             scope, Scope.Roles.OWNER, users=[self.scenario.user]
         )
+        self.assertHasRole(scope=scope, hits_database=True)
+
+    def test_has_role_from_context(self) -> None:
+        """Test has_role with information from context."""
+        with context.local():
+            self.scenario.set_current()
+            self.assertHasNoRole(hits_database=False)
+
+        self.scenario.scope_owners.users.add(self.scenario.user)
 
         with context.local():
             self.scenario.set_current()
-            self.assertAllow(scope.context_has_role(user, Scope.Roles.OWNER))
-            self.assertAllow(scope.context_has_role(user, [Scope.Roles.OWNER]))
+            self.assertHasRole(hits_database=False)
 
     def test_upload_file_stores(self) -> None:
         """`upload_file_stores` returns stores in the correct order."""
@@ -438,7 +517,7 @@ class ScopeTests(TestCase):
         scope = self.scenario.scope
         scope2 = Scope.objects.create(name="Scope2")
         for user in (AnonymousUser(), self.scenario.user):
-            with self.subTest(user=user):
+            with self.subTest(user=repr(user)):
                 self.assertTrue(scope.can_display(user))
                 self.assertTrue(scope2.can_display(user))
                 self.assertQuerySetEqual(
@@ -447,49 +526,14 @@ class ScopeTests(TestCase):
                     ordered=False,
                 )
 
-    def test_can_create_workspace_not_owner(self) -> None:
-        """Test can_create_workspace with user not owner."""
-        self.assertPermission(
+    def test_can_create_workspace(self) -> None:
+        self.assertRolePredicate(
+            self.scenario.scope,
             "can_create_workspace",
-            [AnonymousUser(), self.scenario.user],
-            denied=self.scenario.scope,
+            Scope.Roles.OWNER,
+            workers=Allow.NEVER,
         )
 
-    def test_predicate_deny_from_context(self) -> None:
-        """Test predicates propagating DENY from context_has_role."""
-        with mock.patch(
-            "debusine.db.models.scopes.Scope.context_has_role",
-            return_value=PartialCheckResult.DENY,
-        ):
-            self.assertFalse(
-                self.scenario.scope.can_create_workspace(self.scenario.user)
-            )
-
-    def test_can_create_workspace_owner(self) -> None:
-        """Test can_create_workspace with user owner of the scope."""
-        scope = self.scenario.scope
-        user = self.scenario.user
-        self.playground.create_group_role(
-            scope, Scope.Roles.OWNER, users=[user]
-        )
-        with self.assertNumQueries(1):
-            self.assertTrue(scope.can_create_workspace(user))
-        self.assertQuerySetEqual(
-            Scope.objects.can_create_workspace(user), [scope]
-        )
-
-    def test_can_create_workspace_owner_in_context(self) -> None:
-        """Test can_create_workspace with current user owner of the scope."""
-        scope = self.scenario.scope
-        user = self.scenario.user
-        self.playground.create_group_role(
-            scope, Scope.Roles.OWNER, users=[user]
-        )
-        self.scenario.set_current()
-        # Testing the current user can shortcut checking the current roles
-        with self.assertNumQueries(0):
-            self.assertTrue(scope.can_create_workspace(user))
-
 
 class ScopeRoleTests(TestCase):
     """Tests for the ScopeRole class."""
@@ -500,14 +544,13 @@ class ScopeRoleTests(TestCase):
     group2: ClassVar[Group]
 
     @classmethod
-    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up common test data."""
         super().setUpTestData()
         cls.scope1 = cls.playground.get_or_create_scope(name="Scope1")
         cls.scope2 = cls.playground.get_or_create_scope(name="Scope2")
-        cls.group1 = Group.objects.create(name="Group1", scope=cls.scope1)
-        cls.group2 = Group.objects.create(name="Group2", scope=cls.scope2)
+        cls.group1 = cls.playground.create_group("Group1", scope=cls.scope1)
+        cls.group2 = cls.playground.create_group("Group2", scope=cls.scope2)
 
     def test_str(self) -> None:
         """Test stringification."""
