@@ -299,8 +299,8 @@ class WorkerPoolTests(TestCase):
         """Terminate a worker retries a running work request."""
         worker_pool = self.playground.create_worker_pool()
         worker = self.playground.create_worker(worker_pool=worker_pool)
-        work_request_pending = self.playground.create_work_request()
-        work_request_running = self.playground.create_work_request()
+        work_request_pending = self.playground.create_worker_task()
+        work_request_running = self.playground.create_worker_task()
         work_request_running.assign_worker(worker)
         work_request_running.mark_running()
 
@@ -332,9 +332,10 @@ class WorkerPoolTests(TestCase):
         """If a running work request cannot be retried, it is logged."""
         worker_pool = self.playground.create_worker_pool()
         worker = self.playground.create_worker(worker_pool=worker_pool)
-        work_request_running = self.playground.create_work_request()
+        work_request_running = self.playground.create_worker_task()
         work_request_running.assign_worker(worker)
         work_request_running.task_data = {"invalid": True}
+        work_request_running.configured_task_data = {"invalid": True}
         work_request_running.save()
         work_request_running.mark_running()
 
