@@ -72,7 +72,7 @@ class WorkerManagerTests(TestCase):
         )
         workers["dynamic_connected_busy"].connected_at = timezone.now()
         workers["dynamic_connected_busy"].save()
-        wr = self.playground.create_work_request()
+        wr = self.playground.create_worker_task()
         wr.assign_worker(workers["dynamic_connected_busy"])
         wr.mark_running()
 
@@ -81,7 +81,7 @@ class WorkerManagerTests(TestCase):
         )
         workers["dynamic_connected_now_idle"].connected_at = timezone.now()
         workers["dynamic_connected_now_idle"].save()
-        wr = self.playground.create_work_request()
+        wr = self.playground.create_worker_task()
         wr.assign_worker(workers["dynamic_connected_now_idle"])
         wr.mark_completed(WorkRequest.Results.SUCCESS)
 
@@ -120,7 +120,7 @@ class WorkerManagerTests(TestCase):
         )
 
         # A task is assigned to the worker
-        work_request = self.playground.create_work_request(worker=worker)
+        work_request = self.playground.create_worker_task(worker=worker)
 
         # The worker is not ready (it is busy with a task assigned)
         self.assertQuerySetEqual(Worker.objects.waiting_for_work_request(), [])
@@ -157,13 +157,13 @@ class WorkerManagerTests(TestCase):
 
         # Assign two tasks to the worker; it can still accept more work
         for _ in range(2):
-            work_request = self.playground.create_work_request(worker=worker)
+            work_request = self.playground.create_worker_task(worker=worker)
             self.assertQuerySetEqual(
                 Worker.objects.waiting_for_work_request(), [worker]
             )
 
         # Assign a third task to the worker; it is now as busy as it allows
-        work_request = self.playground.create_work_request(worker=worker)
+        work_request = self.playground.create_worker_task(worker=worker)
         self.assertQuerySetEqual(Worker.objects.waiting_for_work_request(), [])
 
         # The third task finished
@@ -366,7 +366,7 @@ class WorkerManagerTests(TestCase):
         worker.registered_at = timezone.now() - datetime.timedelta(days=2)
         worker.instance_created_at = timezone.now() - datetime.timedelta(days=1)
         worker.save()
-        work_request = self.playground.create_work_request(worker=worker)
+        work_request = self.playground.create_worker_task(worker=worker)
         work_request.mark_running()
         worker.refresh_from_db()
         self.assertTrue(worker.is_busy())
@@ -384,7 +384,7 @@ class WorkerManagerTests(TestCase):
         worker.registered_at = timezone.now() - datetime.timedelta(days=2)
         worker.instance_created_at = timezone.now() - datetime.timedelta(days=1)
         worker.save()
-        work_request = self.playground.create_work_request(worker=worker)
+        work_request = self.playground.create_worker_task(worker=worker)
         work_request.mark_running()
         work_request.mark_completed(WorkRequest.Results.SUCCESS)
         work_request.completed_at = timezone.now() - datetime.timedelta(hours=6)
@@ -481,7 +481,7 @@ class WorkerTests(TestCase):
         )
         self.assertFalse(worker.is_busy())
 
-        work_request = self.playground.create_work_request(worker=worker)
+        work_request = self.playground.create_worker_task(worker=worker)
 
         self.assertEqual(work_request.status, WorkRequest.Statuses.PENDING)
         self.assertTrue(worker.is_busy())
@@ -506,7 +506,7 @@ class WorkerTests(TestCase):
         self.assertFalse(worker.is_at_capacity())
         for _ in range(3):
             self.assertFalse(worker.is_at_capacity())
-            work_request = self.playground.create_work_request(worker=worker)
+            work_request = self.playground.create_worker_task(worker=worker)
             work_request.assign_worker(worker)
         self.assertTrue(worker.is_at_capacity())
         work_request.mark_aborted()
@@ -520,14 +520,14 @@ class WorkerTests(TestCase):
 
         worker_2 = self.playground.create_worker()
 
-        work_request_1 = self.playground.create_work_request(
+        work_request_1 = self.playground.create_worker_task(
             worker=worker_1, mark_running=True
         )
-        work_request_2 = self.playground.create_work_request(
+        work_request_2 = self.playground.create_worker_task(
             worker=worker_1, mark_running=True
         )
-        self.playground.create_work_request(worker=worker_1, mark_running=False)
-        self.playground.create_work_request(worker=worker_2, mark_running=True)
+        self.playground.create_worker_task(worker=worker_1, mark_running=False)
+        self.playground.create_worker_task(worker=worker_2, mark_running=True)
 
         self.assertQuerySetEqual(
             worker_1.running_work_requests(), [work_request_1, work_request_2]
