@@ -100,7 +100,7 @@ class WorkspaceRolesTests(RolesTestCase[WorkspaceRoles]):
                 [Workspace.Roles.CONTRIBUTOR],
             ),
         ):
-            with self.subTest(value=value):
+            with self.subTest(value=repr(value)):
                 self.assertEqual(
                     Workspace.Roles.from_iterable(value),
                     frozenset(expected),
@@ -153,7 +153,7 @@ class WorkspaceRolesTests(RolesTestCase[WorkspaceRoles]):
             (WorkspaceRoles.CONTRIBUTOR, False),
             (WorkspaceRoles.VIEWER, True),
         ):
-            with self.subTest(role=role):
+            with self.subTest(role=repr(role)):
                 expected_qs = [self.scenario.workspace] if expected else []
                 self.assertQuerySetEqual(
                     Workspace.objects.filter(
@@ -394,7 +394,9 @@ class WorkspaceManagerTests(TestCase):
             ),
         ):
             with (
-                self.subTest(user=user, roles=roles, role=role),
+                self.subTest(
+                    user=repr(user), roles=repr(roles), role=repr(role)
+                ),
                 self.scenario.assign_role(self.scenario.workspace, *roles),
             ):
                 results = list(Workspace.objects.with_role(user, role))
@@ -450,7 +452,9 @@ class WorkspaceManagerTests(TestCase):
             ),
         ):
             with (
-                self.subTest(user=user, roles=roles, role=role),
+                self.subTest(
+                    user=repr(user), roles=repr(roles), role=repr(role)
+                ),
                 self.scenario.assign_role(self.scenario.workspace, *roles),
             ):
                 results = list(Workspace.objects.with_role(user, role))
@@ -672,6 +676,13 @@ class WorkspaceTests(TestCase):
             f"/{settings.DEBUSINE_DEFAULT_SCOPE}/System/configure/",
         )
 
+    def test_get_absolute_url_configure_inheritance(self) -> None:
+        """Test the get_absolute_url_configure_inheritance method."""
+        self.assertEqual(
+            self.scenario.workspace.get_absolute_url_configure_inheritance(),
+            f"/{settings.DEBUSINE_DEFAULT_SCOPE}/System/inheritance/",
+        )
+
     def test_get_absolute_url_configure_different_scope(self) -> None:
         """get_absolute_url_configure works in another scope."""
         other_scope = self.playground.get_or_create_scope(name="other")
@@ -833,9 +844,9 @@ class WorkspaceTests(TestCase):
             for role in roles:
                 with (
                     self.subTest(
-                        scope_roles=scope_roles,
-                        workspace_roles=workspace_roles,
-                        role=role,
+                        scope_roles=repr(scope_roles),
+                        workspace_roles=repr(workspace_roles),
+                        role=repr(role),
                     ),
                     context.local(),
                 ):
@@ -903,7 +914,7 @@ class WorkspaceTests(TestCase):
                 actual: dict[int, frozenset[WorkspaceRoles]] = {}
                 for workspace in queryset:
                     with (
-                        self.subTest(workspace=workspace),
+                        self.subTest(workspace=repr(workspace)),
                         self.assertNumQueries(num_queries[workspace.pk]),
                     ):
                         actual[workspace.pk] = workspace.get_roles(
@@ -1374,7 +1385,7 @@ class WorkspaceTests(TestCase):
         context.set_scope(self.scenario.scope)
         context.set_user(self.scenario.user)
         for role in Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR:
-            with self.subTest(role=role), context.local():
+            with self.subTest(role=repr(role)), context.local():
                 setattr(workspace, "user_roles", [role])
                 with self.assertNumQueries(0):
                     workspace.set_current()
@@ -1417,7 +1428,10 @@ class WorkspaceTests(TestCase):
                 [Workspace.Roles.OWNER],
             ),
         ):
-            with self.subTest(public=public, roles=roles), context.local():
+            with (
+                self.subTest(public=public, roles=repr(roles)),
+                context.local(),
+            ):
                 self.scenario.workspace.public = public
                 self.scenario.workspace.save()
                 Group.objects.all().delete()
@@ -1470,7 +1484,7 @@ class WorkspaceTests(TestCase):
             (user, [Workspace.Roles.OWNER], True),
         ):
             with (
-                self.subTest(user=u, roles=roles),
+                self.subTest(user=repr(u), roles=repr(roles)),
                 self.playground.assign_role(self.scenario.workspace, u, *roles),
             ):
                 self.assertPermissionPredicate(
@@ -1490,7 +1504,7 @@ class WorkspaceTests(TestCase):
             (user, [Workspace.Roles.OWNER], True),
         ):
             with (
-                self.subTest(user=u, roles=roles),
+                self.subTest(user=repr(u), roles=repr(roles)),
                 self.playground.assign_role(self.scenario.workspace, u, *roles),
             ):
                 self.assertPermissionPredicate(
@@ -1611,7 +1625,7 @@ class WorkspaceTests(TestCase):
         """Test with_expiration_time with only created_at."""
         self.a.expiration_delay = datetime.timedelta(days=3)
         self.a.save()
-        wr = self.playground.create_work_request(
+        wr = self.playground.create_worker_task(
             result=WorkRequest.Results.SUCCESS, workspace=self.a
         )
         wr.completed_at = self.a.created_at + datetime.timedelta(days=42)
@@ -1738,7 +1752,7 @@ class WorkspaceTests(TestCase):
                 ],
             ),
         ):
-            with self.subTest(user=user, workspace=workspace):
+            with self.subTest(user=repr(user), workspace=repr(workspace)):
                 result = list(
                     workspace.list_accessible_collections(
                         user or AnonymousUser(),
@@ -1919,7 +1933,7 @@ class WorkspaceDeleteTests(TestCase):
             workflow = self.playground.create_workflow(
                 task_name=template, workspace=workspace
             )
-            wr = workflow.create_child(
+            wr = workflow.create_child_worker(
                 "noop", status=WorkRequest.Statuses.PENDING
             )
             wr.assign_worker(self.playground.create_worker())
@@ -1959,7 +1973,9 @@ class WorkspaceDeleteTests(TestCase):
         workflow = self.playground.create_workflow(
             task_name="noop", workspace=experiment_workspace
         )
-        wr = workflow.create_child("noop", status=WorkRequest.Statuses.PENDING)
+        wr = workflow.create_child_worker(
+            "noop", status=WorkRequest.Statuses.PENDING
+        )
         wr.add_event_reaction(
             "on_success",
             ActionUpdateCollectionWithArtifacts(
