@@ -493,6 +493,15 @@ class Workspace(models.Model):
         with urlconf_scope(self.scope.name):
             return reverse("workspaces:update", kwargs={"wname": self.name})
 
+    def get_absolute_url_configure_inheritance(self) -> str:
+        """Return an absolute URL to configure the inheritance chain."""
+        from debusine.server.scopes import urlconf_scope
+
+        with urlconf_scope(self.scope.name):
+            return reverse(
+                "workspaces:update_inheritance", kwargs={"wname": self.name}
+            )
+
     @copy_signature_from(models.Model.save)
     def save(self, **kwargs: Any) -> None:
         """Wrap save with permission checks."""
