@@ -10,15 +10,16 @@
 """Infrastructure to create test scenarios in the database."""
 
 import contextlib
+import datetime as dt
 import tempfile
 import textwrap
 import time
 from collections.abc import Generator
 from dataclasses import dataclass
-from datetime import timedelta
 from pathlib import Path
 from secrets import token_hex
 from typing import Any, Literal, TYPE_CHECKING, assert_never, overload
+from unittest import mock
 
 from django.conf import settings
 from django.contrib.auth.models import Permission
@@ -73,6 +74,7 @@ from debusine.db.models import (
     Scope,
     Token,
     User,
+    WellKnownWorkflowTemplate,
     WorkRequest,
     Worker,
     WorkerPool,
@@ -472,8 +474,8 @@ class Playground(ArtifactPlayground):
         parent: WorkRequest | None = None,
         status: WorkRequest.Statuses | None = None,
         created_by: User | None = None,
-        validate: bool = True,
         workspace: Workspace | None = None,
+        validate: bool = True,
     ) -> WorkRequest:
         """Create a workflow."""
         if created_by is None:
@@ -482,11 +484,18 @@ class Playground(ArtifactPlayground):
         if isinstance(task_name, WorkflowTemplate):
             template = task_name
         else:
-            workflow_template_name = f"{task_name}-template"
+            workflow_template_name = getattr(
+                WellKnownWorkflowTemplate,
+                task_name.upper(),
+                f"{task_name}-template",
+            )
+            templates = WorkflowTemplate.objects.filter(
+                name=workflow_template_name
+            )
+            if workspace is not None:
+                templates = templates.filter(workspace=workspace)
             try:
-                template = WorkflowTemplate.objects.get(
-                    name=workflow_template_name
-                )
+                template = templates.get()
             except WorkflowTemplate.DoesNotExist:
                 template = self.create_workflow_template(
                     name=workflow_template_name,
@@ -499,22 +508,31 @@ class Playground(ArtifactPlayground):
                 task_data, exclude_unset=True
             )
 
-        with context.local():
+        with contextlib.ExitStack() as stack:
+            stack.enter_context(context.local())
             context.reset()
             context.set_scope(template.workspace.scope)
             context.set_user(created_by)
             template.workspace.set_current()
-            with context.disable_permission_checks():
-                return WorkRequest.objects.create_workflow(
-                    template=template,
-                    data=task_data or {},
-                    parent=parent,
-                    status=status,
-                    validate=validate,
+            stack.enter_context(context.disable_permission_checks())
+            if not validate:
+                stack.enter_context(
+                    mock.patch(
+                        "debusine.db.models.WorkRequest._validate_workflow"
+                    )
                 )
+                stack.enter_context(
+                    mock.patch("debusine.db.models.WorkRequest.full_clean")
+                )
+            return WorkRequest.objects.create_workflow(
+                template=template,
+                data=task_data or {},
+                parent=parent,
+                status=status,
+            )
 
     @context.disable_permission_checks()
-    def create_work_request(
+    def create_worker_task(
         self,
         mark_running: bool = False,
         assign_new_worker: bool = False,
@@ -524,10 +542,12 @@ class Playground(ArtifactPlayground):
         task_name: str = "noop",
         task_data: BaseTaskData | dict[str, Any] | None = None,
         assign_contributor_role: bool = False,
+        completed_at: dt.datetime | None = None,
+        validate: bool = True,
         **kwargs: Any,
     ) -> WorkRequest:
         """
-        Return a new instance of WorkRequest.
+        Return a new instance of WorkRequest, for a worker task.
 
         :param mark_running: if True call mark_running() method
         :param assign_new_worker: if True assign worker to the work request
@@ -536,8 +556,11 @@ class Playground(ArtifactPlayground):
           (created_at: 1 year ago, expiration_delay: 1 day)
         :param assign_contributor_role: assign the ``CONTRIBUTOR`` role on
           the workspace to the creator of the work request
+        :param completed_at: if present, set completed_at after creation
         :param kwargs: use them when creating the WorkRequest model
         """
+        assert "task_type" not in kwargs
+
         # FileStore and Workspace are usually created by the migrations
         # When using a TransactionTestCase with async methods
         # the tests don't have access to the created FileStore / Workspace
@@ -550,26 +573,26 @@ class Playground(ArtifactPlayground):
         defaults: dict[str, Any] = {
             "task_name": task_name,
             "workspace": workspace,
+            "task_data": task_data,
         }
         if "created_by" not in kwargs:
             defaults["created_by"] = self.get_default_user()
         if expired:
             defaults.update(
-                created_at=timezone.now() - timedelta(days=365),
-                expiration_delay=timedelta(days=1),
+                created_at=timezone.now() - dt.timedelta(days=365),
+                expiration_delay=dt.timedelta(days=1),
             )
-        if isinstance(task_data, BaseTaskData):
-            defaults["task_data"] = model_to_json_serializable_dict(
-                task_data, exclude_unset=True
-            )
-        else:
-            defaults["task_data"] = task_data or {}
 
         defaults.update(kwargs)
 
         created_at = defaults.pop("created_at", None)
 
-        work_request = WorkRequest.objects.create(**defaults)
+        with contextlib.ExitStack() as stack:
+            if not validate:
+                stack.enter_context(
+                    mock.patch("debusine.db.models.WorkRequest.full_clean")
+                )
+            work_request = WorkRequest.objects.create_worker(**defaults)
 
         # created_at is set to now automatically by create via auto_now_add, so
         # if a specific value was requested we need to set it after creation
@@ -593,12 +616,139 @@ class Playground(ArtifactPlayground):
         if result is not None:
             work_request.mark_completed(result)
 
-        if completed_at := kwargs.get("completed_at"):
+        if completed_at is not None:
             work_request.completed_at = completed_at
             work_request.save()
 
         return work_request
 
+    @context.disable_permission_checks()
+    def create_server_task(
+        self,
+        task_name: str = "servernoop",
+        task_data: BaseTaskData | dict[str, Any] | None = None,
+    ) -> WorkRequest:
+        """Return a new instance of WorkRequest, for a server task."""
+        return WorkRequest.objects.create_server(
+            task_name=task_name,
+            task_data=task_data,
+            created_by=self.get_default_user(),
+            workspace=self.create_workspace(public=True),
+        )
+
+    @context.disable_permission_checks()
+    def create_internal_task(
+        self,
+        task_name: str = "noop",
+        status: WorkRequest.Statuses = WorkRequest.Statuses.PENDING,
+        validate: bool = True,
+    ) -> WorkRequest:
+        """Return a new instance of WorkRequest, for an internal task."""
+        with contextlib.ExitStack() as stack:
+            if not validate:
+                stack.enter_context(
+                    mock.patch("debusine.db.models.WorkRequest.full_clean")
+                )
+            return WorkRequest.objects.create_internal(
+                task_name=task_name,
+                workspace=self.create_workspace(public=True),
+                created_by=self.get_default_user(),
+                status=status,
+            )
+
+    @context.disable_permission_checks()
+    def create_signing_task(
+        self,
+        mark_running: bool = False,
+        workspace: Workspace | None = None,
+        task_name: str = "noop",
+        task_data: BaseTaskData | dict[str, Any] | None = None,
+        validate: bool = True,
+        **kwargs: Any,
+    ) -> WorkRequest:
+        """
+        Return a new instance of WorkRequest, for a signing task.
+
+        :param mark_running: if True call mark_running() method
+        :param kwargs: use them when creating the WorkRequest model
+        """
+        assert "task_type" not in kwargs
+
+        # FileStore and Workspace are usually created by the migrations
+        # When using a TransactionTestCase with async methods
+        # the tests don't have access to the created FileStore / Workspace
+        # yet (need verification, but they seem to be in a non-committed
+        # transaction and the test code is in a different thread because
+        # of the implementation of database_sync_to_async
+        if workspace is None:
+            workspace = self.create_workspace(public=True)
+
+        defaults: dict[str, Any] = {
+            "task_name": task_name,
+            "workspace": workspace,
+        }
+        if "created_by" not in kwargs:
+            defaults["created_by"] = self.get_default_user()
+        if isinstance(task_data, BaseTaskData):
+            defaults["task_data"] = model_to_json_serializable_dict(
+                task_data, exclude_unset=True
+            )
+        else:
+            defaults["task_data"] = task_data or {}
+
+        defaults.update(kwargs)
+
+        with contextlib.ExitStack() as stack:
+            if not validate:
+                stack.enter_context(
+                    mock.patch("debusine.db.models.WorkRequest.full_clean")
+                )
+            work_request = WorkRequest.objects.create_signing(**defaults)
+
+        if mark_running:
+            work_request.mark_running()
+
+        return work_request
+
+    @context.disable_permission_checks()
+    def create_wait_task(
+        self,
+        *,
+        task_name: str = "noop",
+        task_data: BaseTaskData | dict[str, Any] | None = None,
+        mark_running: bool = False,
+        validate: bool = True,
+        **kwargs: Any,
+    ) -> WorkRequest:
+        """
+        Return a new instance of WorkRequest, for a wait task.
+
+        :param mark_running: if True call mark_running() method
+        :param kwargs: use them when creating the WorkRequest model
+        """
+        assert "task_type" not in kwargs
+
+        defaults: dict[str, Any] = {
+            "task_name": task_name,
+            "workspace": self.create_workspace(public=True),
+            "task_data": task_data,
+        }
+        if "created_by" not in kwargs:
+            defaults["created_by"] = self.get_default_user()
+        defaults.update(kwargs)
+
+        with contextlib.ExitStack() as stack:
+            if not validate:
+                stack.enter_context(
+                    mock.patch("debusine.db.models.WorkRequest.full_clean")
+                )
+            work_request = WorkRequest.objects.create_wait(**defaults)
+
+        if mark_running:
+            work_request.mark_running()
+
+        return work_request
+
     @context.disable_permission_checks()
     def create_worker(
         self,
@@ -805,7 +955,7 @@ class Playground(ArtifactPlayground):
             workspace=workspace,
             data=raw_data,
             expiration_delay=(
-                timedelta(expiration_delay)
+                dt.timedelta(expiration_delay)
                 if expiration_delay is not None
                 else None
             ),
@@ -988,12 +1138,15 @@ class Playground(ArtifactPlayground):
     def create_upload_artifacts(
         self,
         *,
+        source: Literal[True] = True,
         src_name: str = ...,
         version: str = ...,
-        source: Literal[True] = True,
+        src_architectures: set[str] | None = ...,
         binary: Literal[False],
         binaries: list[str] | None = ...,
         bin_architecture: str = ...,
+        section: str = ...,
+        priority: str = ...,
         workspace: Workspace | None = ...,
         work_request: WorkRequest | None = None,
         created_by: User | None = ...,
@@ -1004,12 +1157,15 @@ class Playground(ArtifactPlayground):
     def create_upload_artifacts(
         self,
         *,
+        source: Literal[False],
         src_name: str = ...,
         version: str = ...,
-        source: Literal[False],
+        src_architectures: set[str] | None = ...,
         binary: Literal[True] = True,
         binaries: list[str] | None = ...,
         bin_architecture: str = ...,
+        section: str = ...,
+        priority: str = ...,
         workspace: Workspace | None = ...,
         work_request: WorkRequest | None = None,
         created_by: User | None = ...,
@@ -1020,12 +1176,15 @@ class Playground(ArtifactPlayground):
     def create_upload_artifacts(
         self,
         *,
+        source: Literal[True] = True,
         src_name: str = ...,
         version: str = ...,
-        source: Literal[True] = True,
+        src_architectures: set[str] | None = ...,
         binary: Literal[True] = True,
         binaries: list[str] | None = ...,
         bin_architecture: str = ...,
+        section: str = ...,
+        priority: str = ...,
         workspace: Workspace | None = ...,
         work_request: WorkRequest | None = None,
         created_by: User | None = ...,
@@ -1036,12 +1195,15 @@ class Playground(ArtifactPlayground):
     def create_upload_artifacts(
         self,
         *,
+        source: bool = True,
         src_name: str = "hello",
         version: str = "1.0-1",
-        source: bool = True,
+        src_architectures: set[str] | None = None,
         binary: bool = True,
         binaries: list[str] | None = None,
         bin_architecture: str = "all",
+        section: str = "unknown",
+        priority: str = "optional",
         workspace: Workspace | None = None,
         work_request: WorkRequest | None = None,
         created_by: User | None = None,
@@ -1059,12 +1221,15 @@ class Playground(ArtifactPlayground):
             workdir = Path(tempdir)
             upload = self.create_upload(
                 workdir,
+                source=source,
                 src_name=src_name,
                 version=version,
-                source=source,
+                src_architectures=src_architectures,
                 binary=binary,
                 binaries=binaries,
                 bin_architecture=bin_architecture,
+                section=section,
+                priority=priority,
             )
             local_artifacts["upload"] = [upload]
             if source:
@@ -1223,7 +1388,7 @@ class Playground(ArtifactPlayground):
 
     def create_signing_key_asset(
         self,
-        purpose: KeyPurpose = KeyPurpose.OPENPGP,
+        purpose: KeyPurpose = KeyPurpose.OPENPGP_UPLOAD,
         fingerprint: str | None = None,
         public_key: str | None = None,
         description: str | None = "Test Key",
@@ -1451,7 +1616,7 @@ class Playground(ArtifactPlayground):
             ],
         )
         if workflow:
-            return workflow.create_child(
+            return workflow.create_child_worker(
                 task_name="sbuild",
                 task_data=task_data,
                 workflow_data=WorkRequestWorkflowData(
@@ -1464,7 +1629,7 @@ class Playground(ArtifactPlayground):
                 "For an Sbuild task (not workflow): host_architecture must "
                 "be a real architecture (e.g. amd64)"
             )
-            return self.create_work_request(
+            return self.create_worker_task(
                 task_name="sbuild",
                 task_data=task_data,
                 workspace=workspace or self.get_default_workspace(),
