@@ -143,11 +143,19 @@ class DefaultContext(DefaultScopeUser):
     """
 
     workspace: Workspace
+    default_task_configuration_collection: Collection
 
     def build(self, playground: "Playground") -> None:
         """Build the scenario."""
         super().build(playground)
         self.workspace = self.playground.get_default_workspace()
+        self.default_task_configuration_collection, _ = (
+            Collection.objects.get_or_create(
+                workspace=self.workspace,
+                name="default",
+                category=CollectionCategory.TASK_CONFIGURATION,
+            )
+        )
 
     def set_current(self) -> None:
         """Set the current user and workspace from the scenario."""
@@ -326,7 +334,7 @@ class UIPlayground(DefaultContext):
         self.add_results_to_suite(wr)
 
         # Create a variety of work requests
-        wr = self.playground.create_work_request(
+        wr = self.playground.create_worker_task(
             status=WorkRequest.Statuses.COMPLETED,
             result=WorkRequest.Results.SUCCESS,
             task_name="noop",
@@ -334,22 +342,22 @@ class UIPlayground(DefaultContext):
         wr.task_data = {"result": False}
         wr.configured_task_data = {"result": True}
         wr.save()
-        self.playground.create_work_request(
+        self.playground.create_worker_task(
             status=WorkRequest.Statuses.COMPLETED,
             result=WorkRequest.Results.FAILURE,
             task_name="noop",
         )
-        self.playground.create_work_request(
+        self.playground.create_worker_task(
             status=WorkRequest.Statuses.COMPLETED,
             result=WorkRequest.Results.ERROR,
             task_name="noop",
         )
-        self.playground.create_work_request(
+        self.playground.create_worker_task(
             status=WorkRequest.Statuses.ABORTED,
             task_name="noop",
         )
 
-        wr = self.playground.create_work_request(task_name="noop")
+        wr = self.playground.create_worker_task(task_name="noop")
         wr.assign_worker(self.worker_in_pool_busy)
         wr.mark_running()
 
@@ -477,16 +485,7 @@ class UIPlayground(DefaultContext):
         workflow_needs_input.save()
 
         # Create and populate the default task-configuration collection
-        try:
-            task_config_collection = self.workspace.collections.get(
-                name="default", category=CollectionCategory.TASK_CONFIGURATION
-            )
-        except Collection.DoesNotExist:
-            task_config_collection = self.playground.create_collection(
-                "default",
-                CollectionCategory.TASK_CONFIGURATION,
-                workspace=self.workspace,
-            )
+        task_config_collection = self.default_task_configuration_collection
         task_config_collection.manager.add_bare_data(
             BareDataCategory.TASK_CONFIGURATION,
             user=self.user,
