@@ -456,7 +456,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         user = User.objects.create_user(
             username="custom", email="custom@example.org"
         )
-        work_request = playground.create_work_request(created_by=user)
+        work_request = playground.create_worker_task(created_by=user)
         buildlog = playground.create_build_log_artifact(
             source="test",
             version="2.0",
@@ -487,7 +487,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         user = User.objects.create_user(
             username="custom", email="custom@example.org"
         )
-        work_request = playground.create_work_request()
+        work_request = playground.create_worker_task()
         buildlog = playground.create_build_log_artifact(
             work_request=work_request,
             created_by=user,
@@ -541,7 +541,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertIsNone(asset.created_by_work_request)
         data_model = asset.data_model
         assert isinstance(data_model, SigningKeyData)
-        self.assertEqual(data_model.purpose, KeyPurpose.OPENPGP)
+        self.assertEqual(data_model.purpose, KeyPurpose.OPENPGP_UPLOAD)
         self.assertGreater(len(data_model.fingerprint), 8)
         self.assertTrue(
             data_model.public_key.startswith(
@@ -884,6 +884,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                     f"Debusine <noreply@{settings.DEBUSINE_FQDN}>"
                 ),
                 subject="hello",
+                parameter_summary="hello_1.0-1",
                 runtime_context="any:amd64:amd64",
                 configuration_context="bookworm",
                 task_configuration_id=task_config_collection.id,
@@ -927,6 +928,8 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                     'Description': 'Example description',
                     'Maintainer': 'Example Maintainer <example@example.org>',
                     'Package': 'hello',
+                    'Priority': 'optional',
+                    'Section': 'unknown',
                     'Version': '1.0-1',
                 },
                 'srcpkg_name': 'hello',
@@ -1034,6 +1037,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                     f"Debusine <noreply@{settings.DEBUSINE_FQDN}>"
                 ),
                 subject="test",
+                parameter_summary="test_2.0",
                 runtime_context="any:amd64:amd64",
                 configuration_context="bookworm",
             ),
@@ -1070,6 +1074,8 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                     'Description': 'Example description',
                     'Maintainer': 'Example Maintainer <example@example.org>',
                     'Package': 'test',
+                    'Priority': 'optional',
+                    'Section': 'unknown',
                     'Version': '2.0',
                 },
                 'srcpkg_name': 'test',
@@ -1119,9 +1125,6 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(
             wr.event_reactions_json,
             {
-                "on_creation": [],
-                "on_unblock": [],
-                "on_assignment": [],
                 "on_success": [
                     {
                         "action": ActionTypes.UPDATE_COLLECTION_WITH_ARTIFACTS,
@@ -1129,7 +1132,6 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                             "category": ArtifactCategory.UPLOAD
                         },
                         "collection": "internal@collections",
-                        "created_at": None,
                         "name_template": "build-all",
                         "variables": {
                             "architecture": "all",
@@ -1138,7 +1140,6 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                         },
                     }
                 ],
-                "on_failure": [],
             },
         )
 
@@ -1152,7 +1153,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
     def test_create_work_request(self) -> None:
         """Test create_work_request return a saved work request."""
         playground = Playground()
-        work_request = playground.create_work_request()
+        work_request = playground.create_worker_task()
         work_request.refresh_from_db()
         self.assertIsInstance(work_request, WorkRequest)
         self.assertEqual(
@@ -1163,14 +1164,14 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         """Test create_work_request use created_by Token."""
         playground = Playground()
         user = playground.create_user("testuser")
-        work_request = playground.create_work_request(created_by=user)
+        work_request = playground.create_worker_task(created_by=user)
         work_request.refresh_from_db()
         self.assertEqual(work_request.created_by, user)
 
     def test_create_work_request_expired(self) -> None:
         """Test create_work_request expired=True."""
         playground = Playground()
-        work_request = playground.create_work_request(expired=True)
+        work_request = playground.create_worker_task(expired=True)
 
         self.assertIsNotNone(work_request.expire_at)
         assert work_request.expire_at is not None
