@@ -16,6 +16,7 @@ from django.contrib.auth.hashers import check_password
 from debusine.db.context import context
 from debusine.db.models import Scope, Workspace
 from debusine.db.playground import scenarios
+from debusine.db.playground.playground import Playground
 from debusine.test.django import TestCase
 from debusine.utils import extract_generic_type_arguments
 
@@ -85,6 +86,33 @@ class DefaultScopeUserTestBase(ScenarioTestBase[DSU]):
             Scope.Roles.OWNER,
         )
 
+    def test_assign_role(self) -> None:
+        self.assertQuerySetEqual(self.scenario.scope.roles.all(), [])
+        with self.scenario.assign_role(self.scenario.scope, Scope.Roles.OWNER):
+            assignment = self.scenario.scope.roles.get()
+            self.assertEqual(assignment.resource, self.scenario.scope)
+            self.assertEqual(assignment.role, Scope.Roles.OWNER)
+            self.assertQuerySetEqual(
+                assignment.group.users.all(), [self.scenario.user]
+            )
+        self.assertQuerySetEqual(self.scenario.scope.roles.all(), [])
+
+    def test_build_with_different_defaults(self) -> None:
+        other_playground = Playground(
+            default_workspace_name="other", default_username="other"
+        )
+        self.other_scenario = self.scenario_cls()
+        other_playground.build_scenario(self.other_scenario)
+
+        def cleanup() -> None:
+            del self.other_scenario
+
+        self.addCleanup(cleanup)
+
+        # The scope name is not yet customizable in the playground
+        self.assertEqual(self.other_scenario.scope.name, "debusine")
+        self.assertEqual(self.other_scenario.user.username, "other")
+
 
 class DefaultScopeUserTest(
     DefaultScopeUserTestBase[scenarios.DefaultScopeUser]
@@ -134,6 +162,14 @@ class DefaultContextTestBase(DefaultScopeUserTestBase[DC]):
             Workspace.Roles.OWNER,
         )
 
+    def test_build_with_different_defaults(self) -> None:
+        super().test_build_with_different_defaults()
+        self.assertEqual(self.other_scenario.workspace.name, "other")
+        self.assertEqual(
+            self.other_scenario.default_task_configuration_collection.workspace,
+            self.other_scenario.workspace,
+        )
+
 
 class DefaultContextTest(DefaultContextTestBase[scenarios.DefaultContext]):
     """Test DefaultContext scenario."""
