@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -27,7 +27,7 @@ _migration_commands = (
 
 def is_migration_command() -> bool:
     """Return True if a migration-related command is being run."""
-    return sys.argv[1] in _migration_commands
+    return len(sys.argv) > 1 and sys.argv[1] in _migration_commands
 
 
 def all_migrations_applied(admin_command: str, **kwargs: Any) -> list[Error]:
