@@ -8,12 +8,13 @@
 # contained in the LICENSE file.
 
 """Django Application Configuration for the server application."""
+
+import asyncio
 import os
+from concurrent.futures import ThreadPoolExecutor
 
 from django.apps import AppConfig
 
-from debusine.tasks.models import WorkerType
-
 
 class ServerConfig(AppConfig):
     """Django's AppConfig for the server application."""
@@ -47,9 +48,19 @@ class ServerConfig(AppConfig):
         #
         # The reason of setting DEBUSINE_WORKER_MANAGER is to avoid
         # running the disconnect when a debusine sysadmin runs
-        # commands such as "debusine-admin list_workers"
+        # commands such as "debusine-admin worker list"
 
         if os.environ.get("DEBUSINE_WORKER_MANAGER", "0") != "0":
-            for worker in Worker.objects.connected():
-                if worker.worker_type != WorkerType.CELERY:
-                    worker.mark_disconnected()
+            try:
+                asyncio.get_running_loop()
+            except RuntimeError:
+                Worker.objects.mark_all_disconnected()
+            else:
+                # Run this in a separate thread so that it can use the
+                # synchronous ORM, and wait for it to finish.  We can't use
+                # a more conventional async style because AppConfig.ready
+                # has no asynchronous version, even if it's being called as
+                # part of an ASGI server.
+                ThreadPoolExecutor(max_workers=1).submit(
+                    Worker.objects.mark_all_disconnected
+                ).result()
