@@ -9,6 +9,7 @@
 
 """The collection manager for debian:package-build-logs collections."""
 
+from datetime import datetime
 from typing import Any
 
 from django.contrib.auth.models import AnonymousUser
@@ -54,6 +55,8 @@ class DebianPackageBuildLogsManager(CollectionManagerInterface):
         data: dict[str, Any] | None = None,
         name: str | None = None,  # noqa: U100
         replace: bool = False,
+        created_at: datetime | None = None,
+        replaced_by: CollectionItem | None = None,
     ) -> CollectionItem:
         """Add bare data into the managed collection."""
         if data is None:
@@ -80,7 +83,12 @@ class DebianPackageBuildLogsManager(CollectionManagerInterface):
         name = "_".join(name_elements)
 
         if replace:
-            self.remove_bare_data(name, user=user, workflow=workflow)
+            self.remove_items_by_name(
+                name=name,
+                child_types=[CollectionItem.Types.BARE],
+                user=user,
+                workflow=workflow,
+            )
 
         try:
             return CollectionItem.objects.create_from_bare_data(
@@ -88,30 +96,14 @@ class DebianPackageBuildLogsManager(CollectionManagerInterface):
                 parent_collection=self.collection,
                 name=name,
                 data=data,
+                created_at=created_at,
                 created_by_user=user,
                 created_by_workflow=workflow,
+                replaced_by=replaced_by,
             )
         except IntegrityError as exc:
             raise ItemAdditionError(str(exc))
 
-    def do_remove_bare_data(
-        self,
-        name: str,
-        *,
-        user: User | None = None,
-        workflow: WorkRequest | None = None,
-    ) -> None:
-        """Remove a bare data item from the collection."""
-        CollectionItem.active_objects.filter(
-            name=name,
-            child_type=CollectionItem.Types.BARE,
-            parent_collection=self.collection,
-        ).update(
-            removed_by_user=user,
-            removed_by_workflow=workflow,
-            removed_at=timezone.now(),
-        )
-
     def do_add_artifact(
         self,
         artifact: Artifact,
@@ -121,6 +113,8 @@ class DebianPackageBuildLogsManager(CollectionManagerInterface):
         variables: dict[str, Any] | None = None,
         name: str | None = None,  # noqa: U100
         replace: bool = False,
+        created_at: datetime | None = None,
+        replaced_by: CollectionItem | None = None,
     ) -> CollectionItem:
         """Add the artifact into the managed collection."""
         if variables is None:
@@ -166,9 +160,12 @@ class DebianPackageBuildLogsManager(CollectionManagerInterface):
         name = "_".join(name_elements)
 
         if replace:
-            self.do_remove_child_types(
-                [CollectionItem.Types.BARE, CollectionItem.Types.ARTIFACT],
+            self.remove_items_by_name(
                 name=name,
+                child_types=[
+                    CollectionItem.Types.BARE,
+                    CollectionItem.Types.ARTIFACT,
+                ],
                 user=user,
                 workflow=workflow,
             )
@@ -179,27 +176,26 @@ class DebianPackageBuildLogsManager(CollectionManagerInterface):
                 parent_collection=self.collection,
                 name=name,
                 data=data,
+                created_at=created_at,
                 created_by_user=user,
                 created_by_workflow=workflow,
+                replaced_by=replaced_by,
             )
         except IntegrityError as exc:
             raise ItemAdditionError(str(exc))
 
-    def do_remove_artifact(
+    def do_remove_item(
         self,
-        artifact: Artifact,
+        item: CollectionItem,
         *,
         user: User | None = None,
         workflow: WorkRequest | None = None,
     ) -> None:
-        """Remove the artifact from the collection."""
-        CollectionItem.objects.filter(
-            artifact=artifact, parent_collection=self.collection
-        ).update(
-            removed_by_user=user,
-            removed_by_workflow=workflow,
-            removed_at=timezone.now(),
-        )
+        """Remove an item from the collection."""
+        item.removed_by_user = user
+        item.removed_by_workflow = workflow
+        item.removed_at = timezone.now()
+        item.save()
 
     def do_lookup_filter(
         self,
