@@ -74,6 +74,22 @@ class DebianSuiteManager(CollectionManagerInterface):
         }
     )
 
+    def get_signing_keys(self) -> list[str] | None:
+        """Return signing keys for this suite."""
+        if self.collection.data.get("signing_keys") is not None:
+            signing_keys = self.collection.data["signing_keys"]
+        else:
+            try:
+                archive = self.collection.parent_collections.get(
+                    category=CollectionCategory.ARCHIVE
+                )
+            except Collection.DoesNotExist:
+                return None
+            else:
+                signing_keys = archive.data.get("signing_keys")
+        assert signing_keys is None or isinstance(signing_keys, list)
+        return signing_keys
+
     def make_constraints(
         self, item: CollectionItem, source_package_name: str, component: str
     ) -> Generator[CollectionItemMatchConstraint, None, None]:
