@@ -93,6 +93,25 @@ class LookupResultCollection(LookupResult):
     collection: Collection
 
 
+def _lookup_result_expected(
+    result: LookupResult, expect_type: LookupChildType
+) -> bool:
+    """Check whether a lookup result is of the expected type."""
+    if expect_type != LookupChildType.ANY:
+        expected_result_types = {
+            LookupChildType.BARE: [CollectionItem.Types.BARE],
+            LookupChildType.ARTIFACT: [CollectionItem.Types.ARTIFACT],
+            LookupChildType.ARTIFACT_OR_PROMISE: [
+                CollectionItem.Types.ARTIFACT,
+                CollectionItem.Types.BARE,
+            ],
+            LookupChildType.COLLECTION: [CollectionItem.Types.COLLECTION],
+        }[expect_type]
+        if result.result_type not in expected_result_types:
+            return False
+    return True
+
+
 def _normalize_lookup_segments(
     lookup: LookupSingle,
     default_category: CollectionCategory | None = None,
@@ -290,21 +309,11 @@ def lookup_single(
             collection=item.collection,
         )
 
-    if expect_type != LookupChildType.ANY:
-        expected_result_types = {
-            LookupChildType.BARE: [CollectionItem.Types.BARE],
-            LookupChildType.ARTIFACT: [CollectionItem.Types.ARTIFACT],
-            LookupChildType.ARTIFACT_OR_PROMISE: [
-                CollectionItem.Types.ARTIFACT,
-                CollectionItem.Types.BARE,
-            ],
-            LookupChildType.COLLECTION: [CollectionItem.Types.COLLECTION],
-        }[expect_type]
-        if result.result_type not in expected_result_types:
-            raise LookupError(
-                f"{lookup!r} is of type {result.result_type.name.lower()!r}"
-                f" instead of expected {expect_type.name.lower()!r}"
-            )
+    if not _lookup_result_expected(result, expect_type):
+        raise LookupError(
+            f"{lookup!r} is of type {result.result_type.name.lower()!r}"
+            f" instead of expected {expect_type.name.lower()!r}"
+        )
 
     return result
 
@@ -327,23 +336,6 @@ def _lookup_dict(
 
     See :ref:`lookup-dict`.
     """
-    if (
-        (expect_type == LookupChildType.ANY)
-        or (expect_type == lookup.child_type)
-        or (
-            expect_type == LookupChildType.ARTIFACT_OR_PROMISE
-            and lookup.child_type
-            in (LookupChildType.ARTIFACT, LookupChildType.BARE)
-        )
-    ):
-        # Valid request
-        pass
-    else:
-        raise LookupError(
-            f"Only lookups for type {expect_type.name.lower()!r} are allowed "
-            f"here"
-        )
-
     parent_collection_segments = _normalize_lookup_segments(
         lookup.collection,
         default_category=default_category,
@@ -411,7 +403,7 @@ def _lookup_dict(
     objects = objects.select_related("artifact", "collection")
 
     # Execute the query.
-    return [
+    results = [
         LookupResult(
             result_type=CollectionItem.Types(item.child_type),
             parent_collection_lookup=parent_collection_lookup,
@@ -421,6 +413,14 @@ def _lookup_dict(
         )
         for item in objects.order_by("id")
     ]
+    for result in results:
+        if not _lookup_result_expected(result, expect_type):
+            raise LookupError(
+                f"Result from {lookup.export()} is of type"
+                f" {result.result_type.name.lower()!r} instead of expected"
+                f" {expect_type.name.lower()!r}"
+            )
+    return results
 
 
 @overload
