@@ -15,16 +15,15 @@ from unittest import mock
 
 from debusine.artifacts.models import (
     BareDataCategory,
-    CollectionCategory,
     DebusineTaskConfiguration,
     TaskTypes,
 )
 from debusine.client.models import model_to_json_serializable_dict
 from debusine.db.models import Collection, CollectionItem
 from debusine.db.playground import scenarios
+from debusine.server.collections import DebusineTaskConfigurationManager
 from debusine.server.collections.base import ItemAdditionError
 from debusine.server.collections.debusine_task_configuration import (
-    DebusineTaskConfigurationManager,
     apply_configuration,
     build_configuration,
     list_configuration,
@@ -46,11 +45,7 @@ class TaskConfigTestCase(TestCase):
     def setUpTestData(cls) -> None:
         """Set up common test data."""
         super().setUpTestData()
-        cls.collection = cls.scenario.workspace.get_collection(
-            name="default",
-            category=CollectionCategory.TASK_CONFIGURATION,
-            user=cls.scenario.user,
-        )
+        cls.collection = cls.scenario.default_task_configuration_collection
         cls.manager = DebusineTaskConfigurationManager(
             collection=cls.collection
         )
@@ -214,6 +209,19 @@ class ConfigLookupTests(TaskConfigTestCase):
             entry,
         )
 
+    def test_lookup_config_by_name_with_deleted_entries(self) -> None:
+        entry = DebusineTaskConfiguration(
+            task_type=TaskTypes.WORKER, task_name="noop"
+        )
+        self.manager.remove_item(
+            self._add_config(entry), user=self.scenario.user
+        )
+        self._add_config(entry)
+        self.assertEqual(
+            lookup_config_by_name(self.collection, entry.name()),
+            entry,
+        )
+
     def test_lookup_config_by_name_missing(self) -> None:
         """Lookup returns None if the named item does not exist."""
         self.assertIsNone(
