@@ -613,9 +613,7 @@ class TaskHistoryManagerTests(TestCase):
 
     def test_lookup_filter_same_work_request_single(self) -> None:
         """`lookup_filter`: `same_work_request` with a single lookup."""
-        work_requests = [
-            self.playground.create_work_request() for _ in range(2)
-        ]
+        work_requests = [self.playground.create_worker_task() for _ in range(2)]
         upload, _ = self.playground.create_artifact(
             category=ArtifactCategory.UPLOAD, work_request=work_requests[0]
         )
@@ -672,9 +670,7 @@ class TaskHistoryManagerTests(TestCase):
 
     def test_lookup_filter_same_work_request_multiple(self) -> None:
         """`lookup_filter`: `same_work_request` with a multiple lookup."""
-        work_requests = [
-            self.playground.create_work_request() for _ in range(3)
-        ]
+        work_requests = [self.playground.create_worker_task() for _ in range(3)]
         uploads = [
             self.playground.create_artifact(
                 category=ArtifactCategory.UPLOAD, work_request=work_request
@@ -738,8 +734,10 @@ class TaskHistoryManagerTests(TestCase):
 
     def test_lookup_filter_same_workflow_single(self) -> None:
         """`lookup_filter`: `same_workflow` with a single lookup."""
-        work_requests = [self.workflow.create_child("noop") for _ in range(2)]
-        work_requests.append(self.playground.create_work_request())
+        work_requests = [
+            self.workflow.create_child_worker("noop") for _ in range(2)
+        ]
+        work_requests.append(self.playground.create_worker_task())
         upload, _ = self.playground.create_artifact(
             category=ArtifactCategory.UPLOAD, work_request=work_requests[0]
         )
@@ -791,12 +789,14 @@ class TaskHistoryManagerTests(TestCase):
 
     def test_lookup_filter_same_workflow_multiple(self) -> None:
         """`lookup_filter`: `same_workflow` with a multiple lookup."""
-        work_requests = [self.workflow.create_child("noop") for _ in range(2)]
+        work_requests = [
+            self.workflow.create_child_worker("noop") for _ in range(2)
+        ]
         other_workflow = self.playground.create_workflow()
         work_requests.extend(
-            [other_workflow.create_child("noop") for _ in range(2)]
+            [other_workflow.create_child_worker("noop") for _ in range(2)]
         )
-        work_requests.append(self.playground.create_work_request())
+        work_requests.append(self.playground.create_worker_task())
         uploads = [
             self.playground.create_artifact(
                 category=ArtifactCategory.UPLOAD, work_request=work_request
