@@ -13,8 +13,8 @@ import filecmp
 import logging
 import os
 import shutil
-import tempfile
 from collections.abc import Callable, Generator
+from contextlib import contextmanager
 from datetime import datetime, timezone
 from pathlib import Path, PurePath
 from typing import Self
@@ -27,6 +27,7 @@ from debusine.server.file_backend.interface import (
     FileBackendInterface,
 )
 from debusine.server.file_backend.models import LocalFileBackendConfiguration
+from debusine.utils import atomic_writer
 
 logger = logging.getLogger(__name__)
 
@@ -63,6 +64,11 @@ class LocalFileBackendEntry(
         """Return the path to a local copy of the file."""
         return self.backend._base_directory / self._identifier
 
+    @contextmanager
+    def get_temporary_local_path(self) -> Generator[Path]:
+        """Yield the path to a possibly-temporary local copy of the file."""
+        yield self.get_local_path()
+
     def get_url(self, content_type: str | None = None) -> None:  # noqa: U100
         """Return None: no remote URL for a file in LocalFileBackend."""
         return None
@@ -114,15 +120,13 @@ class LocalFileBackendEntry(
             destination_directory.relative_to(self.backend._base_directory),
         )
 
-        # To make it easy to identify files that were not finished
-        # copying: copy to a temp file + rename
-        temporary_file = tempfile.NamedTemporaryFile(
-            dir=destination_directory, suffix=".temp", delete=False
-        )
-        temporary_file.close()
-
-        shutil.copy(local_path, temporary_file.name)
-        os.rename(temporary_file.name, destination_file)
+        with (
+            local_path.open(mode="rb") as local_file,
+            atomic_writer(
+                destination_file, mode="wb", chmod=0o600
+            ) as storage_file,
+        ):
+            shutil.copyfileobj(local_file, storage_file)
 
         self._sync_file(destination_file)
 
