@@ -13,11 +13,13 @@ import argparse
 import sys
 from typing import Any, NoReturn
 
+from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.core.management import CommandError, CommandParser
 
-from debusine.db.models import Collection, DEFAULT_WORKSPACE_NAME, Workspace
+from debusine.db.models import Collection
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
+from debusine.server.management.management_utils import get_workspace
 
 
 class Command(DebusineBaseCommand):
@@ -30,7 +32,10 @@ class Command(DebusineBaseCommand):
         parser.add_argument("name", help="Name")
         parser.add_argument("category", help="Category")
         parser.add_argument(
-            "--workspace", help="Workspace", default=DEFAULT_WORKSPACE_NAME
+            "--workspace",
+            metavar="scope/name",
+            help="Workspace",
+            default=settings.DEBUSINE_DEFAULT_WORKSPACE,
         )
         parser.add_argument(
             "--data",
@@ -51,15 +56,9 @@ class Command(DebusineBaseCommand):
         """Create the collection."""
         name = options["name"]
         category = options["category"]
-        workspace_name = options["workspace"]
         data = self.parse_yaml_data(options["data"].read()) or {}
 
-        try:
-            workspace = Workspace.objects.get(name=workspace_name)
-        except Workspace.DoesNotExist:
-            raise CommandError(
-                f'Workspace "{workspace_name}" not found', returncode=3
-            )
+        workspace = get_workspace(options["workspace"])
 
         try:
             collection = Collection(
