@@ -15,7 +15,7 @@ from django.core.management import CommandError, CommandParser
 
 from debusine.db.models import Token
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import Tokens
+from debusine.server.management.management_utils import Tokens
 
 
 class Command(DebusineBaseCommand):
@@ -44,7 +44,9 @@ class Command(DebusineBaseCommand):
     def handle(self, *args: Any, **options: Any) -> None:
         """Delete (if selected and confirmed) tokens."""
         tokens = Token.objects.get_tokens(
-            username=options['username'], key=options['token']
+            username=options["username"],
+            key=options["token"],
+            include_expired=True,
         )
 
         if not tokens:
