@@ -140,13 +140,13 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
 
     def test_create_updates_existing_signing_key(self) -> None:
         asset = self.playground.create_signing_key_asset(
-            purpose=KeyPurpose.OPENPGP, fingerprint="0123"
+            purpose=KeyPurpose.OPENPGP_UPLOAD, fingerprint="0123"
         )
         other_asset = self.playground.create_signing_key_asset(
-            purpose=KeyPurpose.OPENPGP, fingerprint="4567"
+            purpose=KeyPurpose.OPENPGP_UPLOAD, fingerprint="4567"
         )
         data = {
-            "purpose": KeyPurpose.OPENPGP,
+            "purpose": KeyPurpose.OPENPGP_UPLOAD,
             "fingerprint": "0123",
             "public_key": "a public key",
             "description": "a description",
@@ -221,7 +221,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
                 stdin=io.StringIO(
                     yaml.safe_dump(
                         {
-                            "purpose": KeyPurpose.OPENPGP,
+                            "purpose": KeyPurpose.OPENPGP_UPLOAD,
                             "fingerprint": "0123",
                             "public_key": "a public key",
                             "description": "a description",
