@@ -124,14 +124,12 @@ class DeleteExpiredCommandTests(TestCase):
         artifact_id_3.created_at = timezone.now() - timedelta(days=2)
         artifact_id_3.save()
 
-        collection_1 = self.playground.create_collection(
-            "internal", CollectionCategory.WORKFLOW_INTERNAL
-        )
-        work_request_1 = self.playground.create_work_request(
-            created_at=timezone.now() - timedelta(days=365),
-            expiration_delay=timedelta(days=1),
-            internal_collection=collection_1,
-        )
+        work_request_1 = self.playground.create_workflow()
+        work_request_1.expiration_delay = timedelta(days=1)
+        work_request_1.created_at = timezone.now() - timedelta(days=365)
+        work_request_1.save()
+        collection_1 = work_request_1.internal_collection
+        assert collection_1 is not None
 
         create_time = timezone.now() - timedelta(days=30)
         with mock.patch("django.utils.timezone.now", return_value=create_time):
@@ -194,9 +192,12 @@ class DeleteExpiredCommandTests(TestCase):
         """Test delete_expired verbose: print information."""
         artifact_id_1 = Artifact.objects.all()[0]
         artifact_id_2 = Artifact.objects.all()[1]
-        collection_1 = self.playground.create_collection(
-            "internal", CollectionCategory.WORKFLOW_INTERNAL
-        )
+        work_request_1 = self.playground.create_workflow()
+        work_request_1.created_at = timezone.now() - timedelta(days=365)
+        work_request_1.expiration_delay = timedelta(days=1)
+        work_request_1.save()
+        collection_1 = work_request_1.internal_collection
+        assert collection_1 is not None
         collection_1.manager.add_artifact(
             artifact_id_1,
             user=self.playground.get_default_user(),
@@ -207,11 +208,6 @@ class DeleteExpiredCommandTests(TestCase):
             user=self.playground.get_default_user(),
             name="artifact2",
         )
-        work_request_1 = self.playground.create_work_request(
-            created_at=timezone.now() - timedelta(days=365),
-            expiration_delay=timedelta(days=1),
-            internal_collection=collection_1,
-        )
         create_time = timezone.now() - timedelta(days=30)
         with mock.patch("django.utils.timezone.now", return_value=create_time):
             self.playground.create_workspace(
@@ -1027,7 +1023,7 @@ class DeleteExpiredWorkRequestsTests(TestCase):
 
     def test_one_expired(self) -> None:
         """One expired work request."""
-        wr1 = self.playground.create_work_request(expired=True)
+        wr1 = self.playground.create_worker_task(expired=True)
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
@@ -1044,9 +1040,9 @@ class DeleteExpiredWorkRequestsTests(TestCase):
 
     def test_multiple_expired(self) -> None:
         """Multiple expired work requests."""
-        wr1 = self.playground.create_work_request(expired=True)
-        wr2 = self.playground.create_work_request(expired=True)
-        wr3 = self.playground.create_work_request(parent=wr2, expired=True)
+        wr1 = self.playground.create_worker_task(expired=True)
+        wr2 = self.playground.create_worker_task(expired=True)
+        wr3 = self.playground.create_worker_task(parent=wr2, expired=True)
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
