@@ -11,7 +11,8 @@
 from datetime import timedelta
 from typing import ClassVar
 
-from debusine.db.context import context
+from django.utils import timezone
+
 from debusine.db.models import Token, User
 from debusine.django.management.tests import call_command
 from debusine.server.management.commands.tests.utils import TabularOutputTests
@@ -25,7 +26,6 @@ class ListTokensCommandTests(TabularOutputTests, TestCase):
     token: ClassVar[Token]
 
     @classmethod
-    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up common data for tests."""
         super().setUpTestData()
@@ -42,8 +42,24 @@ class ListTokensCommandTests(TabularOutputTests, TestCase):
         self.assertEqual(output.col(0), [self.token.hash])
         self.assertEqual(output.col(1), [self.user.username])
         self.assertEqual(output.col(2), [self.token.created_at.isoformat()])
-        self.assertEqual(output.col(3), ["True"])
-        self.assertEqual(output.col(4), [self.token.comment])
+        self.assertEqual(output.col(3), ["-"])
+        self.assertEqual(output.col(4), ["True"])
+        self.assertEqual(output.col(5), [self.token.comment])
+
+    def test_list_tokens_expire_at(self) -> None:
+        """list_tokens prints `expire_at` if set."""
+        self.token.expire_at = timezone.now() + timedelta(hours=1)
+        self.token.save()
+
+        with self.assertPrintsTable() as output:
+            call_command('list_tokens')
+
+        self.assertEqual(output.col(0), [self.token.hash])
+        self.assertEqual(output.col(1), [self.user.username])
+        self.assertEqual(output.col(2), [self.token.created_at.isoformat()])
+        self.assertEqual(output.col(3), [self.token.expire_at.isoformat()])
+        self.assertEqual(output.col(4), ["True"])
+        self.assertEqual(output.col(5), [self.token.comment])
 
     def test_list_tokens_filtered_by_username(self) -> None:
         """list_tokens print the correct filtered owners."""
