@@ -14,7 +14,7 @@ import tempfile
 from collections.abc import Generator
 from contextlib import contextmanager
 from datetime import timedelta
-from io import BytesIO
+from io import BytesIO, StringIO
 from pathlib import Path
 from typing import Any, TypedDict
 from unittest import mock
@@ -40,6 +40,7 @@ from debusine.db.models import (
     FileUpload,
 )
 from debusine.django.management.tests import call_command
+from debusine.server.file_backend.interface import FileBackendEntryInterface
 from debusine.server.file_backend.s3 import S3FileBackend
 from debusine.server.management.commands.vacuum_storage import Command
 from debusine.test.django import (
@@ -2021,6 +2022,65 @@ class VacuumStorageCommandTests(VacuumStorageTestMixin, TestCase):
             files_and_directories["files"],
         )
 
+    def test_report_missing_files_stores_referenced_by_the_db(self) -> None:
+        """
+        Verify method does not report a missing file.
+
+        The possible missing file is added during the execution of
+        _report_missing_files_stores_referenced_by_the_db.
+        """
+        path_in_artifact = "README"
+        artifact, _ = self.playground.create_artifact(
+            paths=[path_in_artifact], create_files=True
+        )
+        local_path = self.get_local_file_path(artifact, path_in_artifact)
+
+        # Create a Memory store
+        memory_store = FileStore.objects.create(
+            name="memory",
+            backend=FileStore.BackendChoices.MEMORY,
+            configuration={"name": "memory"},
+        )
+        memory_backend = memory_store.get_backend_object()
+
+        initial_number_files = len(list(memory_backend.list_entries()))
+
+        def add_file_side_effect(
+            store: FileStore,
+        ) -> set[FileBackendEntryInterface[Any, Any]]:
+            if store == memory_store:
+                memory_backend.add_file(local_path)
+
+            return Command._entries_from_files_in_file_store_db(store)
+
+        stdout = StringIO()
+        stderr = StringIO()
+        cmd = Command(stdout=stdout, stderr=stderr, no_color=True)
+        cmd.verbosity = 2
+
+        with mock.patch.object(
+            cmd,
+            "_entries_from_files_in_file_store_db",
+            autospec=True,
+            side_effect=add_file_side_effect,
+        ):
+            ok = cmd._report_missing_files_stores_referenced_by_the_db()
+
+        # One file was added during the execution of
+        # Command._report_missing_files_store_referenced_by_the_db
+        self.assertEqual(
+            len(list(memory_backend.list_entries())), initial_number_files + 1
+        )
+
+        # No files were missing, even if a file was added half-way the operation
+        self.assertTrue(ok)
+
+        # Output does not indicate any file
+        self.assertEqual(
+            stdout.getvalue(), "Checking missing files from stores\n"
+        )
+        self.assertEqual(stderr.getvalue(), "")
+
     @classmethod
     def _set_modified_time_two_days_ago(cls, path: Path) -> None:
         two_days_ago = cls.two_days_ago.timestamp()
