@@ -20,11 +20,9 @@ from debusine.artifacts.models import (
     CollectionCategory,
     SINGLETON_COLLECTION_CATEGORIES,
 )
-from debusine.db.context import context
 from debusine.db.models import (
     ArtifactRelation,
     Scope,
-    WorkRequest,
     WorkflowTemplate,
     Workspace,
     default_workspace,
@@ -346,18 +344,22 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         self.assertEqual(data, expected_data[0:-1])
         # fmt: on
 
-    @context.disable_permission_checks()
     def create_workspace_for_delete(self) -> Workspace:
         """Create a test workspace."""
         scope = self.playground.get_or_create_scope("scope")
         workspace = self.playground.create_workspace(name="Test", scope=scope)
+        self.playground.create_group_role(
+            workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            [self.playground.get_default_user()],
+        )
         for architecture in ("amd64", "s390x"):
             self.playground.create_debian_environment(
                 codename="bookworm",
                 architecture=architecture,
                 workspace=workspace,
             )
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=workspace, task_name="noop"
         )
         collection = self.playground.create_collection(
@@ -433,16 +435,15 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             workspace=workspace,
         )
 
-        workflow = WorkRequest.objects.create_workflow(
-            template=sbuild_template,
-            data={
+        workflow = self.playground.create_workflow(
+            sbuild_template,
+            task_data={
                 "input": {
                     "source_artifact": udev.pk,
                 },
                 "target_distribution": "debian:bookworm",
                 "architectures": ["all", "amd64", "s390x"],
             },
-            created_by=self.playground.get_default_user(),
         )
         workflow.mark_running()
         workflow.save()
@@ -472,6 +473,36 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         with self.assertRaises(Workspace.DoesNotExist):
             Workspace.objects.get(name="Test")
 
+    def test_delete_workspace_missing(self) -> None:
+        """Try to delete a workspace that does not exist."""
+        self.playground.get_or_create_scope("scope")
+        with self.assertRaisesRegex(
+            CommandError, r"Workspace 'Test' not found in scope 'scope'$"
+        ) as exc:
+            call_command("workspace", "delete", "scope/Test", "--yes")
+        self.assertEqual(exc.exception.returncode, 3)
+
+        stdout, stderr, exit_code = call_command(
+            "workspace",
+            "delete",
+            "scope/Test",
+            "--force",
+        )
+        self.assertEqual(stdout, "")
+        self.assertEqual(stderr, "")
+        self.assertEqual(exit_code, 0)
+
+        stdout, stderr, exit_code = call_command(
+            "workspace",
+            "delete",
+            "scope/Test",
+            "--force",
+            "--yes",
+        )
+        self.assertEqual(stdout, "")
+        self.assertEqual(stderr, "")
+        self.assertEqual(exit_code, 0)
+
     def test_delete_workspace_playground_ui_scenario(self) -> None:
         """Delete a workspace with the playground UI scenario."""
         playground = Playground(default_workspace_name="Playground")
