@@ -21,9 +21,9 @@ from rich.table import Table
 class TableOutput:
     """Holder for assertPrintTable result."""
 
-    def __init__(self) -> None:
+    def __init__(self, table: Table | None = None) -> None:
         """Make space to hold a table."""
-        self.table: Table | None = None
+        self.table: Table | None = table
 
     def col(self, index: int) -> list[Any]:
         """Return a list of values for a table column."""
@@ -43,3 +43,15 @@ class TabularOutputTests(unittest.TestCase):
         rprint.assert_called_once()
         output.table = rprint.call_args.args[0]
         self.assertIsInstance(output.table, Table)
+
+    @contextlib.contextmanager
+    def assertPrintsTables(self) -> Generator[list[TableOutput], None, None]:
+        """Check that multiple rich.Tables are printed."""
+        output: list[TableOutput] = []
+        with mock.patch("rich.print") as rprint:
+            yield output
+        rprint.assert_called()
+        for call in rprint.call_args_list:
+            table = call.args[0]
+            self.assertIsInstance(table, Table)
+            output.append(TableOutput(table=table))
