@@ -264,10 +264,13 @@ class DebusineCollector(Collector):
         self, histogram: HistogramMetricFamily, labels: tuple[str, ...]
     ) -> None:
         """Count user activity into histogram."""
+        label_null_filters = {f"{label}__isnull": False for label in labels}
+
         for stat in (
             WorkRequest.objects.filter(
                 task_type=TaskTypes.WORKFLOW, parent__isnull=True
             )
+            .filter(**label_null_filters)
             .values(*labels)
             .annotate(**self._histogram_buckets())
         ):
