@@ -896,3 +896,23 @@ class WorkspaceChainSerializer(serializers.Serializer[Never]):
         child=WorkspaceChainItemField(),
         max_length=100,
     )
+
+
+class CollectionSerializer(
+    DebusineSerializerMixin, serializers.ModelSerializer[Collection]
+):
+    """Serializer of a Collection."""
+
+    validation_error_title = "Cannot deserialize collection"
+
+    name = serializers.CharField(required=False)
+
+    class Meta:
+        model = Collection
+        fields = [
+            'id',
+            'name',
+            'full_history_retention_period',
+            'metadata_only_retention_period',
+            'data',
+        ]
