@@ -139,6 +139,9 @@ class OIDCAuthenticationCallbackView(View):
             except Exception as exc:
                 return response_for_exception(request, exc)
 
-        return redirect(
-            getattr(settings, "SIGNON_DEFAULT_REDIRECT", "homepage:homepage")
-        )
+        if not (next_url := request.session.pop("sso_callback_next_url", None)):
+            next_url = getattr(
+                settings, "SIGNON_DEFAULT_REDIRECT", "homepage:homepage"
+            )
+
+        return redirect(next_url)
