@@ -17,7 +17,7 @@ import shutil
 import subprocess
 import tempfile
 from collections.abc import Generator, Sequence
-from contextlib import contextmanager
+from contextlib import ExitStack, contextmanager
 from dataclasses import dataclass, field
 from pathlib import Path, PurePath
 from typing import Generic, TypeVar
@@ -131,6 +131,32 @@ class APTMirror(
                 f"'{CollectionCategory.SUITE}' not found"
             )
 
+    @functools.cached_property
+    def parent_archive(self) -> Collection | None:
+        """
+        The parent `debian:archive` collection, if it is suitable.
+
+        If ``may_reuse_versions`` is False, then the mirroring task may
+        safely assume that packages with the same version are shared across
+        suites in the same archive.
+        """
+        # TODO: This is too strict: if add_source and add_binary compared
+        # file hashes from the parent archive with the expected hashes
+        # rather than assuming that packages of the same version from the
+        # parent archive will match, then it would be safe to share packages
+        # across suites even if may_reuse_versions=True.
+        if self.collection.data.get("may_reuse_versions", False):
+            return None
+        try:
+            archive = self.collection.parent_collections.get(
+                category=CollectionCategory.ARCHIVE
+            )
+        except Collection.DoesNotExist:
+            return None
+        if archive.data.get("may_reuse_versions", False):
+            return None
+        return archive
+
     def make_apt_environment(self, temp_path: Path) -> dict[str, str]:
         """Make a suitable process environment for running apt commands."""
         env = os.environ.copy()
@@ -319,29 +345,49 @@ class APTMirror(
         assert self.workspace is not None
         package = source["Package"]
         version = source["Version"]
-        (temp_download := temp_path / "download" / name).mkdir(parents=True)
-        try:
-            _run_and_log_errors(
-                [
-                    "apt-get",
-                    "--download-only",
-                    "--only-source",
-                    "source",
-                    f"{package}={version}",
-                ],
-                cwd=temp_download,
-                env=self.make_apt_environment(temp_path),
-            )
-            source_package = SourcePackage.create(
-                name=package,
-                version=version,
-                files=list(temp_download.iterdir()),
-            )
-            source_artifact = Artifact.objects.create_from_local_artifact(
-                source_package,
-                self.workspace,
-                created_by_work_request=self.work_request,
-            )
+
+        with ExitStack() as stack:
+            (temp_download := temp_path / "download" / name).mkdir(parents=True)
+            stack.callback(shutil.rmtree, temp_download)
+
+            if (
+                self.parent_archive is not None
+                and (
+                    source_from_archive := self.parent_archive.manager.lookup(
+                        f"source-version:{name}"
+                    )
+                )
+                is not None
+            ):
+                # The parent archive already has this package, so we don't
+                # need to download it again.
+                assert source_from_archive.artifact is not None
+                source_artifact = source_from_archive.artifact
+            else:
+                # We don't have this package anywhere usable, so we need to
+                # download it again.
+                _run_and_log_errors(
+                    [
+                        "apt-get",
+                        "--download-only",
+                        "--only-source",
+                        "source",
+                        f"{package}={version}",
+                    ],
+                    cwd=temp_download,
+                    env=self.make_apt_environment(temp_path),
+                )
+                source_package = SourcePackage.create(
+                    name=package,
+                    version=version,
+                    files=list(temp_download.iterdir()),
+                )
+                source_artifact = Artifact.objects.create_from_local_artifact(
+                    source_package,
+                    self.workspace,
+                    created_by_work_request=self.work_request,
+                )
+
             self.collection.manager.add_artifact(
                 source_artifact,
                 user=self.work_request.created_by,
@@ -350,8 +396,6 @@ class APTMirror(
                     "section": source["Section"],
                 },
             )
-        finally:
-            shutil.rmtree(temp_download)
 
     def update_sources(self, temp_path: Path, plan: Plan[Sources]) -> None:
         """
@@ -398,46 +442,73 @@ class APTMirror(
         package = binary["Package"]
         version = binary["Version"]
         architecture = binary["Architecture"]
-        (temp_download := temp_path / "download" / name).mkdir(parents=True)
-        try:
-            _run_and_log_errors(
-                ["apt-get", "download", f"{package}:{architecture}={version}"],
-                cwd=temp_download,
-                env=self.make_apt_environment(temp_path),
-            )
-            # The download should have resulted in exactly one file.
-            [file] = temp_download.iterdir()
-            # Rename this to the basename of the file on the mirror, if
-            # necessary.  This may differ from what "apt-get download" gives
-            # us in the case where the binary has an epoch.
-            expected_file = temp_download / PurePath(binary["Filename"]).name
-            if file != expected_file:
-                file.rename(expected_file)
-            binary_package = BinaryPackage.create(file=expected_file)
-            binary_artifact = Artifact.objects.create_from_local_artifact(
-                binary_package,
-                self.workspace,
-                created_by_work_request=self.work_request,
-            )
-            srcpkg_name = binary_package.data.srcpkg_name
-            srcpkg_version = binary_package.data.srcpkg_version
-            source_item = self.collection.manager.lookup(
-                f"source-version:{srcpkg_name}_{srcpkg_version}"
-            )
-            # Add a built-using relationship to the corresponding source if
-            # we can be confident of it.  Suites with
-            # may_reuse_versions=True may have source packages replaced, and
-            # in that case we can't be sure that the source package version
-            # in the binary package's metadata is enough to match it.
-            if source_item is not None and not self.collection.data.get(
-                "may_reuse_versions", False
-            ):
-                assert source_item.artifact is not None
-                ArtifactRelation.objects.create(
-                    artifact=binary_artifact,
-                    target=source_item.artifact,
-                    type=ArtifactRelation.Relations.BUILT_USING,
+
+        with ExitStack() as stack:
+            (temp_download := temp_path / "download" / name).mkdir(parents=True)
+            stack.callback(shutil.rmtree, temp_download)
+
+            if (
+                self.parent_archive is not None
+                and (
+                    binary_from_archive := self.parent_archive.manager.lookup(
+                        f"binary-version:{name}"
+                    )
                 )
+                is not None
+            ):
+                # The parent archive already has this package, so we don't
+                # need to download it again.
+                assert binary_from_archive.artifact is not None
+                binary_artifact = binary_from_archive.artifact
+            else:
+                # We don't have this package anywhere usable, so we need to
+                # download it again.
+                _run_and_log_errors(
+                    [
+                        "apt-get",
+                        "download",
+                        f"{package}:{architecture}={version}",
+                    ],
+                    cwd=temp_download,
+                    env=self.make_apt_environment(temp_path),
+                )
+                # The download should have resulted in exactly one file.
+                [file] = temp_download.iterdir()
+                # Rename this to the basename of the file on the mirror, if
+                # necessary.  This may differ from what "apt-get download"
+                # gives us in the case where the binary has an epoch.
+                expected_file = (
+                    temp_download / PurePath(binary["Filename"]).name
+                )
+                if file != expected_file:
+                    file.rename(expected_file)
+                binary_package = BinaryPackage.create(file=expected_file)
+                binary_artifact = Artifact.objects.create_from_local_artifact(
+                    binary_package,
+                    self.workspace,
+                    created_by_work_request=self.work_request,
+                )
+                srcpkg_name = binary_package.data.srcpkg_name
+                srcpkg_version = binary_package.data.srcpkg_version
+                source_item = self.collection.manager.lookup(
+                    f"source-version:{srcpkg_name}_{srcpkg_version}"
+                )
+                # Add a built-using relationship to the corresponding source
+                # if we can be confident of it.  Suites with
+                # may_reuse_versions=True may have source packages replaced,
+                # and in that case we can't be sure that the source package
+                # version in the binary package's metadata is enough to
+                # match it.
+                if source_item is not None and not self.collection.data.get(
+                    "may_reuse_versions", False
+                ):
+                    assert source_item.artifact is not None
+                    ArtifactRelation.objects.create(
+                        artifact=binary_artifact,
+                        target=source_item.artifact,
+                        type=ArtifactRelation.Relations.BUILT_USING,
+                    )
+
             self.collection.manager.add_artifact(
                 binary_artifact,
                 user=self.work_request.created_by,
@@ -447,8 +518,6 @@ class APTMirror(
                     "priority": binary["Priority"],
                 },
             )
-        finally:
-            shutil.rmtree(temp_download)
 
     def plan_binaries(self, temp_path: Path) -> Plan[Packages]:
         """Plan the update of all binary packages in the collection."""
