@@ -209,6 +209,20 @@ class CopyCollectionItems(
         assert self.work_request is not None
         assert self.workspace is not None
 
+        target_collection_lookups = {
+            copies.target_collection for copies in self.data.copies
+        }
+        target_collections = {
+            lookup: lookup_single(
+                lookup=lookup,
+                workspace=self.work_request.workspace,
+                user=self.work_request.created_by,
+                workflow_root=self.work_request.get_workflow_root(),
+                expect_type=LookupChildType.COLLECTION,
+            ).collection
+            for lookup in target_collection_lookups
+        }
+
         for copies in self.data.copies:
             source_items = lookup_multiple(
                 copies.source_items,
@@ -217,17 +231,10 @@ class CopyCollectionItems(
                 workflow_root=self.work_request.get_workflow_root(),
                 expect_type=LookupChildType.ANY,
             )
-            target_collection = lookup_single(
-                lookup=copies.target_collection,
-                workspace=self.work_request.workspace,
-                user=self.work_request.created_by,
-                workflow_root=self.work_request.get_workflow_root(),
-                expect_type=LookupChildType.COLLECTION,
-            ).collection
             for source_item in source_items:
                 self.copy_item(
                     source_item,
-                    target_collection,
+                    target_collections[copies.target_collection],
                     unembargo=copies.unembargo,
                     replace=copies.replace,
                     name_template=copies.name_template,
