@@ -13,6 +13,8 @@ Task to create a workspace for experiments.
 See docs/reference/devel-blueprints/experiment-workspaces.rst
 """
 
+from datetime import timedelta
+
 from debusine.db.context import context
 from debusine.db.models import Group, WorkflowTemplate, Workspace
 from debusine.server.tasks import BaseServerTask
@@ -87,7 +89,11 @@ class CreateExperimentWorkspace(
             workspace = Workspace.objects.create(
                 scope=base_workspace.scope,
                 name=experiment_workspace_name,
-                expiration_delay=self.data.expiration_delay,
+                expiration_delay=(
+                    None
+                    if self.data.expiration_delay is None
+                    else timedelta(days=self.data.expiration_delay)
+                ),
                 public=self.data.public,
             )
 
