@@ -28,7 +28,11 @@ from debian.deb822 import Deb822, Packages, Release, Sources
 from django.db import connection, connections
 from django_pglocks import advisory_lock
 
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianSourcePackage,
+)
 from debusine.db.locks import LockError, LockType
 from debusine.db.models import Artifact, ArtifactRelation, Collection, Workspace
 from debusine.db.tests.utils import _calculate_hash_from_data
@@ -403,6 +407,49 @@ class APTMirrorTests(TestCase):
         ):
             task.collection
 
+    def test_parent_archive(self) -> None:
+        """`parent_archive` returns the parent archive if it is suitable."""
+        user = self.playground.get_default_user()
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        bookworm = self.create_suite_collection("bookworm")
+        archive.manager.add_collection(bookworm, user=user)
+        task = self.create_apt_mirror_task("bookworm")
+        self.assertEqual(task.parent_archive, archive)
+
+    def test_parent_archive_suite_reuses_versions(self) -> None:
+        """`parent_archive` returns None if the suite reuses versions."""
+        user = self.playground.get_default_user()
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        bookworm = self.create_suite_collection("bookworm")
+        bookworm.data["may_reuse_versions"] = True
+        bookworm.save()
+        archive.manager.add_collection(bookworm, user=user)
+        task = self.create_apt_mirror_task("bookworm")
+        self.assertIsNone(task.parent_archive)
+
+    def test_parent_archive_no_parent_archive(self) -> None:
+        """`parent_archive` returns None if there is no parent archive."""
+        self.create_suite_collection("bookworm")
+        task = self.create_apt_mirror_task("bookworm")
+        self.assertIsNone(task.parent_archive)
+
+    def test_parent_archive_reuses_versions(self) -> None:
+        """`parent_archive` returns None if the archive reuses versions."""
+        user = self.playground.get_default_user()
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        archive.data["may_reuse_versions"] = True
+        archive.save()
+        bookworm = self.create_suite_collection("bookworm")
+        archive.manager.add_collection(bookworm, user=user)
+        task = self.create_apt_mirror_task("bookworm")
+        self.assertIsNone(task.parent_archive)
+
     def test_fetch_indexes(self) -> None:
         """`fetch_indexes` sets up apt and calls "apt-get update"."""
         temp_path = self.create_temporary_directory()
@@ -616,7 +663,7 @@ class APTMirrorTests(TestCase):
         """`plan_sources` plans to replace sources in the collection."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -701,7 +748,7 @@ class APTMirrorTests(TestCase):
         """`plan_sources` plans to remove sources from the collection."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -782,7 +829,7 @@ class APTMirrorTests(TestCase):
         """`add_source` downloads and adds a source package."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -836,13 +883,91 @@ class APTMirrorTests(TestCase):
         self.assertEqual(item.data["component"], "main")
         self.assertEqual(item.data["section"], "devel")
 
+    def test_add_source_already_in_parent_archive(self) -> None:
+        """`add_source` reuses matching artifacts from the parent archive."""
+        temp_path = self.create_temporary_directory()
+        user = self.playground.get_default_user()
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        bookworm = self.create_suite_collection("bookworm")
+        trixie = self.create_suite_collection("trixie")
+        archive.manager.add_collection(bookworm, user=user)
+        archive.manager.add_collection(trixie, user=user)
+        work_request = self.playground.create_worker_task(
+            assign_contributor_role=True
+        )
+        task = self.create_apt_mirror_task("trixie")
+        task.set_work_request(work_request)
+
+        bookworm_artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            data=DebianSourcePackage(
+                name="hello", version="1.0", type="dpkg", dsc_fields={}
+            ),
+        )
+        bookworm.manager.add_artifact(
+            bookworm_artifact,
+            user=user,
+            variables={"component": "main", "section": "devel"},
+        )
+
+        sources_entries = [
+            self.write_sample_source_package(temp_path, "hello", version)
+            for version in ("1.0", "1.1")
+        ]
+
+        work_request.set_current()
+
+        # A source package that is already in the archive is not downloaded.
+        with self.patch_download({}) as mock_run:
+            task.add_source(
+                temp_path,
+                name="hello_1.0",
+                source=sources_entries[0],
+                component="main",
+            )
+        mock_run.assert_not_called()
+
+        # A source package that is not already in the archive is downloaded.
+        dsc_contents = (temp_path / "hello_1.1.dsc").read_bytes()
+        tar_contents = (temp_path / "hello_1.1.tar.xz").read_bytes()
+        with self.patch_download(
+            {
+                "hello=1.1": {
+                    "hello_1.1.dsc": dsc_contents,
+                    "hello_1.1.tar.xz": tar_contents,
+                }
+            }
+        ) as mock_run:
+            task.add_source(
+                temp_path,
+                name="hello_1.1",
+                source=sources_entries[1],
+                component="main",
+            )
+        self.assert_downloaded_source(mock_run, temp_path, "hello", "1.1")
+
+        trixie_item_1 = trixie.manager.lookup("source-version:hello_1.0")
+        assert trixie_item_1 is not None
+        self.assertEqual(trixie_item_1.artifact, bookworm_artifact)
+        self.assertEqual(trixie_item_1.created_by_user, work_request.created_by)
+        self.assertEqual(trixie_item_1.data["component"], "main")
+        self.assertEqual(trixie_item_1.data["section"], "devel")
+        trixie_item_2 = trixie.manager.lookup("source-version:hello_1.1")
+        assert trixie_item_2 is not None
+        self.assertNotEqual(trixie_item_2.artifact, bookworm_artifact)
+        self.assertEqual(trixie_item_2.created_by_user, work_request.created_by)
+        self.assertEqual(trixie_item_2.data["component"], "main")
+        self.assertEqual(trixie_item_2.data["section"], "devel")
+
     def test_update_sources(self) -> None:
         """`update_sources` executes a plan to update sources."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
         collection.data["may_reuse_versions"] = True
         collection.save()
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1020,7 +1145,7 @@ class APTMirrorTests(TestCase):
         """`plan_binaries` plans to replace binaries in the collection."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1101,7 +1226,7 @@ class APTMirrorTests(TestCase):
         """`plan_binaries` plans to remove binaries from the collection."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1232,7 +1357,7 @@ class APTMirrorTests(TestCase):
         """`add_binary` downloads and adds a binary package."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1300,7 +1425,7 @@ class APTMirrorTests(TestCase):
         """`add_binary` renames downloaded binary packages if necessary."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1367,7 +1492,7 @@ class APTMirrorTests(TestCase):
         """`add_binary` adds a relation to a matching source package."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1426,7 +1551,7 @@ class APTMirrorTests(TestCase):
         collection = self.create_suite_collection("bookworm")
         collection.data["may_reuse_versions"] = True
         collection.save()
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1473,13 +1598,112 @@ class APTMirrorTests(TestCase):
         assert binary_item.artifact is not None
         self.assertFalse(binary_item.artifact.relations.exists())
 
+    def test_add_binary_already_in_parent_archive(self) -> None:
+        """`add_binary` reuses matching artifacts from the parent archive."""
+        temp_path = self.create_temporary_directory()
+        user = self.playground.get_default_user()
+        archive = self.playground.create_singleton_collection(
+            CollectionCategory.ARCHIVE
+        )
+        bookworm = self.create_suite_collection("bookworm")
+        trixie = self.create_suite_collection("trixie")
+        archive.manager.add_collection(bookworm, user=user)
+        archive.manager.add_collection(trixie, user=user)
+        work_request = self.playground.create_worker_task(
+            assign_contributor_role=True
+        )
+        task = self.create_apt_mirror_task("trixie")
+        task.set_work_request(work_request)
+
+        bookworm_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello",
+                srcpkg_version="1:1.0-1",
+                package="libhello1",
+                version="1.0-1",
+                architecture="amd64",
+            )
+        )
+        bookworm.manager.add_artifact(
+            bookworm_artifact,
+            user=user,
+            variables={
+                "component": "main",
+                "section": "libs",
+                "priority": "optional",
+            },
+        )
+
+        packages_entries = [
+            self.write_sample_binary_package(
+                temp_path,
+                "libhello1",
+                version,
+                "amd64",
+                srcpkg_name="hello",
+                srcpkg_version=f"1:{version}",
+                section="libs",
+            )
+            for version in ("1.0-1", "1.0-2")
+        ]
+
+        work_request.set_current()
+
+        # A binary package that is already in the archive is not downloaded.
+        with self.patch_download({}) as mock_run:
+            task.add_binary(
+                temp_path,
+                name="libhello1_1.0-1_amd64",
+                binary=packages_entries[0],
+                component="main",
+            )
+        mock_run.assert_not_called()
+
+        # A binary package that is not already in the archive is downloaded.
+        deb_contents = (temp_path / "libhello1_1.0-2_amd64.deb").read_bytes()
+        with self.patch_download(
+            {
+                "libhello1:amd64=1.0-2": {
+                    "libhello1_1.0-2_amd64.deb": deb_contents
+                }
+            }
+        ) as mock_run:
+            task.add_binary(
+                temp_path,
+                name="libhello1_1.0-2_amd64",
+                binary=packages_entries[1],
+                component="main",
+            )
+        self.assert_downloaded_binary(
+            mock_run, temp_path, "libhello1", "1.0-2", "amd64"
+        )
+
+        trixie_item_1 = trixie.manager.lookup(
+            "binary-version:libhello1_1.0-1_amd64"
+        )
+        assert trixie_item_1 is not None
+        self.assertEqual(trixie_item_1.artifact, bookworm_artifact)
+        self.assertEqual(trixie_item_1.created_by_user, work_request.created_by)
+        self.assertEqual(trixie_item_1.data["component"], "main")
+        self.assertEqual(trixie_item_1.data["section"], "libs")
+        self.assertEqual(trixie_item_1.data["priority"], "optional")
+        trixie_item_2 = trixie.manager.lookup(
+            "binary-version:libhello1_1.0-2_amd64"
+        )
+        assert trixie_item_2 is not None
+        self.assertNotEqual(trixie_item_2.artifact, bookworm_artifact)
+        self.assertEqual(trixie_item_2.created_by_user, work_request.created_by)
+        self.assertEqual(trixie_item_2.data["component"], "main")
+        self.assertEqual(trixie_item_2.data["section"], "libs")
+        self.assertEqual(trixie_item_2.data["priority"], "optional")
+
     def test_update_binaries(self) -> None:
         """`update_binaries` executes a plan to update binaries."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
         collection.data["may_reuse_versions"] = True
         collection.save()
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1673,7 +1897,7 @@ class APTMirrorTests(TestCase):
         """``plan_indexes`` plans to replace indexes in the collection."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1756,7 +1980,7 @@ class APTMirrorTests(TestCase):
         """``plan_indexes`` plans to remove indexes from the collection."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
@@ -1818,7 +2042,7 @@ class APTMirrorTests(TestCase):
         """``add_index`` adds a repository index."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1857,7 +2081,7 @@ class APTMirrorTests(TestCase):
         """``update_indexes`` executes a plan to update repository indexes."""
         temp_path = self.create_temporary_directory()
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             assign_contributor_role=True
         )
         task = self.create_apt_mirror_task("bookworm")
@@ -1976,7 +2200,7 @@ class APTMirrorTests(TestCase):
             )
 
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.playground.create_group_role(
             work_request.workspace,
             Workspace.Roles.CONTRIBUTOR,
@@ -2097,7 +2321,7 @@ class APTMirrorTests(TestCase):
     def test_execute_lock_error(self) -> None:
         """The task fails if its lock is already held."""
         collection = self.create_suite_collection("bookworm")
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         task = self.create_apt_mirror_task("bookworm")
         task.set_work_request(work_request)
 
