@@ -83,7 +83,7 @@ class BaseServerTaskTests(TestCase):
 
     def test_set_work_request(self) -> None:
         """set_work_request sets appropriate attributes."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
 
         self.task.set_work_request(work_request)
 
@@ -97,7 +97,7 @@ class BaseServerTaskTests(TestCase):
 
     def test_execute_sets_up_context(self) -> None:
         """`execute` sets up a suitable context."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         def fake_execute() -> bool:
@@ -124,7 +124,7 @@ class BaseServerTaskTests(TestCase):
             file.write("log")
         assert self.task._debug_log_files_directory is not None
 
-        self.task.set_work_request(self.playground.create_work_request())
+        self.task.set_work_request(self.playground.create_worker_task())
         self.task._source_artifacts_ids = [
             artifact.id for artifact in source_artifacts or []
         ]
@@ -214,7 +214,7 @@ class BaseServerTaskTests(TestCase):
 
     def test_enforce(self) -> None:
         """Test enforcing permissions."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         work_request.set_current()
         self.task.set_work_request(work_request)
 
