@@ -18,7 +18,6 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
     CollectionCategory,
-    TaskTypes,
 )
 from debusine.client.models import LookupChildType
 from debusine.db.models import (
@@ -103,7 +102,7 @@ class CopyCollectionItemsTests(TestCase):
     ) -> CollectionItem:
         """Create an artifact collection item with a build log."""
         if work_request is None:
-            work_request = self.playground.create_work_request(
+            work_request = self.playground.create_worker_task(
                 workspace=self.source_workspace
             )
         artifact = self.playground.create_build_log_artifact(
@@ -245,7 +244,7 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact(self) -> None:
         """Copy an artifact item."""
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
@@ -309,7 +308,7 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_needs_unembargo(self) -> None:
         """Copying an artifact from private to public needs unembargo=True."""
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.create_build_log(work_request=work_request)
@@ -331,7 +330,7 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_with_unembargo(self) -> None:
         """Copying an artifact from private to public can unembargo it."""
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=self.source_workspace, assign_contributor_role=True
         )
         self.playground.create_group_role(
@@ -376,7 +375,7 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_variables(self) -> None:
         """Pass additional variables while copying an artifact."""
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
@@ -435,7 +434,7 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_incomplete_file(self) -> None:
         """We cannot copy artifacts with incomplete files."""
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
@@ -461,7 +460,7 @@ class CopyCollectionItemsTests(TestCase):
 
     def test_copy_item_artifact_not_in_any_file_store(self) -> None:
         """We cannot copy artifacts that are not in any file store."""
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
@@ -499,7 +498,7 @@ class CopyCollectionItemsTests(TestCase):
             configuration={"base_directory": settings.DEBUSINE_STORE_DIRECTORY},
         )
         self.source_scope.file_stores.set([file_store])
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             workspace=self.source_workspace
         )
         self.playground.create_group_role(
@@ -607,8 +606,7 @@ class CopyCollectionItemsTests(TestCase):
                 "category": ArtifactCategory.BINARY_PACKAGE,
             }
         )
-        copy_collection_items = workflow.create_child(
-            task_type=TaskTypes.SERVER,
+        copy_collection_items = workflow.create_child_server(
             task_name="copycollectionitems",
             task_data=CopyCollectionItemsData(
                 copies=[
@@ -645,8 +643,6 @@ class CopyCollectionItemsTests(TestCase):
         worker = self.playground.create_worker(worker_type=WorkerType.CELERY)
         copy_collection_items.assign_worker(worker)
         task = copy_collection_items.get_task()
-        assert isinstance(task, CopyCollectionItems)
-        task.set_work_request(copy_collection_items)
 
         self.assertTrue(task.execute())
 
