@@ -50,10 +50,8 @@ class CreateExperimentWorkspaceTests(TestCase):
 
     def _task(self, **kwargs: Any) -> CreateExperimentWorkspace:
         """Create a CreateExperimentWorkspace task."""
-        workspace = self.scenario.workspace
         data = CreateExperimentWorkspaceData(**kwargs)
-        work_request = self.playground.create_work_request(
-            workspace=workspace,
+        work_request = self.playground.create_server_task(
             task_name="create_experiment_workspace",
             task_data=data,
         )
