@@ -19,11 +19,7 @@ from typing import Any
 from django.db import connection, connections
 from django_pglocks import advisory_lock
 
-from debusine.artifacts.models import (
-    ArtifactCategory,
-    CollectionCategory,
-    TaskTypes,
-)
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.db.locks import LockError, LockType
 from debusine.db.models import Artifact, Collection, CollectionItem
 from debusine.db.playground import scenarios
@@ -66,8 +62,7 @@ class GenerateSuiteIndexesTests(TestCase):
         """Create an instance of the :py:class:`GenerateSuiteIndexes` task."""
         if generate_at is None:
             generate_at = datetime.now(timezone.utc)
-        work_request = self.playground.create_work_request(
-            task_type=TaskTypes.SERVER,
+        work_request = self.playground.create_server_task(
             task_name="generatesuiteindexes",
             task_data=GenerateSuiteIndexesData(
                 suite_collection=suite_collection, generate_at=generate_at
@@ -79,7 +74,6 @@ class GenerateSuiteIndexesTests(TestCase):
             )
         task = work_request.get_task()
         assert isinstance(task, GenerateSuiteIndexes)
-        task.set_work_request(work_request)
         return task
 
     def test_compute_dynamic_data(self) -> None:
@@ -796,7 +790,7 @@ class GenerateSuiteIndexesTests(TestCase):
             ),
             (datetime(2025, 4, 15, tzinfo=timezone.utc), None),
         ):
-            with self.subTest(generate_at=generate_at):
+            with self.subTest(generate_at=generate_at.isoformat()):
                 task = self.create_task(
                     suite_collection=suite_collection, generate_at=generate_at
                 )
