@@ -30,7 +30,13 @@ from debusine.server.tasks.update_derived_collection import (
     DerivedCollectionChange,
     DerivedCollectionChangeKind,
 )
-from debusine.tasks.models import LintianData, LintianInput, LookupMultiple
+from debusine.tasks.models import (
+    ActionUpdateCollectionWithArtifacts,
+    EventReactions,
+    LintianData,
+    LintianInput,
+    LookupMultiple,
+)
 from debusine.test.django import TestCase
 
 
@@ -52,7 +58,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         super().setUpTestData()
         cls.user = cls.playground.get_default_user()
         cls.workspace = cls.playground.get_default_workspace()
-        cls.work_request = cls.playground.create_work_request(
+        cls.work_request = cls.playground.create_worker_task(
             workspace=cls.workspace, created_by=cls.user
         )
         cls.base_collection = Collection.objects.create(
@@ -246,7 +252,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         source_package_item = self.create_source_package_item(
             self.base_manager, "a", "1.0"
         )
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         child_work_request = self.task.make_child_work_request(
@@ -272,13 +278,10 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         self.assert_work_request_event_reactions(
             child_work_request,
             on_success=[
-                {
-                    "action": "update-collection-with-artifacts",
-                    "artifact_filters": {"category": ArtifactCategory.LINTIAN},
-                    "collection": (
-                        f"bookworm@{CollectionCategory.SUITE_LINTIAN}"
-                    ),
-                }
+                ActionUpdateCollectionWithArtifacts(
+                    artifact_filters={"category": ArtifactCategory.LINTIAN},
+                    collection=f"bookworm@{CollectionCategory.SUITE_LINTIAN}",
+                )
             ],
         )
 
@@ -293,7 +296,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             )
             for package in ("liba1", "liba-dev")
         ]
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         child_work_request = self.task.make_child_work_request(
@@ -323,13 +326,10 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         self.assert_work_request_event_reactions(
             child_work_request,
             on_success=[
-                {
-                    "action": "update-collection-with-artifacts",
-                    "artifact_filters": {"category": ArtifactCategory.LINTIAN},
-                    "collection": (
-                        f"bookworm@{CollectionCategory.SUITE_LINTIAN}"
-                    ),
-                }
+                ActionUpdateCollectionWithArtifacts(
+                    artifact_filters={"category": ArtifactCategory.LINTIAN},
+                    collection=f"bookworm@{CollectionCategory.SUITE_LINTIAN}",
+                )
             ],
         )
 
@@ -341,7 +341,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             )
             for package in ("liba1", "liba-dev")
         ]
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         child_work_request = self.task.make_child_work_request(
@@ -368,19 +368,16 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         self.assert_work_request_event_reactions(
             child_work_request,
             on_success=[
-                {
-                    "action": "update-collection-with-artifacts",
-                    "artifact_filters": {"category": ArtifactCategory.LINTIAN},
-                    "collection": (
-                        f"bookworm@{CollectionCategory.SUITE_LINTIAN}"
-                    ),
-                }
+                ActionUpdateCollectionWithArtifacts(
+                    artifact_filters={"category": ArtifactCategory.LINTIAN},
+                    collection=(f"bookworm@{CollectionCategory.SUITE_LINTIAN}"),
+                )
             ],
         )
 
     def test_make_child_work_request_with_no_packages(self) -> None:
         """`make_child_work_request` raises ValueError: no packages."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         with self.assertRaisesRegex(
@@ -398,7 +395,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             self.create_source_package_item(self.base_manager, package, "1.0")
             for package in ("a", "b")
         ]
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         with self.assertRaisesRegex(
@@ -420,7 +417,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         source_package_item = self.create_source_package_item(
             self.base_manager, "a", "1.0"
         )
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
         child_task_data = {
             "environment": "debian/match:codename=bookworm:architecture=amd64"
@@ -460,9 +457,9 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         source_package_item = self.create_source_package_item(
             self.base_manager, "a", "1.0"
         )
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
-        self.playground.create_work_request(
+        self.playground.create_worker_task(
             task_name="lintian",
             task_data=LintianData(
                 input=LintianInput(
@@ -473,19 +470,16 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             workspace=self.workspace,
             created_by=work_request.created_by,
             parent=work_request,
-            event_reactions_json={
-                "on_success": [
-                    {
-                        "action": "update-collection-with-artifacts",
-                        "artifact_filters": {
-                            "category": ArtifactCategory.LINTIAN
-                        },
-                        "collection": (
+            event_reactions=EventReactions(
+                on_success=[
+                    ActionUpdateCollectionWithArtifacts(
+                        artifact_filters={"category": ArtifactCategory.LINTIAN},
+                        collection=(
                             f"bookworm@{CollectionCategory.SUITE_LINTIAN}"
                         ),
-                    }
+                    )
                 ]
-            },
+            ),
         )
 
         self.assertIsNone(
@@ -527,7 +521,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         self.task.data.child_task_data = {
             "environment": "debian/match:codename=bookworm:architecture=amd64"
         }
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         self.assertTrue(self.task.execute())
@@ -563,15 +557,12 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
             self.assert_work_request_event_reactions(
                 child_work_request,
                 on_success=[
-                    {
-                        "action": "update-collection-with-artifacts",
-                        "artifact_filters": {
-                            "category": ArtifactCategory.LINTIAN
-                        },
-                        "collection": (
+                    ActionUpdateCollectionWithArtifacts(
+                        artifact_filters={"category": ArtifactCategory.LINTIAN},
+                        collection=(
                             f"bookworm@{CollectionCategory.SUITE_LINTIAN}"
                         ),
-                    }
+                    )
                 ],
             )
         # The task removed items that were planned to be removed.
@@ -593,7 +584,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         self.task.data.base_collection = (
             f"bookworm@{CollectionCategory.ENVIRONMENTS}"
         )
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         with self.assertRaisesRegex(
@@ -613,7 +604,7 @@ class UpdateSuiteLintianCollectionTests(CollectionTestMixin, TestCase):
         self.task.data.derived_collection = (
             f"bookworm@{CollectionCategory.ENVIRONMENTS}"
         )
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         self.task.set_work_request(work_request)
 
         with self.assertRaisesRegex(
