@@ -46,12 +46,20 @@ class ExternalDebsignTaskTests(TestCase):
             ExternalDebsignDynamicData(unsigned_id=1),
         )
 
+    def test_get_input_artifacts_ids(self) -> None:
+        """Test get_input_artifacts_ids."""
+        task = ExternalDebsign(task_data={"unsigned": 1})
+        self.assertEqual(task.get_input_artifacts_ids(), [])
+
+        task.dynamic_data = ExternalDebsignDynamicData(unsigned_id=1)
+        self.assertEqual(task.get_input_artifacts_ids(), [1])
+
     def test_execute(self) -> None:
         """Executing the task does nothing, successfully."""
         task = ExternalDebsign(
             task_data={"unsigned": 1}, dynamic_task_data={"unsigned_id": 1}
         )
-        task.set_work_request(self.playground.create_work_request())
+        task.set_work_request(self.playground.create_worker_task())
         self.assertTrue(task.execute())
 
     def test_label(self) -> None:
