@@ -15,7 +15,6 @@ from unittest import mock
 
 from django.db import connections, transaction
 
-from debusine.artifacts.models import TaskTypes
 from debusine.db.models import WorkRequest, Worker
 from debusine.server.celery import (
     ServerTaskRunError,
@@ -65,15 +64,19 @@ class RunServerTaskTests(TestCase):
         super().setUp()
         self.worker = Worker.objects.get_or_create_celery()
 
-    def create_assigned_work_request(self, **kwargs: Any) -> WorkRequest:
+    def create_assigned_work_request(
+        self, task_name: str = "servernoop", **kwargs: Any
+    ) -> WorkRequest:
         """Create a work request and assign it to self.worker."""
-        work_request = self.playground.create_work_request(**kwargs)
+        work_request = self.playground.create_server_task(
+            task_name=task_name, **kwargs
+        )
         work_request.assign_worker(self.worker)
         return work_request
 
     def test_no_work_request(self) -> None:
         """The Celery task fails if the work request does not exist."""
-        work_request_id = self.playground.create_work_request().id + 1
+        work_request_id = self.playground.create_worker_task().id + 1
         with self.assertLogsContains(
             f"Work request {work_request_id} does not exist",
             logger="debusine.server.celery",
@@ -99,9 +102,7 @@ class RunServerTaskTests(TestCase):
 
     def test_bad_task_name(self) -> None:
         """The Celery task fails if the work request has a bad task name."""
-        work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER, task_name="servernoop"
-        )
+        work_request = self.create_assigned_work_request()
         work_request.task_name = "nonexistent"
         work_request.save()
         expected_message = "'nonexistent' is not a registered Server task_name"
@@ -137,9 +138,7 @@ class RunServerTaskTests(TestCase):
 
     def test_bad_task_data(self) -> None:
         """The Celery task fails if the work request has bad task data."""
-        work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER, task_name="servernoop"
-        )
+        work_request = self.create_assigned_work_request()
         work_request.task_data = {"nonexistent": True}
         work_request.configured_task_data = {"nonexistent": True}
         work_request.save()
@@ -180,8 +179,8 @@ class RunServerTaskTests(TestCase):
 
     def test_external_task(self) -> None:
         """The Celery task fails if asked to run an external task."""
-        work_request = self.create_assigned_work_request(
-            task_name="noop", task_data={"result": True}
+        work_request = self.playground.create_worker_task(
+            task_name="noop", task_data={"result": True}, worker=self.worker
         )
         expected_message = "Cannot run on a Celery worker"
         with (
@@ -217,8 +216,6 @@ class RunServerTaskTests(TestCase):
     def test_task_raises_exception(self) -> None:
         """The Celery task fails if the task raises an exception."""
         work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER,
-            task_name="servernoop",
             task_data=ServerNoopData(exception=True),
         )
         expected_message = "Execution failed: Client requested an exception"
@@ -257,9 +254,7 @@ class RunServerTaskTests(TestCase):
 
     def test_task_aborted(self) -> None:
         """If the task is aborted, the Celery task logs but does not fail."""
-        work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER, task_name="servernoop"
-        )
+        work_request = self.create_assigned_work_request()
         with (
             self.assertLogsContains(
                 "Task: servernoop has been aborted",
@@ -285,8 +280,6 @@ class RunServerTaskTests(TestCase):
     def test_task_returns_false(self) -> None:
         """If the task returns False, the Celery task records that."""
         work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER,
-            task_name="servernoop",
             task_data=ServerNoopData(result=False),
         )
         with transaction.atomic():
@@ -303,8 +296,6 @@ class RunServerTaskTests(TestCase):
     def test_task_returns_true(self) -> None:
         """If the task returns True, the Celery task records that."""
         work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER,
-            task_name="servernoop",
             task_data=ServerNoopData(result=True),
         )
         with transaction.atomic():
@@ -325,8 +316,7 @@ class RunServerTaskTransactionTests(TransactionTestCase):
     def test_task_not_managing_transactions(self) -> None:
         """An atomic block is used for tasks that do not manage transactions."""
         worker = Worker.objects.get_or_create_celery()
-        work_request = self.playground.create_work_request(
-            task_type=TaskTypes.SERVER,
+        work_request = self.playground.create_server_task(
             task_name="sampletasknotmanagingtransactions",
             task_data={},
         )
@@ -338,8 +328,7 @@ class RunServerTaskTransactionTests(TransactionTestCase):
     def test_task_managing_transactions(self) -> None:
         """An atomic block is not used for tasks that manage transactions."""
         worker = Worker.objects.get_or_create_celery()
-        work_request = self.playground.create_work_request(
-            task_type=TaskTypes.SERVER,
+        work_request = self.playground.create_server_task(
             task_name="sampletaskmanagingtransactions",
             task_data={},
         )
@@ -356,7 +345,7 @@ class UpdateWorkflowTransactionTests(TransactionTestCase):
         """The Celery task updates workflow properties."""
         with mock.patch("django.db.transaction.on_commit"):
             workflow = self.playground.create_workflow()
-            work_request = workflow.create_child("noop")
+            work_request = workflow.create_child_worker("noop")
             self.assertTrue(work_request.mark_aborted())
             self.assertTrue(work_request.workflows_need_update)
 
