@@ -262,7 +262,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         """Connect succeeds and consumer send expected two messages."""
         worker = await self.acreate_worker()
 
-        await database_sync_to_async(self.playground.create_work_request)(
+        await database_sync_to_async(self.playground.create_worker_task)(
             status=WorkRequest.Statuses.PENDING, worker=worker
         )
 
@@ -278,7 +278,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         """Connect succeeds and consumer send expected two messages."""
         worker = await self.acreate_worker()
 
-        await database_sync_to_async(self.playground.create_work_request)(
+        await database_sync_to_async(self.playground.create_worker_task)(
             status=WorkRequest.Statuses.RUNNING, worker=worker
         )
 
@@ -339,7 +339,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         communicator, worker = await self.connect_to_new_worker()
 
         work_request: WorkRequest = await database_sync_to_async(
-            self.playground.create_work_request
+            self.playground.create_worker_task
         )()
 
         await database_sync_to_async(work_request.assign_worker)(worker)
@@ -473,7 +473,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         communicator, worker = await self.connect_to_new_worker()
 
         def assign_work_request(worker: Worker) -> None:
-            work_request = self.playground.create_work_request()
+            work_request = self.playground.create_worker_task()
             work_request.assign_worker(worker)
 
         await database_sync_to_async(assign_work_request)(worker)
@@ -659,14 +659,14 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         now = timezone.now()
 
         work_request_notified = await database_sync_to_async(
-            self.playground.create_work_request
+            self.playground.create_worker_task
         )(assign_new_worker=True, mark_running=True, result=result)
 
         work_request_notified.completed_at = now - timedelta(minutes=1)
         await work_request_notified.asave()
 
         work_request_to_notify = await database_sync_to_async(
-            self.playground.create_work_request
+            self.playground.create_worker_task
         )(
             assign_new_worker=True,
             mark_running=True,
@@ -708,7 +708,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         now = timezone.now()
 
-        await database_sync_to_async(self.playground.create_work_request)(
+        await database_sync_to_async(self.playground.create_worker_task)(
             assign_new_worker=True,
             mark_running=True,
             result=result,
@@ -740,14 +740,14 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         now = timezone.now()
 
         work_request_last_created_1 = await database_sync_to_async(
-            self.playground.create_work_request
+            self.playground.create_worker_task
         )(assign_new_worker=True, mark_running=True, result=result)
 
         work_request_last_created_1.completed_at = now
         await work_request_last_created_1.asave()
 
         work_request_last_created_2 = await database_sync_to_async(
-            self.playground.create_work_request
+            self.playground.create_worker_task
         )(
             assign_new_worker=True,
             mark_running=True,
@@ -792,10 +792,10 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def create_work_request(self, workspace: Workspace) -> WorkRequest:
+    async def create_worker_task(self, workspace: Workspace) -> WorkRequest:
         """Create a work request."""
         work_request = await database_sync_to_async(
-            self.playground.create_work_request
+            self.playground.create_worker_task
         )(
             workspace=workspace,
             assign_new_worker=True,
@@ -822,7 +822,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         # the work request
         self.assertTrue(await communicator.receive_nothing())
 
-        work_request = await self.create_work_request(workspace)
+        work_request = await self.create_worker_task(workspace)
         assert work_request.completed_at is not None
 
         actual = await communicator.receive_json_from()
@@ -863,7 +863,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         result = WorkRequest.Results.SUCCESS
 
-        await database_sync_to_async(self.playground.create_work_request)(
+        await database_sync_to_async(self.playground.create_worker_task)(
             workspace=workspace,
             assign_new_worker=True,
             mark_running=True,
@@ -883,7 +883,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
             self.playground.create_workspace
         )()
 
-        work_request = await self.create_work_request(workspace)
+        work_request = await self.create_worker_task(workspace)
         assert work_request.completed_at is not None
 
         actual = await communicator.receive_json_from()
