@@ -23,6 +23,7 @@ from debusine.assets.models import AssetCategory
 from debusine.db.models import (
     Artifact,
     Identity,
+    User,
     WorkRequest,
     WorkerPoolStatistics,
 )
@@ -112,6 +113,14 @@ class DebusineCollectorTests(TestCase):
         with self.assertRaises(KeyError):
             self.sample_by_label(metric, labels)
 
+    def assertNoNullLabels(self, metric: Metric) -> None:
+        """Assert that all metric labels have string values."""
+        for sample in metric.samples:
+            for key in sample.labels:
+                self.assertIsNotNone(
+                    sample.labels[key], msg=f"Null value for label '{key}'"
+                )
+
     def test_count_by_category(self) -> None:
         self.playground.create_artifact()
 
@@ -253,11 +262,16 @@ class DebusineCollectorTests(TestCase):
             ("user2", None, (4, 5)),
             ("user3", "salsa", (364,)),
             ("user5", "salsa", ()),
+            (None, "salsa", (45,)),
         ):
-            user = self.playground.create_user(username)
+            user: User | None = None
+            subject = "ex-user"
+            if username:
+                user = self.playground.create_user(username)
+                subject = username
             if issuer:
                 Identity.objects.create(
-                    user=user, issuer=issuer, subject=username, claims={}
+                    user=user, issuer=issuer, subject=subject, claims={}
                 )
             for days in ages:
                 wr = self.playground.create_workflow(created_by=user)
@@ -279,6 +293,7 @@ class DebusineCollectorTests(TestCase):
             gauge, 2, scope=scope, issuer="salsa", le="+Inf"
         )
         self.assertNoMetric(gauge, scope=scope, issuer="", le="1")
+        self.assertNoNullLabels(gauge)
 
     @override_settings(SIGNON_PROVIDERS=[Provider(name="salsa", label="Salsa")])
     def test_count_user_identities(self) -> None:
@@ -295,22 +310,23 @@ class DebusineCollectorTests(TestCase):
         self.assertMetricEquals(gauge, 0, active="0", issuer="salsa")
 
     def test_count_work_requests(self) -> None:
-        self.playground.create_work_request(
+        self.playground.create_worker_task(
             task_name="noop",
             status=WorkRequest.Statuses.PENDING,
         )
-        self.playground.create_work_request(
+        self.playground.create_signing_task(
             task_name="noop",
             status=WorkRequest.Statuses.RUNNING,
-            task_type=TaskTypes.SIGNING,
         )
-        self.playground.create_work_request(
+        # Disable validation to allow setting an arbitrary task data
+        self.playground.create_worker_task(
             task_name="noop",
             task_data={
                 "host_architecture": "amd64",
                 "backend": BackendType.UNSHARE,
             },
             status=WorkRequest.Statuses.RUNNING,
+            validate=False,
         )
         gauge = self.collector().count_work_requests()
         self.assertEqual(gauge.name, "work_requests")
@@ -355,6 +371,7 @@ class DebusineCollectorTests(TestCase):
             host_architecture="amd64",
             backend=BackendType.QEMU,
         )
+        self.assertNoNullLabels(gauge)
 
     def test_count_workers(self) -> None:
         self.playground.create_worker(
@@ -381,7 +398,7 @@ class DebusineCollectorTests(TestCase):
             worker_type=WorkerType.SIGNING,
         )
         worker3.mark_connected()
-        self.playground.create_work_request(
+        self.playground.create_worker_task(
             task_name="noop",
             status=WorkRequest.Statuses.RUNNING,
             worker=worker3,
@@ -449,6 +466,7 @@ class DebusineCollectorTests(TestCase):
             architecture_i386="0",
             architecture_x32="0",
         )
+        self.assertNoNullLabels(gauge)
 
     @freeze_time(time.time())
     def test_count_worker_pool_runtime(self) -> None:
