@@ -588,7 +588,7 @@ class ScopeWorkerAllocatorTests(TestCase):
             scope=self.playground.get_or_create_scope(name="other-scope")
         )
         work_requests = [
-            self.playground.create_work_request(workspace=workspace)
+            self.playground.create_worker_task(workspace=workspace)
             for workspace in (
                 self.scenario.workspace,
                 self.scenario.workspace,
