@@ -16,15 +16,20 @@ from datetime import timedelta
 from typing import Any, TYPE_CHECKING
 from unittest import mock
 
+from celery import states
+from celery.contrib.testing.app import TestApp, setup_default_app
 from channels.db import database_sync_to_async
+from django.db import transaction
+from django.db.models import JSONField, QuerySet
+from django.db.models.functions import Cast
 from django.test import override_settings
 from django.utils import timezone
+from django_celery_results.models import TaskResult
 
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianSourcePackage,
-    TaskTypes,
 )
 from debusine.assets import KeyPurpose
 from debusine.db.models import (
@@ -33,9 +38,10 @@ from debusine.db.models import (
     Token,
     WorkRequest,
     Worker,
-    default_workspace,
 )
+from debusine.db.playground import scenarios
 from debusine.db.tests.utils import RunInParallelTransaction
+from debusine.project.celery import TaskWithBetterArgumentRepresentation
 from debusine.server.scheduler import (
     _work_request_changed,
     _worker_changed,
@@ -58,6 +64,9 @@ from debusine.tasks.models import (
     LookupSingle,
     OutputData,
     OutputDataError,
+    SbuildBuildComponent,
+    SbuildData,
+    SbuildInput,
     WorkerType,
 )
 from debusine.tasks.server import ArtifactInfo
@@ -88,17 +97,16 @@ class FakeTaskDatabase(FakeTaskDatabaseBase, TaskDatabase):
 class SchedulerTestCase(PlaygroundTestCase):
     """Additional methods to help test scheduler related functions."""
 
-    SAMPLE_TASK_DATA = {
-        "input": {
-            "source_artifact": 10,
-        },
-        "environment": "debian/match:codename=sid",
-        "host_architecture": "amd64",
-        "build_components": [
-            "any",
-            "all",
+    SAMPLE_TASK_DATA = SbuildData(
+        input=SbuildInput(source_artifact=10),
+        environment="debian/match:codename=sid",
+        host_architecture="amd64",
+        build_components=[
+            SbuildBuildComponent("any"),
+            SbuildBuildComponent.ANY,
+            SbuildBuildComponent.ALL,
         ],
-    }
+    )
 
     single_lookups: dict[
         tuple[LookupSingle, CollectionCategory | None], ArtifactInfo
@@ -166,13 +174,11 @@ class SchedulerTestCase(PlaygroundTestCase):
         self, assign_to: Worker | None = None
     ) -> WorkRequest:
         """Create and return a WorkRequest."""
-        work_request = self.playground.create_work_request(task_name='sbuild')
-        work_request.task_data = dict(self.SAMPLE_TASK_DATA)
-        work_request.save()
-
+        work_request = self.playground.create_worker_task(
+            task_name='sbuild', task_data=self.SAMPLE_TASK_DATA
+        )
         if assign_to:
             work_request.assign_worker(assign_to)
-
         return work_request
 
     def set_tasks_allowlist(self, allowlist: list[str]) -> None:
@@ -208,16 +214,12 @@ class SchedulerTests(SchedulerTestCase, TestCase):
     def test_only_unhandled_work_requests_available(self) -> None:
         """schedule() returns nothing when no work request is available."""
         self._create_worker(enabled=True)
-        user = self.playground.get_default_user()
         WorkRequest.objects.all().delete()
 
-        WorkRequest.objects.create(
-            workspace=default_workspace(),
-            created_by=user,
+        self.playground.create_internal_task(
             status=WorkRequest.Statuses.PENDING,
-            task_type=TaskTypes.INTERNAL,
             task_name="unhandled",
-            task_data={},
+            validate=False,
         )
 
         self.assertEqual(schedule(), [])
@@ -296,10 +298,9 @@ class SchedulerTests(SchedulerTestCase, TestCase):
         parent = self.playground.create_workflow(
             status=WorkRequest.Statuses.RUNNING
         )
-        return parent.create_child(
+        return parent.create_child_wait(
             task_name="delay",
             status=status,
-            task_type=TaskTypes.WAIT,
             task_data=DelayData(delay_until=timezone.now() + delay_offset),
             workflow_data=WorkRequestWorkflowData(needs_input=False),
         )
@@ -354,10 +355,9 @@ class SchedulerTests(SchedulerTestCase, TestCase):
         parent = self.playground.create_workflow(
             status=WorkRequest.Statuses.RUNNING
         )
-        work_request = parent.create_child(
+        work_request = parent.create_child_wait(
             task_name="waitnoop",
             status=WorkRequest.Statuses.PENDING,
-            task_type=TaskTypes.WAIT,
             workflow_data=WorkRequestWorkflowData(needs_input=False),
         )
 
@@ -371,6 +371,8 @@ class SchedulerTests(SchedulerTestCase, TestCase):
 class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
     """Test schedule_for_worker()."""
 
+    scenario = scenarios.DefaultContext()
+
     def setUp(self) -> None:
         """Initialize test case."""
         super().setUp()
@@ -577,7 +579,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
         )
         # Lintian tasks don't care what architecture they run on, so they
         # defer to the worker's host architecture.
-        work_request = self.playground.create_work_request(
+        work_request = self.playground.create_worker_task(
             task_name="lintian",
             task_data=LintianData(
                 input=LintianInput(source_artifact=1),
@@ -605,6 +607,9 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
                 data=create_system_tarball_data(),
             ),
         }
+        task_config_collection = (
+            self.scenario.default_task_configuration_collection
+        )
 
         result = schedule_for_worker(self.worker)
 
@@ -620,6 +625,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
                 "subject": "hello",
                 "runtime_context": "binary-all+binary-any+source:sid",
                 "configuration_context": "sid",
+                "task_configuration_id": task_config_collection.pk,
             },
         )
 
@@ -692,8 +698,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
             enabled=True, worker_type=WorkerType.SIGNING
         )
         with override_settings(DISABLE_AUTOMATIC_SCHEDULING=True):
-            work_request = self.playground.create_work_request(
-                task_type=TaskTypes.SIGNING,
+            work_request = self.playground.create_signing_task(
                 task_name="generatekey",
                 task_data=GenerateKeyData(
                     purpose=KeyPurpose.UEFI, description="A UEFI key"
@@ -745,13 +750,13 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
         }
 
         for _ in range(3):
-            work_request = self.playground.create_work_request(
-                task_type=TaskTypes.SERVER, task_name="servernoop"
+            work_request = self.playground.create_server_task(
+                task_name="servernoop"
             )
             self.assertEqual(work_request, schedule_for_worker(worker))
 
-        work_request = self.playground.create_work_request(
-            task_type=TaskTypes.SERVER, task_name="servernoop"
+        work_request = self.playground.create_server_task(
+            task_name="servernoop"
         )
         self.assertIsNone(schedule_for_worker(worker))
 
@@ -909,6 +914,17 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             r"[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$",
         )
 
+    def find_workflow_task_results(self) -> QuerySet[TaskResult]:
+        task_results = (
+            TaskResult.objects.filter(
+                task_name="debusine.server.workflows.celery.run_workflow_task"
+            )
+            .annotate(task_args_array=Cast("task_args", JSONField()))
+            .order_by("date_created")
+        )
+        assert isinstance(task_results, QuerySet)
+        return task_results
+
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
     def test_server_task(self) -> None:
         """schedule_for_worker() dispatches server tasks to Celery."""
@@ -917,8 +933,8 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             "system:worker_type": WorkerType.CELERY,
             "server:servernoop:version": ServerNoop.TASK_VERSION,
         }
-        work_request = self.playground.create_work_request(
-            task_type=TaskTypes.SERVER, task_name="servernoop"
+        work_request = self.playground.create_server_task(
+            task_name="servernoop"
         )
 
         with mock.patch(
@@ -953,6 +969,13 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
 
     def test_workflow_callback_another_workflow_running(self) -> None:
         """Callbacks for multiple workflows may run concurrently."""
+        test_app = TestApp(
+            set_as_current=True,
+            broker="memory://",
+            task_cls=TaskWithBetterArgumentRepresentation,
+        )
+        self.enterContext(setup_default_app(test_app))
+
         WorkRequest.objects.all().delete()
         workflows = [self.playground.create_workflow() for _ in range(3)]
         for workflow in workflows:
@@ -964,17 +987,14 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             for workflow in workflows[:2]
         ]
 
-        with mock.patch(
-            "debusine.server.scheduler.run_workflow_task.apply_async"
-        ) as mock_apply_async:
+        with transaction.atomic():
             self.assertEqual(schedule(), wrs)
 
-        self.assertEqual(mock_apply_async.call_count, 2)
-        mock_apply_async.assert_has_calls(
-            [
-                mock.call(args=(wrs[0].id,), task_id=mock.ANY),
-                mock.call(args=(wrs[1].id,), task_id=mock.ANY),
-            ]
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[wr.id] for wr in wrs[:2]],
         )
         self.assertQuerySetEqual(
             WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
@@ -990,16 +1010,17 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
         )
         # A running work request in the same parent workflow that isn't a
         # workflow callback doesn't stop the workflow callback running.
-        self.playground.create_work_request(
+        self.playground.create_worker_task(
             parent=workflows[-1], assign_new_worker=True, mark_running=True
         )
 
-        with mock.patch(
-            "debusine.server.scheduler.run_workflow_task.apply_async"
-        ) as mock_apply_async:
+        with transaction.atomic():
             self.assertCountEqual(schedule(), [wrs[-1]])
-        mock_apply_async.assert_called_once_with(
-            args=(wrs[-1].id,), task_id=mock.ANY
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[wr.id] for wr in wrs],
         )
         self.assertQuerySetEqual(
             WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
@@ -1010,6 +1031,13 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
 
     def test_workflow_callback_same_workflow_running(self) -> None:
         """Callbacks for the same workflow may not run concurrently."""
+        test_app = TestApp(
+            set_as_current=True,
+            broker="memory://",
+            task_cls=TaskWithBetterArgumentRepresentation,
+        )
+        self.enterContext(setup_default_app(test_app))
+
         WorkRequest.objects.all().delete()
         workflow = self.playground.create_workflow()
         workflow.mark_running()
@@ -1020,13 +1048,14 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             for i in range(2)
         ]
 
-        with mock.patch(
-            "debusine.server.scheduler.run_workflow_task.apply_async"
-        ) as mock_apply_async:
+        with transaction.atomic():
             self.assertEqual(schedule(), [wrs[0]])
 
-        mock_apply_async.assert_called_once_with(
-            args=(wrs[0].id,), task_id=mock.ANY
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[wrs[0].id]],
         )
         self.assertQuerySetEqual(
             WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
@@ -1041,12 +1070,16 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             )
         )
 
-        with mock.patch(
-            "debusine.server.scheduler.run_workflow_task.apply_async"
-        ) as mock_apply_async:
+        with transaction.atomic():
             self.assertEqual(schedule(), [])
 
-        mock_apply_async.assert_not_called()
+        workflow_task_results = self.find_workflow_task_results().filter(
+            status=states.PENDING
+        )
+        self.assertQuerySetEqual(
+            workflow_task_results.values_list("task_args_array", flat=True),
+            [[wrs[0].id]],
+        )
         self.assertQuerySetEqual(
             WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
             .order_by("id")
@@ -1058,6 +1091,37 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             ],
         )
 
+        # Completing the Celery task (albeit artificially) allows the
+        # scheduler to proceed.
+        self.assertTrue(
+            WorkRequest.objects.get(id=wrs[0].id).mark_completed(
+                WorkRequest.Results.SUCCESS
+            )
+        )
+        workflow_task_result = workflow_task_results.get()
+        workflow_task_result.status = states.SUCCESS
+        workflow_task_result.save()
+
+        with transaction.atomic():
+            self.assertEqual(schedule(), [wrs[1]])
+
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[wrs[1].id]],
+        )
+        self.assertQuerySetEqual(
+            WorkRequest.objects.filter(id__in={wr.id for wr in wrs})
+            .order_by("id")
+            .values_list("status", flat=True),
+            [
+                WorkRequest.Statuses.COMPLETED,
+                WorkRequest.Statuses.RUNNING,
+                WorkRequest.Statuses.PENDING,
+            ],
+        )
+
     def test_workflows(self) -> None:
         """schedule() dispatches workflows to Celery."""
         WorkRequest.objects.all().delete()
@@ -1114,6 +1178,108 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
         self.assertEqual(root_wr.status, WorkRequest.Statuses.RUNNING)
         self.assertEqual(sub_wr.status, WorkRequest.Statuses.PENDING)
 
+    def test_sub_workflow_another_workflow_running(self) -> None:
+        """Sub-workflows of different root workflows may run concurrently."""
+        test_app = TestApp(
+            set_as_current=True,
+            broker="memory://",
+            task_cls=TaskWithBetterArgumentRepresentation,
+        )
+        self.enterContext(setup_default_app(test_app))
+
+        WorkRequest.objects.all().delete()
+        roots = [self.playground.create_workflow() for _ in range(3)]
+        # Mark the root workflows running, to simulate the situation where
+        # they can only do part of their work and have to be called back
+        # later.
+        for root in roots:
+            self.assertTrue(root.mark_running())
+        for root in roots[:2]:
+            child = self.playground.create_workflow(parent=root)
+            self.assertTrue(child.mark_running())
+            self.playground.create_workflow(parent=child)
+
+        with transaction.atomic():
+            self.assertEqual(schedule(), roots[:2])
+
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[root.id] for root in roots[:2]],
+        )
+
+        self.playground.create_workflow(parent=roots[-1])
+        # A running work request in the same parent workflow that isn't a
+        # sub-workflow doesn't stop the sub-workflow running.
+        self.playground.create_worker_task(
+            parent=roots[-1], assign_new_worker=True, mark_running=True
+        )
+
+        with transaction.atomic():
+            self.assertCountEqual(schedule(), [roots[-1]])
+
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[root.id] for root in roots],
+        )
+
+    def test_sub_workflow_same_workflow_running(self) -> None:
+        """Sub-workflows of the same root workflow may not run concurrently."""
+        test_app = TestApp(
+            set_as_current=True,
+            broker="memory://",
+            task_cls=TaskWithBetterArgumentRepresentation,
+        )
+        self.enterContext(setup_default_app(test_app))
+
+        WorkRequest.objects.all().delete()
+        root = self.playground.create_workflow()
+        root.mark_running()
+        for _ in range(2):
+            self.playground.create_workflow(parent=root)
+
+        with transaction.atomic():
+            self.assertEqual(schedule(), [root])
+
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[root.id]],
+        )
+
+        self.playground.create_workflow(parent=root)
+
+        with transaction.atomic():
+            self.assertEqual(schedule(), [])
+
+        workflow_task_results = self.find_workflow_task_results().filter(
+            status=states.PENDING
+        )
+        self.assertQuerySetEqual(
+            workflow_task_results.values_list("task_args_array", flat=True),
+            [[root.id]],
+        )
+
+        # Completing the Celery task (albeit artificially) allows the
+        # scheduler to proceed.
+        workflow_task_result = workflow_task_results.get()
+        workflow_task_result.status = states.SUCCESS
+        workflow_task_result.save()
+
+        with transaction.atomic():
+            self.assertEqual(schedule(), [root])
+
+        self.assertQuerySetEqual(
+            self.find_workflow_task_results()
+            .filter(status=states.PENDING)
+            .values_list("task_args_array", flat=True),
+            [[root.id]],
+        )
+
 
 def _method_has_kwargs(method: Callable[..., Any]) -> bool:
     """
