@@ -75,7 +75,7 @@ class WorkRequestSerializerTests(TestCase):
             'worker.lan', token=cls.playground.create_bare_token()
         )
 
-        cls.work_request = cls.playground.create_work_request(
+        cls.work_request = cls.playground.create_worker_task(
             started_at=datetime(2022, 2, 20, 15, 19, 1, 158424, timezone.utc),
             completed_at=datetime(2022, 2, 20, 16, 2, 3, 558425, timezone.utc),
             status=WorkRequest.Statuses.COMPLETED,
@@ -696,7 +696,7 @@ class AssetSerializerTests(TestCase):
 
     def test_valid_data_work_request(self) -> None:
         """Deserialize an asset with valid data, created by a work request."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
         serializer = AssetSerializer(
             data={
                 "category": AssetCategory.SIGNING_KEY,
@@ -705,7 +705,7 @@ class AssetSerializerTests(TestCase):
                     description="A Test Key",
                     fingerprint="ABC123",
                     public_key="PUBLIC KEY",
-                    purpose=KeyPurpose.OPENPGP,
+                    purpose=KeyPurpose.OPENPGP_UPLOAD,
                 ).dict(),
                 "work_request": work_request.id,
             },
@@ -727,7 +727,7 @@ class AssetSerializerTests(TestCase):
                     description="A Test Key",
                     fingerprint="ABC123",
                     public_key="PUBLIC KEY",
-                    purpose=KeyPurpose.OPENPGP,
+                    purpose=KeyPurpose.OPENPGP_UPLOAD,
                 ).dict(),
             },
         )
@@ -743,7 +743,7 @@ class LookupSingleSerializerTests(TestCase):
 
     def test_expect_type(self) -> None:
         """Test possible values of `expect_type`."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
 
         for expect_type, validated_expect_type in (
             # All the items in LookupChildType are valid.
@@ -775,7 +775,7 @@ class LookupMultipleSerializerTests(TestCase):
 
     def test_expect_type(self) -> None:
         """Test possible values of `expect_type`."""
-        work_request = self.playground.create_work_request()
+        work_request = self.playground.create_worker_task()
 
         for expect_type, validated_expect_type in (
             # All the items in LookupChildType are valid.
