@@ -18,7 +18,10 @@ from debusine.server.views.artifacts import (
 )
 from debusine.server.views.assets import AssetPermissionCheckView, AssetView
 from debusine.server.views.base import Whoami
-from debusine.server.views.collections import TaskConfigurationCollectionView
+from debusine.server.views.collections import (
+    CollectionView,
+    TaskConfigurationCollectionView,
+)
 from debusine.server.views.enroll import EnrollView
 from debusine.server.views.lookups import LookupMultipleView, LookupSingleView
 from debusine.server.views.open_metrics import OpenMetricsView
@@ -171,6 +174,11 @@ urlpatterns = [
         WorkspaceInheritanceView.as_view(),
         name="workspace-inheritance",
     ),
+    path(
+        "1.0/collection/<str:workspace>/<str:ccat>/<str:cname>/",
+        CollectionView.as_view(),
+        name="collection",
+    ),
     path(
         "1.0/open-metrics/",
         OpenMetricsView.as_view(),
